"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const solidJs = require("solid-js");
const core = require("@git-diff-view/core");
function reconcileArrays(parentNode, a, b) {
  let bLength = b.length, aEnd = a.length, bEnd = bLength, aStart = 0, bStart = 0, after = a[aEnd - 1].nextSibling, map = null;
  while (aStart < aEnd || bStart < bEnd) {
    if (a[aStart] === b[bStart]) {
      aStart++;
      bStart++;
      continue;
    }
    while (a[aEnd - 1] === b[bEnd - 1]) {
      aEnd--;
      bEnd--;
    }
    if (aEnd === aStart) {
      const node = bEnd < bLength ? bStart ? b[bStart - 1].nextSibling : b[bEnd - bStart] : after;
      while (bStart < bEnd) parentNode.insertBefore(b[bStart++], node);
    } else if (bEnd === bStart) {
      while (aStart < aEnd) {
        if (!map || !map.has(a[aStart])) a[aStart].remove();
        aStart++;
      }
    } else if (a[aStart] === b[bEnd - 1] && b[bStart] === a[aEnd - 1]) {
      const node = a[--aEnd].nextSibling;
      parentNode.insertBefore(b[bStart++], a[aStart++].nextSibling);
      parentNode.insertBefore(b[--bEnd], node);
      a[aEnd] = b[bEnd];
    } else {
      if (!map) {
        map = /* @__PURE__ */ new Map();
        let i = bStart;
        while (i < bEnd) map.set(b[i], i++);
      }
      const index = map.get(a[aStart]);
      if (index != null) {
        if (bStart < index && index < bEnd) {
          let i = aStart, sequence = 1, t;
          while (++i < aEnd && i < bEnd) {
            if ((t = map.get(a[i])) == null || t !== index + sequence) break;
            sequence++;
          }
          if (sequence > index - bStart) {
            const node = a[aStart];
            while (bStart < index) parentNode.insertBefore(b[bStart++], node);
          } else parentNode.replaceChild(b[bStart++], a[aStart++]);
        } else aStart++;
      } else a[aStart++].remove();
    }
  }
}
const $$EVENTS = "_$DX_DELEGATE";
function template(html, isImportNode, isSVG, isMathML) {
  let node;
  const create = () => {
    const t = document.createElement("template");
    t.innerHTML = html;
    return t.content.firstChild;
  };
  const fn = () => (node || (node = create())).cloneNode(true);
  fn.cloneNode = fn;
  return fn;
}
function delegateEvents(eventNames, document2 = window.document) {
  const e = document2[$$EVENTS] || (document2[$$EVENTS] = /* @__PURE__ */ new Set());
  for (let i = 0, l = eventNames.length; i < l; i++) {
    const name = eventNames[i];
    if (!e.has(name)) {
      e.add(name);
      document2.addEventListener(name, eventHandler);
    }
  }
}
function setAttribute(node, name, value) {
  if (isHydrating(node)) return;
  if (value == null) node.removeAttribute(name);
  else node.setAttribute(name, value);
}
function className(node, value) {
  if (isHydrating(node)) return;
  if (value == null) node.removeAttribute("class");
  else node.className = value;
}
function style(node, value, prev) {
  if (!value) return prev ? setAttribute(node, "style") : value;
  const nodeStyle = node.style;
  if (typeof value === "string") return nodeStyle.cssText = value;
  typeof prev === "string" && (nodeStyle.cssText = prev = void 0);
  prev || (prev = {});
  value || (value = {});
  let v, s;
  for (s in prev) {
    value[s] == null && nodeStyle.removeProperty(s);
    delete prev[s];
  }
  for (s in value) {
    v = value[s];
    if (v !== prev[s]) {
      nodeStyle.setProperty(s, v);
      prev[s] = v;
    }
  }
  return prev;
}
function use(fn, element, arg) {
  return solidJs.untrack(() => fn(element, arg));
}
function insert(parent, accessor, marker, initial) {
  if (marker !== void 0 && !initial) initial = [];
  if (typeof accessor !== "function") return insertExpression(parent, accessor, initial, marker);
  solidJs.createRenderEffect((current) => insertExpression(parent, accessor(), current, marker), initial);
}
function isHydrating(node) {
  return !!solidJs.sharedConfig.context && !solidJs.sharedConfig.done && (!node || node.isConnected);
}
function eventHandler(e) {
  if (solidJs.sharedConfig.registry && solidJs.sharedConfig.events) {
    if (solidJs.sharedConfig.events.find(([el, ev]) => ev === e)) return;
  }
  let node = e.target;
  const key = `$$${e.type}`;
  const oriTarget = e.target;
  const oriCurrentTarget = e.currentTarget;
  const retarget = (value) => Object.defineProperty(e, "target", {
    configurable: true,
    value
  });
  const handleNode = () => {
    const handler = node[key];
    if (handler && !node.disabled) {
      const data = node[`${key}Data`];
      data !== void 0 ? handler.call(node, data, e) : handler.call(node, e);
      if (e.cancelBubble) return;
    }
    node.host && typeof node.host !== "string" && !node.host._$host && node.contains(e.target) && retarget(node.host);
    return true;
  };
  const walkUpTree = () => {
    while (handleNode() && (node = node._$host || node.parentNode || node.host)) ;
  };
  Object.defineProperty(e, "currentTarget", {
    configurable: true,
    get() {
      return node || document;
    }
  });
  if (solidJs.sharedConfig.registry && !solidJs.sharedConfig.done) solidJs.sharedConfig.done = _$HY.done = true;
  if (e.composedPath) {
    const path = e.composedPath();
    retarget(path[0]);
    for (let i = 0; i < path.length - 2; i++) {
      node = path[i];
      if (!handleNode()) break;
      if (node._$host) {
        node = node._$host;
        walkUpTree();
        break;
      }
      if (node.parentNode === oriCurrentTarget) {
        break;
      }
    }
  } else walkUpTree();
  retarget(oriTarget);
}
function insertExpression(parent, value, current, marker, unwrapArray) {
  const hydrating = isHydrating(parent);
  if (hydrating) {
    !current && (current = [...parent.childNodes]);
    let cleaned = [];
    for (let i = 0; i < current.length; i++) {
      const node = current[i];
      if (node.nodeType === 8 && node.data.slice(0, 2) === "!$") node.remove();
      else cleaned.push(node);
    }
    current = cleaned;
  }
  while (typeof current === "function") current = current();
  if (value === current) return current;
  const t = typeof value, multi = marker !== void 0;
  parent = multi && current[0] && current[0].parentNode || parent;
  if (t === "string" || t === "number") {
    if (hydrating) return current;
    if (t === "number") {
      value = value.toString();
      if (value === current) return current;
    }
    if (multi) {
      let node = current[0];
      if (node && node.nodeType === 3) {
        node.data !== value && (node.data = value);
      } else node = document.createTextNode(value);
      current = cleanChildren(parent, current, marker, node);
    } else {
      if (current !== "" && typeof current === "string") {
        current = parent.firstChild.data = value;
      } else current = parent.textContent = value;
    }
  } else if (value == null || t === "boolean") {
    if (hydrating) return current;
    current = cleanChildren(parent, current, marker);
  } else if (t === "function") {
    solidJs.createRenderEffect(() => {
      let v = value();
      while (typeof v === "function") v = v();
      current = insertExpression(parent, v, current, marker);
    });
    return () => current;
  } else if (Array.isArray(value)) {
    const array = [];
    const currentArray = current && Array.isArray(current);
    if (normalizeIncomingArray(array, value, current, unwrapArray)) {
      solidJs.createRenderEffect(() => current = insertExpression(parent, array, current, marker, true));
      return () => current;
    }
    if (hydrating) {
      if (!array.length) return current;
      if (marker === void 0) return current = [...parent.childNodes];
      let node = array[0];
      if (node.parentNode !== parent) return current;
      const nodes = [node];
      while ((node = node.nextSibling) !== marker) nodes.push(node);
      return current = nodes;
    }
    if (array.length === 0) {
      current = cleanChildren(parent, current, marker);
      if (multi) return current;
    } else if (currentArray) {
      if (current.length === 0) {
        appendNodes(parent, array, marker);
      } else reconcileArrays(parent, current, array);
    } else {
      current && cleanChildren(parent);
      appendNodes(parent, array);
    }
    current = array;
  } else if (value.nodeType) {
    if (hydrating && value.parentNode) return current = multi ? [value] : value;
    if (Array.isArray(current)) {
      if (multi) return current = cleanChildren(parent, current, marker, value);
      cleanChildren(parent, current, null, value);
    } else if (current == null || current === "" || !parent.firstChild) {
      parent.appendChild(value);
    } else parent.replaceChild(value, parent.firstChild);
    current = value;
  } else ;
  return current;
}
function normalizeIncomingArray(normalized, array, current, unwrap) {
  let dynamic = false;
  for (let i = 0, len = array.length; i < len; i++) {
    let item = array[i], prev = current && current[normalized.length], t;
    if (item == null || item === true || item === false) ;
    else if ((t = typeof item) === "object" && item.nodeType) {
      normalized.push(item);
    } else if (Array.isArray(item)) {
      dynamic = normalizeIncomingArray(normalized, item, prev) || dynamic;
    } else if (t === "function") {
      if (unwrap) {
        while (typeof item === "function") item = item();
        dynamic = normalizeIncomingArray(
          normalized,
          Array.isArray(item) ? item : [item],
          Array.isArray(prev) ? prev : [prev]
        ) || dynamic;
      } else {
        normalized.push(item);
        dynamic = true;
      }
    } else {
      const value = String(item);
      if (prev && prev.nodeType === 3 && prev.data === value) normalized.push(prev);
      else normalized.push(document.createTextNode(value));
    }
  }
  return dynamic;
}
function appendNodes(parent, array, marker = null) {
  for (let i = 0, len = array.length; i < len; i++) parent.insertBefore(array[i], marker);
}
function cleanChildren(parent, current, marker, replacement) {
  if (marker === void 0) return parent.textContent = "";
  const node = replacement || document.createTextNode("");
  if (current.length) {
    let inserted = false;
    for (let i = current.length - 1; i >= 0; i--) {
      const el = current[i];
      if (node !== el) {
        const isParent = el.parentNode === parent;
        if (!inserted && !i)
          isParent ? parent.replaceChild(node, el) : parent.insertBefore(node, marker);
        else isParent && el.remove();
      } else inserted = true;
    }
  } else parent.insertBefore(node, marker);
  return [node];
}
function __classPrivateFieldGet(receiver, state, kind, f) {
  if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
  if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
  return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
}
function __classPrivateFieldSet(receiver, state, value, kind, f) {
  if (typeof state === "function" ? receiver !== state || true : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
  return state.set(receiver, value), value;
}
typeof SuppressedError === "function" ? SuppressedError : function(error, suppressed, message) {
  var e = new Error(message);
  return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
};
var _TextMeasure_instances, _TextMeasure_key, _TextMeasure_map, _TextMeasure_getInstance;
let canvasCtx = null;
const getKey = (font, text) => {
  return `${font.fontFamily}-${font.fontStyle}-${font.fontSize}-${text}`;
};
const getStableKey = (font, text) => {
  return getKey(font, "0".repeat(text.length));
};
class TextMeasure {
  constructor() {
    _TextMeasure_instances.add(this);
    _TextMeasure_key.set(this, "");
    _TextMeasure_map.set(this, {});
  }
  measure(text, font) {
    const currentKey = getStableKey(font, text);
    if (__classPrivateFieldGet(this, _TextMeasure_map, "f")[currentKey]) {
      return __classPrivateFieldGet(this, _TextMeasure_map, "f")[currentKey];
    }
    const instance2 = __classPrivateFieldGet(this, _TextMeasure_instances, "m", _TextMeasure_getInstance).call(this);
    if (font) {
      const currentFontKey = `${font.fontFamily}-${font.fontStyle}-${font.fontSize}`;
      if (__classPrivateFieldGet(this, _TextMeasure_key, "f") !== currentFontKey) {
        __classPrivateFieldSet(this, _TextMeasure_key, currentFontKey);
        instance2.font = `${font.fontStyle || ""} ${font.fontSize || ""} ${font.fontFamily || ""}`;
      }
    } else {
      instance2.font = "";
    }
    const textWidth = instance2.measureText(text).width;
    return textWidth;
  }
}
_TextMeasure_key = /* @__PURE__ */ new WeakMap(), _TextMeasure_map = /* @__PURE__ */ new WeakMap(), _TextMeasure_instances = /* @__PURE__ */ new WeakSet(), _TextMeasure_getInstance = function _TextMeasure_getInstance2() {
  canvasCtx = canvasCtx || document.createElement("canvas").getContext("2d");
  return canvasCtx;
};
let instance = null;
const getTextMeasureInstance = () => {
  instance = instance || new TextMeasure();
  return instance;
};
const addContentBGName = "--diff-add-content--";
const delContentBGName = "--diff-del-content--";
const borderColorName = "--diff-border--";
const addLineNumberBGName = "--diff-add-lineNumber--";
const delLineNumberBGName = "--diff-del-lineNumber--";
const plainContentBGName = "--diff-plain-content--";
const expandContentBGName = "--diff-expand-content--";
const plainLineNumberColorName = "--diff-plain-lineNumber-color--";
const expandLineNumberColorName = "--diff-expand-lineNumber-color--";
const plainLineNumberBGName = "--diff-plain-lineNumber--";
const expandLineNumberBGName = "--diff-expand-lineNumber--";
const hunkContentBGName = "--diff-hunk-content--";
const hunkContentColorName = "--diff-hunk-content-color--";
const hunkLineNumberBGName = "--diff-hunk-lineNumber--";
const addContentHighlightBGName = "--diff-add-content-highlight--";
const delContentHighlightBGName = "--diff-del-content-highlight--";
const addWidgetBGName = "--diff-add-widget--";
const addWidgetColorName = "--diff-add-widget-color--";
const emptyBGName = "--diff-empty-content--";
const getContentBG = (isAdded, isDelete, hasDiff) => {
  return isAdded ? `var(${addContentBGName})` : isDelete ? `var(${delContentBGName})` : hasDiff ? `var(${plainContentBGName})` : `var(${expandContentBGName})`;
};
const getLineNumberBG = (isAdded, isDelete, hasDiff) => {
  return isAdded ? `var(${addLineNumberBGName})` : isDelete ? `var(${delLineNumberBGName})` : hasDiff ? `var(${plainLineNumberBGName})` : `var(${expandLineNumberBGName})`;
};
const removeAllSelection = () => {
  const selection = window.getSelection();
  for (let i = 0; i < selection.rangeCount; i++) {
    selection.removeRange(selection.getRangeAt(i));
  }
};
const syncScroll = (left, right) => {
  const onScroll = function(event) {
    if (event === null || event.target === null)
      return;
    if (event.target === left) {
      right.scrollTop = left.scrollTop;
      right.scrollLeft = left.scrollLeft;
    } else {
      left.scrollTop = right.scrollTop;
      left.scrollLeft = right.scrollLeft;
    }
  };
  if (!left.onscroll) {
    left.onscroll = onScroll;
  }
  if (!right.onscroll) {
    right.onscroll = onScroll;
  }
  return () => {
    left.onscroll = null;
    right.onscroll = null;
  };
};
const diffFontSizeName = "--diff-font-size--";
const diffAsideWidthName = "--diff-aside-width--";
var NewLineSymbol;
(function(NewLineSymbol2) {
  NewLineSymbol2[NewLineSymbol2["CRLF"] = 1] = "CRLF";
  NewLineSymbol2[NewLineSymbol2["CR"] = 2] = "CR";
  NewLineSymbol2[NewLineSymbol2["LF"] = 3] = "LF";
  NewLineSymbol2[NewLineSymbol2["NEWLINE"] = 4] = "NEWLINE";
  NewLineSymbol2[NewLineSymbol2["NORMAL"] = 5] = "NORMAL";
  NewLineSymbol2[NewLineSymbol2["NULL"] = 6] = "NULL";
})(NewLineSymbol || (NewLineSymbol = {}));
const getSymbol = (symbol) => {
  switch (symbol) {
    case NewLineSymbol.LF:
      return "␊";
    case NewLineSymbol.CR:
      return "␍";
    case NewLineSymbol.CRLF:
      return "␍␊";
    default:
      return "";
  }
};
exports.DiffModeEnum = void 0;
(function(DiffModeEnum) {
  DiffModeEnum[DiffModeEnum["SplitGitHub"] = 1] = "SplitGitHub";
  DiffModeEnum[DiffModeEnum["SplitGitLab"] = 2] = "SplitGitLab";
  DiffModeEnum[DiffModeEnum["Split"] = 3] = "Split";
  DiffModeEnum[DiffModeEnum["Unified"] = 4] = "Unified";
})(exports.DiffModeEnum || (exports.DiffModeEnum = {}));
const DiffViewContext = solidJs.createContext();
const useDiffViewContext = () => solidJs.useContext(DiffViewContext);
const generateHook = (key) => {
  return () => {
    const reactiveHook = useDiffViewContext();
    const [state, setState] = solidJs.createSignal(reactiveHook == null ? void 0 : reactiveHook.getReadonlyState()[key]);
    solidJs.createEffect(() => {
      const init = () => setState(() => reactiveHook == null ? void 0 : reactiveHook.getReadonlyState()[key]);
      init();
      const unsubscribe = reactiveHook == null ? void 0 : reactiveHook.subscribe(
        (s) => s[key],
        () => init()
      );
      solidJs.onCleanup(() => unsubscribe == null ? void 0 : unsubscribe());
    });
    return state;
  };
};
const useEnableWrap = generateHook("enableWrap");
const useFontSize = generateHook("fontSize");
const useIsMounted = generateHook("isMounted");
const useTextWidth = ({
  text,
  font
}) => {
  var _a;
  const isMounted = useIsMounted();
  let baseSize = 6;
  const fontSize = parseInt(((_a = font == null ? void 0 : font()) == null ? void 0 : _a.fontSize) || "14");
  baseSize += fontSize > 10 ? (fontSize - 10) * 0.6 : 0;
  const [width, setWidth] = solidJs.createSignal(baseSize * text().length);
  const measureText = () => {
    const isMountedValue = isMounted();
    if (!isMountedValue) return;
    setWidth(getTextMeasureInstance().measure(text(), (font == null ? void 0 : font()) || {}));
  };
  solidJs.createEffect(measureText);
  return width;
};
const useMode = generateHook("mode");
const useId = generateHook("id");
const useDomWidth = ({ selector, enable }) => {
  const id = useId();
  const mounted = useIsMounted();
  const [width, setWidth] = solidJs.createSignal(0);
  const observeWidth = () => {
    if (!mounted()) return;
    if (enable()) {
      const container = document.querySelector(`#diff-root${id()}`);
      const wrapper = container == null ? void 0 : container.querySelector(selector());
      if (!wrapper) return;
      const typedWrapper = wrapper;
      const cb = () => {
        const rect = wrapper == null ? void 0 : wrapper.getBoundingClientRect();
        setWidth((rect == null ? void 0 : rect.width) ?? 0);
      };
      cb();
      const cleanCb = () => {
        var _a, _b, _c;
        (_a = typedWrapper == null ? void 0 : typedWrapper.__observeCallback) == null ? void 0 : _a.delete(cb);
        if (((_b = typedWrapper == null ? void 0 : typedWrapper.__observeCallback) == null ? void 0 : _b.size) === 0) {
          (_c = typedWrapper.__observeInstance) == null ? void 0 : _c.disconnect();
          typedWrapper.removeAttribute("data-observe");
          delete typedWrapper.__observeCallback;
          delete typedWrapper.__observeInstance;
        }
      };
      if (typedWrapper.__observeCallback) {
        typedWrapper.__observeCallback.add(cb);
        solidJs.onCleanup(() => cleanCb());
        return;
      }
      typedWrapper.__observeCallback = /* @__PURE__ */ new Set();
      typedWrapper.__observeCallback.add(cb);
      const observer = new ResizeObserver(() => {
        var _a, _b;
        return (_b = (_a = typedWrapper == null ? void 0 : typedWrapper.__observeCallback) == null ? void 0 : _a.forEach) == null ? void 0 : _b.call(_a, (cb2) => cb2());
      });
      typedWrapper.__observeInstance = observer;
      observer.observe(typedWrapper);
      typedWrapper.setAttribute("data-observe", "height");
      solidJs.onCleanup(() => cleanCb());
    }
  };
  solidJs.createEffect(observeWidth);
  return width;
};
const useSyncHeight = ({
  selector,
  wrapper,
  side,
  enable
}) => {
  const id = useId();
  const isMounted = useIsMounted();
  const observeHeight = () => {
    if (!isMounted()) return;
    if (enable()) {
      let clean = () => {
      };
      const container = document.querySelector(`#diff-root${id()}`);
      const elements = Array.from((container == null ? void 0 : container.querySelectorAll(selector())) || []);
      const wrappers = wrapper() ? Array.from((container == null ? void 0 : container.querySelectorAll(wrapper())) || []) : elements;
      if (elements.length === 2 && wrappers.length === 2) {
        const ele1 = elements[0];
        const ele2 = elements[1];
        const wrapper1 = wrappers[0];
        const wrapper2 = wrappers[1];
        const target = ele1.getAttribute("data-side") === side() ? ele1 : ele2;
        const typedTarget = target;
        const cb = () => {
          ele1.style.height = "auto";
          ele2.style.height = "auto";
          const rect1 = ele1.getBoundingClientRect();
          const rect2 = ele2.getBoundingClientRect();
          const maxHeight = Math.max(rect1.height, rect2.height);
          wrapper1.style.height = maxHeight + "px";
          wrapper2.style.height = maxHeight + "px";
          wrapper1.setAttribute("data-sync-height", String(maxHeight));
          wrapper2.setAttribute("data-sync-height", String(maxHeight));
        };
        cb();
        const cleanCb = () => {
          var _a, _b, _c;
          (_a = typedTarget.__observeCallback) == null ? void 0 : _a.delete(cb);
          if (((_b = typedTarget.__observeCallback) == null ? void 0 : _b.size) === 0) {
            (_c = typedTarget.__observeInstance) == null ? void 0 : _c.disconnect();
            target.removeAttribute("data-observe");
            delete typedTarget.__observeCallback;
            delete typedTarget.__observeInstance;
          }
        };
        if (typedTarget.__observeCallback) {
          typedTarget.__observeCallback.add(cb);
          clean = cleanCb;
          return;
        }
        typedTarget.__observeCallback = /* @__PURE__ */ new Set();
        typedTarget.__observeCallback.add(cb);
        const observer = new ResizeObserver(() => {
          var _a;
          return (_a = typedTarget.__observeCallback) == null ? void 0 : _a.forEach((cb2) => cb2());
        });
        typedTarget.__observeInstance = observer;
        observer.observe(target);
        target.setAttribute("data-observe", "height");
        clean = cleanCb;
      }
      solidJs.onCleanup(() => clean());
    }
  };
  solidJs.createEffect(observeHeight);
};
const useExtendData = generateHook("extendData");
const useRenderExtend = generateHook("renderExtendLine");
const useRenderWidget = generateHook("renderWidgetLine");
const useEnableAddWidget = generateHook("enableAddWidget");
const useEnableHighlight = generateHook("enableHighlight");
const useOnAddWidgetClick = generateHook("onAddWidgetClick");
var _tmpl$$j = /* @__PURE__ */ template(`<div><button class="diff-add-widget z-[1] flex h-full w-full origin-center cursor-pointer items-center justify-center rounded-md text-[1.2em]">+`), _tmpl$2$e = /* @__PURE__ */ template(`<div class="diff-add-widget-wrapper invisible absolute left-[100%] top-[1px] translate-x-[-50%] select-none transition-transform hover:scale-110 group-hover:visible"><button class="diff-add-widget z-[1] flex h-full w-full origin-center cursor-pointer items-center justify-center rounded-md text-[1.2em]">+`);
const DiffSplitAddWidget = (props) => {
  return (() => {
    var _el$ = _tmpl$$j(), _el$2 = _el$.firstChild;
    `calc(var(${diffFontSizeName}) * 1.4)` != null ? _el$.style.setProperty("width", `calc(var(${diffFontSizeName}) * 1.4)`) : _el$.style.removeProperty("width");
    `calc(var(${diffFontSizeName}) * 1.4)` != null ? _el$.style.setProperty("height", `calc(var(${diffFontSizeName}) * 1.4)`) : _el$.style.removeProperty("height");
    _el$2.$$click = () => {
      var _a, _b, _c;
      props.onOpenAddWidget(props.lineNumber, props.side);
      (_c = (_b = (_a = props.onWidgetClick) == null ? void 0 : _a.call(props)) == null ? void 0 : _b.current) == null ? void 0 : _c.call(_b, props.lineNumber, props.side);
    };
    `var(${addWidgetColorName})` != null ? _el$2.style.setProperty("color", `var(${addWidgetColorName})`) : _el$2.style.removeProperty("color");
    `var(${addWidgetBGName})` != null ? _el$2.style.setProperty("background-color", `var(${addWidgetBGName})`) : _el$2.style.removeProperty("background-color");
    solidJs.createRenderEffect(() => className(_el$, "diff-add-widget-wrapper invisible select-none transition-transform hover:scale-110 group-hover:visible" + (props.class ? " " + props.class : "")));
    return _el$;
  })();
};
const DiffUnifiedAddWidget = (props) => {
  return (() => {
    var _el$3 = _tmpl$2$e(), _el$4 = _el$3.firstChild;
    `calc(var(${diffFontSizeName}) * 1.4)` != null ? _el$3.style.setProperty("width", `calc(var(${diffFontSizeName}) * 1.4)`) : _el$3.style.removeProperty("width");
    `calc(var(${diffFontSizeName}) * 1.4)` != null ? _el$3.style.setProperty("height", `calc(var(${diffFontSizeName}) * 1.4)`) : _el$3.style.removeProperty("height");
    _el$4.$$click = () => {
      var _a, _b, _c;
      props.onOpenAddWidget(props.lineNumber, props.side);
      (_c = (_b = (_a = props.onWidgetClick) == null ? void 0 : _a.call(props)) == null ? void 0 : _b.current) == null ? void 0 : _c.call(_b, props.lineNumber, props.side);
    };
    `var(${addWidgetColorName})` != null ? _el$4.style.setProperty("color", `var(${addWidgetColorName})`) : _el$4.style.removeProperty("color");
    `var(${addWidgetBGName})` != null ? _el$4.style.setProperty("background-color", `var(${addWidgetBGName})`) : _el$4.style.removeProperty("background-color");
    return _el$3;
  })();
};
delegateEvents(["click"]);
var _tmpl$$i = /* @__PURE__ */ template(`<svg aria-label="No newline at end of file"role=img viewBox="0 0 16 16"version=1.1 fill=currentColor><path d="M4.25 7.25a.75.75 0 0 0 0 1.5h7.5a.75.75 0 0 0 0-1.5h-7.5Z"></path><path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0Zm-1.5 0a6.5 6.5 0 1 0-13 0 6.5 6.5 0 0 0 13 0Z">`);
const DiffNoNewLine = () => {
  return _tmpl$$i();
};
var _tmpl$$h = /* @__PURE__ */ template(`<span class=diff-line-content-raw><span data-template>`), _tmpl$2$d = /* @__PURE__ */ template(`<span class=diff-line-content-raw>`), _tmpl$3$a = /* @__PURE__ */ template(`<span class=diff-line-content-raw><span><span data-diff-highlight class=rounded-[0.2em]>`), _tmpl$4$7 = /* @__PURE__ */ template(`<span data-newline-symbol>`), _tmpl$5$6 = /* @__PURE__ */ template(`<span data-no-newline-at-end-of-file-symbol>`), _tmpl$6$4 = /* @__PURE__ */ template(`<span class=diff-line-syntax-raw><span data-template>`), _tmpl$7$4 = /* @__PURE__ */ template(`<span class=diff-line-syntax-raw>`), _tmpl$8$2 = /* @__PURE__ */ template(`<span>`), _tmpl$9$1 = /* @__PURE__ */ template(`<span class=diff-line-syntax-raw><span>`), _tmpl$10$1 = /* @__PURE__ */ template(`<span><span data-diff-highlight>`), _tmpl$11$1 = /* @__PURE__ */ template(`<div class="diff-line-content-item pl-[2.0em]"><span class="diff-line-content-operator ml-[-1.5em] inline-block w-[1.5em] select-none indent-[0.2em]">`);
const DiffString = (props) => {
  const getIsNewLineSymbolChanged = () => {
    var _a, _b;
    return (_b = (_a = props.diffLine) == null ? void 0 : _a.changes) == null ? void 0 : _b.newLineSymbol;
  };
  const initTemplateIfNeed = () => {
    var _a;
    if (props.enableTemplate && props.operator && props.diffLine && !((_a = props.diffLine) == null ? void 0 : _a.plainTemplate) && typeof core.getPlainDiffTemplate === "function") {
      core.getPlainDiffTemplate({
        diffLine: props.diffLine,
        rawLine: props.rawLine,
        operator: props.operator
      });
    }
    if (props.enableTemplate && props.plainLine && !props.plainLine.template) {
      props.plainLine.template = core.getPlainLineTemplate(props.plainLine.value);
    }
  };
  solidJs.createMemo(initTemplateIfNeed);
  return solidJs.createComponent(solidJs.Show, {
    get when() {
      var _a, _b;
      return (_b = (_a = props.diffLine) == null ? void 0 : _a.changes) == null ? void 0 : _b.hasLineChange;
    },
    get fallback() {
      return solidJs.createComponent(solidJs.Show, {
        get when() {
          var _a;
          return props.enableTemplate && ((_a = props.plainLine) == null ? void 0 : _a.template);
        },
        get fallback() {
          return (() => {
            var _el$5 = _tmpl$2$d();
            insert(_el$5, () => props.rawLine);
            return _el$5;
          })();
        },
        get children() {
          var _el$3 = _tmpl$$h(), _el$4 = _el$3.firstChild;
          solidJs.createRenderEffect(() => {
            var _a;
            return _el$4.innerHTML = (_a = props.plainLine) == null ? void 0 : _a.template;
          });
          return _el$3;
        }
      });
    },
    get children() {
      return solidJs.createComponent(solidJs.Show, {
        get when() {
          var _a;
          return props.enableTemplate && ((_a = props.diffLine) == null ? void 0 : _a.plainTemplate);
        },
        get fallback() {
          return function() {
            var _a, _b;
            const range = (_b = (_a = props.diffLine) == null ? void 0 : _a.changes) == null ? void 0 : _b.range;
            const str1 = props.rawLine.slice(0, range == null ? void 0 : range.location);
            const str2 = props.rawLine.slice(range == null ? void 0 : range.location, ((range == null ? void 0 : range.location) || 0) + ((range == null ? void 0 : range.length) || 0));
            const str3 = props.rawLine.slice(((range == null ? void 0 : range.location) || 0) + ((range == null ? void 0 : range.length) || 0));
            const isLast = str2.includes("\n");
            const _str2 = isLast ? str2.replace("\n", "").replace("\r", "") : str2;
            return (() => {
              var _el$6 = _tmpl$3$a(), _el$7 = _el$6.firstChild, _el$8 = _el$7.firstChild;
              insert(_el$7, str1, _el$8);
              insert(_el$8, isLast ? [_str2, (() => {
                var _el$9 = _tmpl$4$7();
                insert(_el$9, () => getSymbol(getIsNewLineSymbolChanged()));
                return _el$9;
              })()] : str2);
              insert(_el$7, str3, null);
              insert(_el$6, (() => {
                var _c$2 = solidJs.createMemo(() => getIsNewLineSymbolChanged() === NewLineSymbol.NEWLINE);
                return () => _c$2() && (() => {
                  var _el$10 = _tmpl$5$6();
                  `var(${diffFontSizeName})` != null ? _el$10.style.setProperty("width", `var(${diffFontSizeName})`) : _el$10.style.removeProperty("width");
                  `var(${diffFontSizeName})` != null ? _el$10.style.setProperty("height", `var(${diffFontSizeName})`) : _el$10.style.removeProperty("height");
                  insert(_el$10, solidJs.createComponent(DiffNoNewLine, {}));
                  solidJs.createRenderEffect(() => className(_el$10, props.enableWrap ? "block !text-red-500" : "inline-block align-middle !text-red-500"));
                  return _el$10;
                })();
              })(), null);
              solidJs.createRenderEffect((_p$) => {
                var _v$ = range == null ? void 0 : range.location, _v$2 = ((range == null ? void 0 : range.location) || 0) + ((range == null ? void 0 : range.length) || 0), _v$3 = props.operator === "add" ? `var(${addContentHighlightBGName})` : `var(${delContentHighlightBGName})`;
                _v$ !== _p$.e && setAttribute(_el$7, "data-range-start", _p$.e = _v$);
                _v$2 !== _p$.t && setAttribute(_el$7, "data-range-end", _p$.t = _v$2);
                _v$3 !== _p$.a && ((_p$.a = _v$3) != null ? _el$8.style.setProperty("background-color", _v$3) : _el$8.style.removeProperty("background-color"));
                return _p$;
              }, {
                e: void 0,
                t: void 0,
                a: void 0
              });
              return _el$6;
            })();
          }();
        },
        get children() {
          var _el$ = _tmpl$$h(), _el$2 = _el$.firstChild;
          insert(_el$, (() => {
            var _c$ = solidJs.createMemo(() => getIsNewLineSymbolChanged() === NewLineSymbol.NEWLINE);
            return () => _c$() && (() => {
              var _el$11 = _tmpl$5$6();
              `var(${diffFontSizeName})` != null ? _el$11.style.setProperty("width", `var(${diffFontSizeName})`) : _el$11.style.removeProperty("width");
              `var(${diffFontSizeName})` != null ? _el$11.style.setProperty("height", `var(${diffFontSizeName})`) : _el$11.style.removeProperty("height");
              insert(_el$11, solidJs.createComponent(DiffNoNewLine, {}));
              solidJs.createRenderEffect(() => className(_el$11, props.enableWrap ? "block !text-red-500" : "inline-block align-middle !text-red-500"));
              return _el$11;
            })();
          })(), null);
          solidJs.createRenderEffect(() => {
            var _a;
            return _el$2.innerHTML = (_a = props.diffLine) == null ? void 0 : _a.plainTemplate;
          });
          return _el$;
        }
      });
    }
  });
};
const DiffSyntax = (props) => {
  const getRange = () => {
    var _a, _b;
    return (_b = (_a = props.diffLine) == null ? void 0 : _a.changes) == null ? void 0 : _b.range;
  };
  const getIsNewLineSymbolChanged = () => {
    var _a, _b;
    return (_b = (_a = props.diffLine) == null ? void 0 : _a.changes) == null ? void 0 : _b.newLineSymbol;
  };
  const initTemplateIfNeed = () => {
    var _a, _b;
    if (props.enableTemplate && props.diffLine && props.syntaxLine && props.operator && !((_a = props.diffLine) == null ? void 0 : _a.syntaxTemplate) && typeof core.getSyntaxDiffTemplate === "function") {
      core.getSyntaxDiffTemplate({
        diffLine: props.diffLine,
        syntaxLine: props.syntaxLine,
        operator: props.operator
      });
    }
    if (props.enableTemplate && props.syntaxLine && !((_b = props.syntaxLine) == null ? void 0 : _b.template)) {
      props.syntaxLine.template = core.getSyntaxLineTemplate(props.syntaxLine);
    }
  };
  solidJs.createMemo(initTemplateIfNeed);
  return solidJs.createComponent(solidJs.Show, {
    get when() {
      return props.syntaxLine;
    },
    get fallback() {
      return solidJs.createComponent(DiffString, {
        get rawLine() {
          return props.rawLine;
        },
        get diffLine() {
          return props.diffLine;
        },
        get operator() {
          return props.operator;
        },
        get enableWrap() {
          return props.enableWrap;
        },
        get enableTemplate() {
          return props.enableTemplate;
        }
      });
    },
    get children() {
      return solidJs.createComponent(solidJs.Show, {
        get when() {
          var _a, _b;
          return (_b = (_a = props.diffLine) == null ? void 0 : _a.changes) == null ? void 0 : _b.hasLineChange;
        },
        get fallback() {
          return solidJs.createComponent(solidJs.Show, {
            get when() {
              var _a;
              return props.enableTemplate && ((_a = props.syntaxLine) == null ? void 0 : _a.template);
            },
            get fallback() {
              return (() => {
                var _el$16 = _tmpl$7$4();
                insert(_el$16, solidJs.createComponent(solidJs.For, {
                  get each() {
                    var _a;
                    return ((_a = props.syntaxLine) == null ? void 0 : _a.nodeList) || [];
                  },
                  children: ({
                    node,
                    wrapper
                  }) => (() => {
                    var _el$17 = _tmpl$8$2();
                    insert(_el$17, () => node.value);
                    solidJs.createRenderEffect((_p$) => {
                      var _a, _b, _c;
                      var _v$4 = node.startIndex, _v$5 = node.endIndex, _v$6 = (_b = (_a = wrapper == null ? void 0 : wrapper.properties) == null ? void 0 : _a.className) == null ? void 0 : _b.join(" "), _v$7 = (_c = wrapper == null ? void 0 : wrapper.properties) == null ? void 0 : _c.style;
                      _v$4 !== _p$.e && setAttribute(_el$17, "data-start", _p$.e = _v$4);
                      _v$5 !== _p$.t && setAttribute(_el$17, "data-end", _p$.t = _v$5);
                      _v$6 !== _p$.a && className(_el$17, _p$.a = _v$6);
                      _p$.o = style(_el$17, _v$7, _p$.o);
                      return _p$;
                    }, {
                      e: void 0,
                      t: void 0,
                      a: void 0,
                      o: void 0
                    });
                    return _el$17;
                  })()
                }));
                return _el$16;
              })();
            },
            get children() {
              var _el$14 = _tmpl$6$4(), _el$15 = _el$14.firstChild;
              solidJs.createRenderEffect(() => {
                var _a;
                return _el$15.innerHTML = (_a = props.syntaxLine) == null ? void 0 : _a.template;
              });
              return _el$14;
            }
          });
        },
        get children() {
          return solidJs.createComponent(solidJs.Show, {
            get when() {
              var _a;
              return props.enableTemplate && ((_a = props.diffLine) == null ? void 0 : _a.syntaxTemplate);
            },
            get fallback() {
              return (() => {
                var _el$18 = _tmpl$9$1(), _el$19 = _el$18.firstChild;
                insert(_el$19, solidJs.createComponent(solidJs.For, {
                  get each() {
                    var _a;
                    return (_a = props.syntaxLine) == null ? void 0 : _a.nodeList;
                  },
                  children: ({
                    node,
                    wrapper
                  }) => {
                    if (node.endIndex < getRange().location || getRange().location + getRange().length < node.startIndex) {
                      return (() => {
                        var _el$20 = _tmpl$8$2();
                        insert(_el$20, () => node.value);
                        solidJs.createRenderEffect((_p$) => {
                          var _a, _b, _c;
                          var _v$10 = node.startIndex, _v$11 = node.endIndex, _v$12 = (_b = (_a = wrapper == null ? void 0 : wrapper.properties) == null ? void 0 : _a.className) == null ? void 0 : _b.join(" "), _v$13 = (_c = wrapper == null ? void 0 : wrapper.properties) == null ? void 0 : _c.style;
                          _v$10 !== _p$.e && setAttribute(_el$20, "data-start", _p$.e = _v$10);
                          _v$11 !== _p$.t && setAttribute(_el$20, "data-end", _p$.t = _v$11);
                          _v$12 !== _p$.a && className(_el$20, _p$.a = _v$12);
                          _p$.o = style(_el$20, _v$13, _p$.o);
                          return _p$;
                        }, {
                          e: void 0,
                          t: void 0,
                          a: void 0,
                          o: void 0
                        });
                        return _el$20;
                      })();
                    } else {
                      const index1 = getRange().location - node.startIndex;
                      const index2 = index1 < 0 ? 0 : index1;
                      const str1 = node.value.slice(0, index2);
                      const str2 = node.value.slice(index2, index1 + getRange().length);
                      const str3 = node.value.slice(index1 + getRange().length);
                      const isStart = str1.length || getRange().location === node.startIndex;
                      const isEnd = str3.length || node.endIndex === getRange().location + getRange().length - 1;
                      const isLast = str2.includes("\n");
                      const _str2 = isLast ? str2.replace("\n", "").replace("\r", "") : str2;
                      return (() => {
                        var _el$21 = _tmpl$10$1(), _el$22 = _el$21.firstChild;
                        insert(_el$21, str1, _el$22);
                        (isStart ? "0.2em" : void 0) != null ? _el$22.style.setProperty("border-top-left-radius", isStart ? "0.2em" : void 0) : _el$22.style.removeProperty("border-top-left-radius");
                        (isStart ? "0.2em" : void 0) != null ? _el$22.style.setProperty("border-bottom-left-radius", isStart ? "0.2em" : void 0) : _el$22.style.removeProperty("border-bottom-left-radius");
                        (isEnd || isLast ? "0.2em" : void 0) != null ? _el$22.style.setProperty("border-top-right-radius", isEnd || isLast ? "0.2em" : void 0) : _el$22.style.removeProperty("border-top-right-radius");
                        (isEnd || isLast ? "0.2em" : void 0) != null ? _el$22.style.setProperty("border-bottom-right-radius", isEnd || isLast ? "0.2em" : void 0) : _el$22.style.removeProperty("border-bottom-right-radius");
                        insert(_el$22, isLast ? [_str2, (() => {
                          var _el$23 = _tmpl$4$7();
                          insert(_el$23, () => getSymbol(getIsNewLineSymbolChanged()));
                          return _el$23;
                        })()] : str2);
                        insert(_el$21, str3, null);
                        solidJs.createRenderEffect((_p$) => {
                          var _a, _b, _c;
                          var _v$14 = node.startIndex, _v$15 = node.endIndex, _v$16 = (_b = (_a = wrapper == null ? void 0 : wrapper.properties) == null ? void 0 : _a.className) == null ? void 0 : _b.join(" "), _v$17 = (_c = wrapper == null ? void 0 : wrapper.properties) == null ? void 0 : _c.style, _v$18 = props.operator === "add" ? `var(${addContentHighlightBGName})` : `var(${delContentHighlightBGName})`;
                          _v$14 !== _p$.e && setAttribute(_el$21, "data-start", _p$.e = _v$14);
                          _v$15 !== _p$.t && setAttribute(_el$21, "data-end", _p$.t = _v$15);
                          _v$16 !== _p$.a && className(_el$21, _p$.a = _v$16);
                          _p$.o = style(_el$21, _v$17, _p$.o);
                          _v$18 !== _p$.i && ((_p$.i = _v$18) != null ? _el$22.style.setProperty("background-color", _v$18) : _el$22.style.removeProperty("background-color"));
                          return _p$;
                        }, {
                          e: void 0,
                          t: void 0,
                          a: void 0,
                          o: void 0,
                          i: void 0
                        });
                        return _el$21;
                      })();
                    }
                  }
                }));
                insert(_el$18, (() => {
                  var _c$4 = solidJs.createMemo(() => getIsNewLineSymbolChanged() === NewLineSymbol.NEWLINE);
                  return () => _c$4() && (() => {
                    var _el$24 = _tmpl$5$6();
                    `var(${diffFontSizeName})` != null ? _el$24.style.setProperty("width", `var(${diffFontSizeName})`) : _el$24.style.removeProperty("width");
                    `var(${diffFontSizeName})` != null ? _el$24.style.setProperty("height", `var(${diffFontSizeName})`) : _el$24.style.removeProperty("height");
                    insert(_el$24, solidJs.createComponent(DiffNoNewLine, {}));
                    solidJs.createRenderEffect(() => className(_el$24, props.enableWrap ? "block !text-red-500" : "inline-block align-middle !text-red-500"));
                    return _el$24;
                  })();
                })(), null);
                solidJs.createRenderEffect((_p$) => {
                  var _a, _b, _c;
                  var _v$8 = (_a = getRange()) == null ? void 0 : _a.location, _v$9 = (((_b = getRange()) == null ? void 0 : _b.location) || 0) + (((_c = getRange()) == null ? void 0 : _c.length) || 0);
                  _v$8 !== _p$.e && setAttribute(_el$19, "data-range-start", _p$.e = _v$8);
                  _v$9 !== _p$.t && setAttribute(_el$19, "data-range-end", _p$.t = _v$9);
                  return _p$;
                }, {
                  e: void 0,
                  t: void 0
                });
                return _el$18;
              })();
            },
            get children() {
              var _el$12 = _tmpl$6$4(), _el$13 = _el$12.firstChild;
              insert(_el$12, (() => {
                var _c$3 = solidJs.createMemo(() => getIsNewLineSymbolChanged() === NewLineSymbol.NEWLINE);
                return () => _c$3() && (() => {
                  var _el$25 = _tmpl$5$6();
                  `var(${diffFontSizeName})` != null ? _el$25.style.setProperty("width", `var(${diffFontSizeName})`) : _el$25.style.removeProperty("width");
                  `var(${diffFontSizeName})` != null ? _el$25.style.setProperty("height", `var(${diffFontSizeName})`) : _el$25.style.removeProperty("height");
                  insert(_el$25, solidJs.createComponent(DiffNoNewLine, {}));
                  solidJs.createRenderEffect(() => className(_el$25, props.enableWrap ? "block !text-red-500" : "inline-block align-middle !text-red-500"));
                  return _el$25;
                })();
              })(), null);
              solidJs.createRenderEffect(() => {
                var _a;
                return _el$13.innerHTML = (_a = props.diffLine) == null ? void 0 : _a.syntaxTemplate;
              });
              return _el$12;
            }
          });
        }
      });
    }
  });
};
const DiffContent = (props) => {
  const getIsAdded = () => {
    var _a;
    return ((_a = props.diffLine) == null ? void 0 : _a.type) === core.DiffLineType.Add;
  };
  const getIsDelete = () => {
    var _a;
    return ((_a = props.diffLine) == null ? void 0 : _a.type) === core.DiffLineType.Delete;
  };
  const getIsMaxLineLengthToIgnoreSyntax = () => {
    var _a, _b, _c;
    return ((_a = props.syntaxLine) == null ? void 0 : _a.nodeList) ? ((_c = (_b = props.syntaxLine) == null ? void 0 : _b.nodeList) == null ? void 0 : _c.length) > 150 : false;
  };
  const getIsEnableTemplate = () => {
    var _a, _b;
    return ((_b = (_a = props.diffFile).getIsEnableTemplate) == null ? void 0 : _b.call(_a)) ?? true;
  };
  return (() => {
    var _el$26 = _tmpl$11$1(), _el$27 = _el$26.firstChild;
    insert(_el$27, (() => {
      var _c$5 = solidJs.createMemo(() => !!getIsAdded());
      return () => _c$5() ? "+" : getIsDelete() ? "-" : " ";
    })());
    insert(_el$26, (() => {
      var _c$6 = solidJs.createMemo(() => !!(props.enableHighlight && props.syntaxLine && !getIsMaxLineLengthToIgnoreSyntax()));
      return () => _c$6() ? solidJs.createComponent(DiffSyntax, {
        get operator() {
          return solidJs.createMemo(() => !!getIsAdded())() ? "add" : getIsDelete() ? "del" : void 0;
        },
        get rawLine() {
          return props.rawLine;
        },
        get diffLine() {
          return props.diffLine;
        },
        get syntaxLine() {
          return props.syntaxLine;
        },
        get enableWrap() {
          return props.enableWrap;
        },
        get enableTemplate() {
          return getIsEnableTemplate();
        }
      }) : solidJs.createComponent(DiffString, {
        get operator() {
          return solidJs.createMemo(() => !!getIsAdded())() ? "add" : getIsDelete() ? "del" : void 0;
        },
        get rawLine() {
          return props.rawLine;
        },
        get diffLine() {
          return props.diffLine;
        },
        get plainLine() {
          return props.plainLine;
        },
        get enableWrap() {
          return props.enableWrap;
        },
        get enableTemplate() {
          return getIsEnableTemplate();
        }
      });
    })(), null);
    solidJs.createRenderEffect((_p$) => {
      var _v$19 = props.enableWrap ? "pre-wrap" : "pre", _v$20 = props.enableWrap ? "break-all" : "initial", _v$21 = getIsAdded() ? "+" : getIsDelete() ? "-" : void 0;
      _v$19 !== _p$.e && ((_p$.e = _v$19) != null ? _el$26.style.setProperty("white-space", _v$19) : _el$26.style.removeProperty("white-space"));
      _v$20 !== _p$.t && ((_p$.t = _v$20) != null ? _el$26.style.setProperty("word-break", _v$20) : _el$26.style.removeProperty("word-break"));
      _v$21 !== _p$.a && setAttribute(_el$27, "data-operator", _p$.a = _v$21);
      return _p$;
    }, {
      e: void 0,
      t: void 0,
      a: void 0
    });
    return _el$26;
  })();
};
const DiffWidgetContext = solidJs.createContext();
const useDiffWidgetContext = () => solidJs.useContext(DiffWidgetContext);
var _tmpl$$g = /* @__PURE__ */ template(`<tr>`), _tmpl$2$c = /* @__PURE__ */ template(`<td><span>`), _tmpl$3$9 = /* @__PURE__ */ template(`<td>`), _tmpl$4$6 = /* @__PURE__ */ template(`<td colspan=2><span>&ensp;`);
const DiffSplitContentLine$1 = (props) => {
  const [_, setWidget] = useDiffWidgetContext() || [];
  const enableAddWidget = useEnableAddWidget();
  const enableHighlight = useEnableHighlight();
  const onAddWidgetClick = useOnAddWidgetClick();
  const getCurrentLine = () => props.side === core.SplitSide.old ? props.diffFile.getSplitLeftLine(props.index) : props.diffFile.getSplitRightLine(props.index);
  const currentLine = solidJs.createMemo(getCurrentLine);
  const getCUrrentLineHasDiff = () => {
    var _a;
    return !!((_a = currentLine()) == null ? void 0 : _a.diff);
  };
  const currentLineHasDiff = solidJs.createMemo(getCUrrentLineHasDiff);
  const currentLineHasChange = solidJs.createMemo(() => {
    var _a;
    return core.checkDiffLineIncludeChange((_a = currentLine()) == null ? void 0 : _a.diff);
  });
  const getCurrentLineHasHidden = () => {
    var _a;
    return (_a = currentLine()) == null ? void 0 : _a.isHidden;
  };
  const currentLineHasHidden = solidJs.createMemo(getCurrentLineHasHidden);
  const currentLineHasContent = solidJs.createMemo(() => {
    var _a;
    return !!((_a = currentLine()) == null ? void 0 : _a.lineNumber);
  });
  const getCurrentSyntaxLine = () => {
    var _a, _b;
    return props.side === core.SplitSide.old ? props.diffFile.getOldSyntaxLine(((_a = currentLine()) == null ? void 0 : _a.lineNumber) || 0) : props.diffFile.getNewSyntaxLine(((_b = currentLine()) == null ? void 0 : _b.lineNumber) || 0);
  };
  const getCurrentPlainLine = () => {
    var _a, _b;
    return props.side === core.SplitSide.old ? props.diffFile.getOldPlainLine(((_a = currentLine()) == null ? void 0 : _a.lineNumber) || 0) : props.diffFile.getNewPlainLine(((_b = currentLine()) == null ? void 0 : _b.lineNumber) || 0);
  };
  const [currentSyntaxLine, setCurrentSyntaxLine] = solidJs.createSignal(getCurrentSyntaxLine());
  const [currentPlainLine, setCurrentPlainLine] = solidJs.createSignal(getCurrentPlainLine());
  const init = () => {
    setCurrentSyntaxLine(getCurrentSyntaxLine);
    setCurrentPlainLine(getCurrentPlainLine);
  };
  solidJs.createEffect(() => {
    init();
    const cb = props.diffFile.subscribe(init);
    solidJs.onCleanup(cb);
  });
  const onOpenAddWidget = (lineNumber, side) => setWidget == null ? void 0 : setWidget({
    side,
    lineNumber
  });
  const isAdded = () => {
    var _a, _b;
    return ((_b = (_a = currentLine()) == null ? void 0 : _a.diff) == null ? void 0 : _b.type) === core.DiffLineType.Add;
  };
  const isDelete = () => {
    var _a, _b;
    return ((_b = (_a = currentLine()) == null ? void 0 : _a.diff) == null ? void 0 : _b.type) === core.DiffLineType.Delete;
  };
  return solidJs.createComponent(solidJs.Show, {
    get when() {
      return !currentLineHasHidden();
    },
    get children() {
      var _el$ = _tmpl$$g();
      insert(_el$, (() => {
        var _c$ = solidJs.createMemo(() => !!currentLineHasContent());
        return () => _c$() ? [(() => {
          var _el$2 = _tmpl$2$c(), _el$3 = _el$2.firstChild;
          `var(${diffAsideWidthName})` != null ? _el$2.style.setProperty("width", `var(${diffAsideWidthName})`) : _el$2.style.removeProperty("width");
          `var(${diffAsideWidthName})` != null ? _el$2.style.setProperty("min-width", `var(${diffAsideWidthName})`) : _el$2.style.removeProperty("min-width");
          `var(${diffAsideWidthName})` != null ? _el$2.style.setProperty("max-width", `var(${diffAsideWidthName})`) : _el$2.style.removeProperty("max-width");
          insert(_el$2, (() => {
            var _c$2 = solidJs.createMemo(() => !!(currentLineHasDiff() && enableAddWidget()));
            return () => _c$2() && solidJs.createComponent(DiffSplitAddWidget, {
              get index() {
                return props.index;
              },
              get lineNumber() {
                var _a;
                return ((_a = currentLine()) == null ? void 0 : _a.lineNumber) || 0;
              },
              get side() {
                return props.side;
              },
              get diffFile() {
                return props.diffFile;
              },
              onWidgetClick: onAddWidgetClick,
              "class": "absolute left-[100%] z-[1] translate-x-[-50%]",
              onOpenAddWidget
            });
          })(), _el$3);
          insert(_el$3, () => {
            var _a;
            return (_a = currentLine()) == null ? void 0 : _a.lineNumber;
          });
          solidJs.createRenderEffect((_p$) => {
            var _a;
            var _v$5 = `diff-line-${core.SplitSide[props.side]}-num sticky left-0 z-[1] w-[1%] min-w-[40px] select-none pl-[10px] pr-[10px] text-right align-top`, _v$6 = getLineNumberBG(isAdded(), isDelete(), currentLineHasDiff()), _v$7 = `var(${currentLineHasDiff() ? plainLineNumberColorName : expandLineNumberColorName})`, _v$8 = (_a = currentLine()) == null ? void 0 : _a.lineNumber, _v$9 = currentLineHasChange() ? void 0 : 0.5;
            _v$5 !== _p$.e && className(_el$2, _p$.e = _v$5);
            _v$6 !== _p$.t && ((_p$.t = _v$6) != null ? _el$2.style.setProperty("background-color", _v$6) : _el$2.style.removeProperty("background-color"));
            _v$7 !== _p$.a && ((_p$.a = _v$7) != null ? _el$2.style.setProperty("color", _v$7) : _el$2.style.removeProperty("color"));
            _v$8 !== _p$.o && setAttribute(_el$3, "data-line-num", _p$.o = _v$8);
            _v$9 !== _p$.i && ((_p$.i = _v$9) != null ? _el$3.style.setProperty("opacity", _v$9) : _el$3.style.removeProperty("opacity"));
            return _p$;
          }, {
            e: void 0,
            t: void 0,
            a: void 0,
            o: void 0,
            i: void 0
          });
          return _el$2;
        })(), (() => {
          var _el$4 = _tmpl$3$9();
          insert(_el$4, solidJs.createComponent(DiffContent, {
            enableWrap: false,
            get diffFile() {
              return props.diffFile;
            },
            get rawLine() {
              var _a;
              return ((_a = currentLine()) == null ? void 0 : _a.value) || "";
            },
            get diffLine() {
              var _a;
              return (_a = currentLine()) == null ? void 0 : _a.diff;
            },
            get plainLine() {
              return currentPlainLine();
            },
            get syntaxLine() {
              return currentSyntaxLine();
            },
            get enableHighlight() {
              return !!enableHighlight();
            }
          }));
          solidJs.createRenderEffect((_p$) => {
            var _v$10 = `diff-line-${core.SplitSide[props.side]}-content pr-[10px] align-top`, _v$11 = getContentBG(isAdded(), isDelete(), currentLineHasDiff());
            _v$10 !== _p$.e && className(_el$4, _p$.e = _v$10);
            _v$11 !== _p$.t && ((_p$.t = _v$11) != null ? _el$4.style.setProperty("background-color", _v$11) : _el$4.style.removeProperty("background-color"));
            return _p$;
          }, {
            e: void 0,
            t: void 0
          });
          return _el$4;
        })()] : (() => {
          var _el$5 = _tmpl$4$6();
          `var(${emptyBGName})` != null ? _el$5.style.setProperty("background-color", `var(${emptyBGName})`) : _el$5.style.removeProperty("background-color");
          solidJs.createRenderEffect(() => className(_el$5, `diff-line-${core.SplitSide[props.side]}-placeholder select-none`));
          return _el$5;
        })();
      })());
      solidJs.createRenderEffect((_p$) => {
        var _v$ = props.lineNumber, _v$2 = currentLineHasDiff() || !currentLineHasContent() ? "diff" : "plain", _v$3 = core.SplitSide[props.side], _v$4 = "diff-line" + (currentLineHasContent() ? " group" : "");
        _v$ !== _p$.e && setAttribute(_el$, "data-line", _p$.e = _v$);
        _v$2 !== _p$.t && setAttribute(_el$, "data-state", _p$.t = _v$2);
        _v$3 !== _p$.a && setAttribute(_el$, "data-side", _p$.a = _v$3);
        _v$4 !== _p$.o && className(_el$, _p$.o = _v$4);
        return _p$;
      }, {
        e: void 0,
        t: void 0,
        a: void 0,
        o: void 0
      });
      return _el$;
    }
  });
};
var _tmpl$$f = /* @__PURE__ */ template(`<tr data-state=extend class="diff-line diff-line-extend">`), _tmpl$2$b = /* @__PURE__ */ template(`<td colspan=2><div class="diff-line-extend-wrapper sticky left-0 z-[1]">`), _tmpl$3$8 = /* @__PURE__ */ template(`<td colspan=2><div>`);
const DiffSplitExtendLine$1 = (props) => {
  const extendData = useExtendData();
  const renderExtend = useRenderExtend();
  const lineSelector = solidJs.createMemo(() => `div[data-line="${props.lineNumber}-extend-content"]`);
  const lineWrapperSelector = solidJs.createMemo(() => `tr[data-line="${props.lineNumber}-extend"]`);
  const wrapperSelector = solidJs.createMemo(() => props.side === core.SplitSide.old ? ".old-diff-table-wrapper" : ".new-diff-table-wrapper");
  const oldLine = solidJs.createMemo(() => props.diffFile.getSplitLeftLine(props.index));
  const newLine = solidJs.createMemo(() => props.diffFile.getSplitRightLine(props.index));
  const enableExpand = solidJs.createMemo(() => props.diffFile.getExpandEnabled());
  const oldLineExtend = solidJs.createMemo(() => {
    var _a, _b, _c;
    return (_c = (_a = extendData()) == null ? void 0 : _a.oldFile) == null ? void 0 : _c[((_b = oldLine()) == null ? void 0 : _b.lineNumber) || ""];
  });
  const newLineExtend = solidJs.createMemo(() => {
    var _a, _b, _c;
    return (_c = (_a = extendData()) == null ? void 0 : _a.newFile) == null ? void 0 : _c[((_b = newLine()) == null ? void 0 : _b.lineNumber) || ""];
  });
  const currentItem = solidJs.createMemo(() => props.side === core.SplitSide.old ? oldLine() : newLine());
  const currentIsHidden = solidJs.createMemo(() => {
    var _a;
    return (_a = currentItem()) == null ? void 0 : _a.isHidden;
  });
  const currentExtend = solidJs.createMemo(() => props.side === core.SplitSide.old ? oldLineExtend() : newLineExtend());
  const currentLineNumber = solidJs.createMemo(() => {
    var _a, _b;
    return props.side === core.SplitSide.old ? (_a = oldLine()) == null ? void 0 : _a.lineNumber : (_b = newLine()) == null ? void 0 : _b.lineNumber;
  });
  const currentIsShow = solidJs.createMemo(() => Boolean((oldLineExtend() || newLineExtend()) && (!currentIsHidden() || enableExpand()) && renderExtend()));
  const currentEnable = solidJs.createMemo(() => (props.side === core.SplitSide.old ? !!oldLineExtend() : !!newLineExtend()) && currentIsShow());
  const extendSide = solidJs.createMemo(() => core.SplitSide[currentExtend() ? props.side : props.side === core.SplitSide.new ? core.SplitSide.old : core.SplitSide.new]);
  useSyncHeight({
    selector: lineSelector,
    wrapper: lineWrapperSelector,
    side: extendSide,
    enable: currentIsShow
  });
  const width = useDomWidth({
    selector: wrapperSelector,
    enable: currentEnable
  });
  return solidJs.createComponent(solidJs.Show, {
    get when() {
      return currentIsShow();
    },
    get children() {
      var _el$ = _tmpl$$f();
      insert(_el$, (() => {
        var _c$ = solidJs.createMemo(() => !!(renderExtend() && currentExtend()));
        return () => _c$() ? (() => {
          var _el$2 = _tmpl$2$b(), _el$3 = _el$2.firstChild;
          insert(_el$3, (() => {
            var _c$2 = solidJs.createMemo(() => width() > 0);
            return () => {
              var _a, _b;
              return _c$2() && ((_b = renderExtend()) == null ? void 0 : _b({
                diffFile: props.diffFile,
                side: props.side,
                lineNumber: currentLineNumber() || 0,
                data: (_a = currentExtend()) == null ? void 0 : _a.data,
                onUpdate: props.diffFile.notifyAll
              }));
            };
          })());
          solidJs.createRenderEffect((_p$) => {
            var _v$3 = `diff-line-extend-${core.SplitSide[props.side]}-content p-0`, _v$4 = `${props.lineNumber}-extend-content`, _v$5 = core.SplitSide[props.side], _v$6 = width() + "px";
            _v$3 !== _p$.e && className(_el$2, _p$.e = _v$3);
            _v$4 !== _p$.t && setAttribute(_el$3, "data-line", _p$.t = _v$4);
            _v$5 !== _p$.a && setAttribute(_el$3, "data-side", _p$.a = _v$5);
            _v$6 !== _p$.o && ((_p$.o = _v$6) != null ? _el$3.style.setProperty("width", _v$6) : _el$3.style.removeProperty("width"));
            return _p$;
          }, {
            e: void 0,
            t: void 0,
            a: void 0,
            o: void 0
          });
          return _el$2;
        })() : (() => {
          var _el$4 = _tmpl$3$8(), _el$5 = _el$4.firstChild;
          `var(${emptyBGName})` != null ? _el$4.style.setProperty("background-color", `var(${emptyBGName})`) : _el$4.style.removeProperty("background-color");
          solidJs.createRenderEffect((_p$) => {
            var _v$7 = `diff-line-extend-${core.SplitSide[props.side]}-placeholder select-none p-0`, _v$8 = `${props.lineNumber}-extend-content`, _v$9 = core.SplitSide[props.side];
            _v$7 !== _p$.e && className(_el$4, _p$.e = _v$7);
            _v$8 !== _p$.t && setAttribute(_el$5, "data-line", _p$.t = _v$8);
            _v$9 !== _p$.a && setAttribute(_el$5, "data-side", _p$.a = _v$9);
            return _p$;
          }, {
            e: void 0,
            t: void 0,
            a: void 0
          });
          return _el$4;
        })();
      })());
      solidJs.createRenderEffect((_p$) => {
        var _v$ = `${props.lineNumber}-extend`, _v$2 = core.SplitSide[props.side];
        _v$ !== _p$.e && setAttribute(_el$, "data-line", _p$.e = _v$);
        _v$2 !== _p$.t && setAttribute(_el$, "data-side", _p$.t = _v$2);
        return _p$;
      }, {
        e: void 0,
        t: void 0
      });
      return _el$;
    }
  });
};
var _tmpl$$e = /* @__PURE__ */ template(`<svg aria-hidden=true height=16 viewBox="0 0 16 16"version=1.1 width=16><path d="m8.177 14.323 2.896-2.896a.25.25 0 0 0-.177-.427H8.75V7.764a.75.75 0 1 0-1.5 0V11H5.104a.25.25 0 0 0-.177.427l2.896 2.896a.25.25 0 0 0 .354 0ZM2.25 5a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5ZM6 4.25a.75.75 0 0 1-.75.75h-.5a.75.75 0 0 1 0-1.5h.5a.75.75 0 0 1 .75.75ZM8.25 5a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5ZM12 4.25a.75.75 0 0 1-.75.75h-.5a.75.75 0 0 1 0-1.5h.5a.75.75 0 0 1 .75.75Zm2.25.75a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5Z">`), _tmpl$2$a = /* @__PURE__ */ template(`<svg aria-hidden=true height=16 viewBox="0 0 16 16"version=1.1 width=16><path d="M7.823 1.677 4.927 4.573A.25.25 0 0 0 5.104 5H7.25v3.236a.75.75 0 1 0 1.5 0V5h2.146a.25.25 0 0 0 .177-.427L8.177 1.677a.25.25 0 0 0-.354 0ZM13.75 11a.75.75 0 0 0 0 1.5h.5a.75.75 0 0 0 0-1.5h-.5Zm-3.75.75a.75.75 0 0 1 .75-.75h.5a.75.75 0 0 1 0 1.5h-.5a.75.75 0 0 1-.75-.75ZM7.75 11a.75.75 0 0 0 0 1.5h.5a.75.75 0 0 0 0-1.5h-.5ZM4 11.75a.75.75 0 0 1 .75-.75h.5a.75.75 0 0 1 0 1.5h-.5a.75.75 0 0 1-.75-.75ZM1.75 11a.75.75 0 0 0 0 1.5h.5a.75.75 0 0 0 0-1.5h-.5Z">`), _tmpl$3$7 = /* @__PURE__ */ template(`<svg aria-hidden=true height=16 viewBox="0 0 16 16"version=1.1 width=16><path d="m8.177.677 2.896 2.896a.25.25 0 0 1-.177.427H8.75v1.25a.75.75 0 0 1-1.5 0V4H5.104a.25.25 0 0 1-.177-.427L7.823.677a.25.25 0 0 1 .354 0ZM7.25 10.75a.75.75 0 0 1 1.5 0V12h2.146a.25.25 0 0 1 .177.427l-2.896 2.896a.25.25 0 0 1-.354 0l-2.896-2.896A.25.25 0 0 1 5.104 12H7.25v-1.25Zm-5-2a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5ZM6 8a.75.75 0 0 1-.75.75h-.5a.75.75 0 0 1 0-1.5h.5A.75.75 0 0 1 6 8Zm2.25.75a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5ZM12 8a.75.75 0 0 1-.75.75h-.5a.75.75 0 0 1 0-1.5h.5A.75.75 0 0 1 12 8Zm2.25.75a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5Z">`);
const ExpandDown = (props) => {
  return (() => {
    var _el$ = _tmpl$$e();
    solidJs.createRenderEffect(() => setAttribute(_el$, "class", props.class));
    return _el$;
  })();
};
const ExpandUp = (props) => {
  return (() => {
    var _el$2 = _tmpl$2$a();
    solidJs.createRenderEffect(() => setAttribute(_el$2, "class", props.class));
    return _el$2;
  })();
};
const ExpandAll = (props) => {
  return (() => {
    var _el$3 = _tmpl$3$7();
    solidJs.createRenderEffect(() => setAttribute(_el$3, "class", props.class));
    return _el$3;
  })();
};
var _tmpl$$d = /* @__PURE__ */ template(`<tr data-state=hunk class="diff-line diff-line-hunk">`), _tmpl$2$9 = /* @__PURE__ */ template(`<td class="diff-line-hunk-action sticky left-0 w-[1%] min-w-[40px] select-none p-[1px]">`), _tmpl$3$6 = /* @__PURE__ */ template(`<td class="diff-line-hunk-content pr-[10px] align-middle"><div class=pl-[1.5em]>`), _tmpl$4$5 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[6px]"title="Expand Up"data-title="Expand Up">`), _tmpl$5$5 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[6px]"title="Expand Down"data-title="Expand Down">`), _tmpl$6$3 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[6px]"title="Expand All"data-title="Expand All">`), _tmpl$7$3 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[2px]"title="Expand Down"data-title="Expand Down">`), _tmpl$8$1 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[2px]"title="Expand Up"data-title="Expand Up">`), _tmpl$9 = /* @__PURE__ */ template(`<div class=min-h-[28px]>&ensp;`), _tmpl$10 = /* @__PURE__ */ template(`<td class="diff-line-hunk-placeholder select-none"colspan=2><div class=min-h-[28px]>&ensp;`), _tmpl$11 = /* @__PURE__ */ template(`<tr data-state=hunk class="diff-line diff-line-hunk"><td class="diff-line-hunk-action sticky left-0 w-[1%] min-w-[40px] select-none p-[1px]"></td><td class="diff-line-hunk-content pr-[10px] align-middle"><div class=pl-[1.5em]>`);
const DiffSplitHunkLineGitHub$1 = (props) => {
  const currentHunk = solidJs.createMemo(() => props.diffFile.getSplitHunkLine(props.index));
  const enableExpand = solidJs.createMemo(() => props.diffFile.getExpandEnabled());
  const couldExpand = solidJs.createMemo(() => {
    var _a;
    return enableExpand() && ((_a = currentHunk()) == null ? void 0 : _a.splitInfo);
  });
  const lineSelector = solidJs.createMemo(() => `tr[data-line="${props.lineNumber}-hunk"]`);
  const currentShowExpand = solidJs.createMemo(() => props.side === core.SplitSide.old);
  const checkCurrentShowExpandAll = () => {
    const hunk = currentHunk();
    return hunk && hunk.splitInfo && hunk.splitInfo.endHiddenIndex - hunk.splitInfo.startHiddenIndex < core.composeLen;
  };
  const [currentShowExpandAll, setCurrentShowExpandAll] = solidJs.createSignal(checkCurrentShowExpandAll());
  const checkCurrentIsShow = () => {
    const hunk = currentHunk();
    return hunk && hunk.splitInfo && hunk.splitInfo.startHiddenIndex < hunk.splitInfo.endHiddenIndex;
  };
  const [currentIsShow, setCurrentIsShow] = solidJs.createSignal(checkCurrentIsShow());
  const currentIsFirstLine = solidJs.createMemo(() => {
    const hunk = currentHunk();
    return hunk && hunk.isFirst;
  });
  const currentIsPureHunk = solidJs.createMemo(() => {
    const hunk = currentHunk();
    return hunk && props.diffFile._getIsPureDiffRender() && !hunk.splitInfo;
  });
  const currentIsLastLine = solidJs.createMemo(() => {
    const hunk = currentHunk();
    return hunk && hunk.isLast;
  });
  solidJs.createEffect(() => {
    const init = () => {
      setCurrentIsShow(checkCurrentIsShow);
      setCurrentShowExpandAll(checkCurrentShowExpandAll);
    };
    init();
    const cb = props.diffFile.subscribe(init);
    solidJs.onCleanup(cb);
  });
  const currentSyncHeightSide = solidJs.createMemo(() => core.SplitSide[core.SplitSide.old]);
  const currentEnableSyncHeight = solidJs.createMemo(() => props.side === core.SplitSide.new && !!currentIsShow());
  useSyncHeight({
    selector: lineSelector,
    wrapper: lineSelector,
    side: currentSyncHeightSide,
    enable: currentEnableSyncHeight
  });
  return solidJs.createComponent(solidJs.Show, {
    get when() {
      return currentIsShow() || currentIsPureHunk();
    },
    get children() {
      var _el$ = _tmpl$$d();
      `var(${hunkContentBGName})` != null ? _el$.style.setProperty("background-color", `var(${hunkContentBGName})`) : _el$.style.removeProperty("background-color");
      insert(_el$, (() => {
        var _c$ = solidJs.createMemo(() => !!currentShowExpand());
        return () => _c$() ? [(() => {
          var _el$2 = _tmpl$2$9();
          `var(${hunkLineNumberBGName})` != null ? _el$2.style.setProperty("background-color", `var(${hunkLineNumberBGName})`) : _el$2.style.removeProperty("background-color");
          `var(${plainLineNumberColorName})` != null ? _el$2.style.setProperty("color", `var(${plainLineNumberColorName})`) : _el$2.style.removeProperty("color");
          `var(${diffAsideWidthName})` != null ? _el$2.style.setProperty("width", `var(${diffAsideWidthName})`) : _el$2.style.removeProperty("width");
          `var(${diffAsideWidthName})` != null ? _el$2.style.setProperty("min-width", `var(${diffAsideWidthName})`) : _el$2.style.removeProperty("min-width");
          `var(${diffAsideWidthName})` != null ? _el$2.style.setProperty("max-width", `var(${diffAsideWidthName})`) : _el$2.style.removeProperty("max-width");
          insert(_el$2, (() => {
            var _c$2 = solidJs.createMemo(() => !!couldExpand());
            return () => _c$2() ? solidJs.createMemo(() => !!currentIsFirstLine())() ? (() => {
              var _el$5 = _tmpl$4$5();
              _el$5.$$click = () => props.diffFile.onSplitHunkExpand("up", props.index);
              insert(_el$5, solidJs.createComponent(ExpandUp, {
                "class": "fill-current"
              }));
              return _el$5;
            })() : solidJs.createMemo(() => !!currentIsLastLine())() ? (() => {
              var _el$6 = _tmpl$5$5();
              _el$6.$$click = () => props.diffFile.onSplitHunkExpand("down", props.index);
              insert(_el$6, solidJs.createComponent(ExpandDown, {
                "class": "fill-current"
              }));
              return _el$6;
            })() : solidJs.createMemo(() => !!currentShowExpandAll())() ? (() => {
              var _el$7 = _tmpl$6$3();
              _el$7.$$click = () => props.diffFile.onSplitHunkExpand("all", props.index);
              insert(_el$7, solidJs.createComponent(ExpandAll, {
                "class": "fill-current"
              }));
              return _el$7;
            })() : [(() => {
              var _el$8 = _tmpl$7$3();
              _el$8.$$click = () => props.diffFile.onSplitHunkExpand("down", props.index);
              insert(_el$8, solidJs.createComponent(ExpandDown, {
                "class": "fill-current"
              }));
              return _el$8;
            })(), (() => {
              var _el$9 = _tmpl$8$1();
              _el$9.$$click = () => props.diffFile.onSplitHunkExpand("up", props.index);
              insert(_el$9, solidJs.createComponent(ExpandUp, {
                "class": "fill-current"
              }));
              return _el$9;
            })()] : _tmpl$9();
          })());
          return _el$2;
        })(), (() => {
          var _el$3 = _tmpl$3$6(), _el$4 = _el$3.firstChild;
          `var(${hunkContentBGName})` != null ? _el$3.style.setProperty("background-color", `var(${hunkContentBGName})`) : _el$3.style.removeProperty("background-color");
          `var(${hunkContentColorName})` != null ? _el$4.style.setProperty("color", `var(${hunkContentColorName})`) : _el$4.style.removeProperty("color");
          insert(_el$4, () => {
            var _a, _b, _c;
            return ((_b = (_a = currentHunk()) == null ? void 0 : _a.splitInfo) == null ? void 0 : _b.plainText) || ((_c = currentHunk()) == null ? void 0 : _c.text);
          });
          return _el$3;
        })()] : (() => {
          var _el$11 = _tmpl$10();
          `var(${hunkContentBGName})` != null ? _el$11.style.setProperty("background-color", `var(${hunkContentBGName})`) : _el$11.style.removeProperty("background-color");
          return _el$11;
        })();
      })());
      solidJs.createRenderEffect((_p$) => {
        var _v$ = `${props.lineNumber}-hunk`, _v$2 = core.SplitSide[props.side];
        _v$ !== _p$.e && setAttribute(_el$, "data-line", _p$.e = _v$);
        _v$2 !== _p$.t && setAttribute(_el$, "data-side", _p$.t = _v$2);
        return _p$;
      }, {
        e: void 0,
        t: void 0
      });
      return _el$;
    }
  });
};
const DiffSplitHunkLineGitLab$1 = (props) => {
  const currentHunk = solidJs.createMemo(() => props.diffFile.getSplitHunkLine(props.index));
  const enableExpand = solidJs.createMemo(() => props.diffFile.getExpandEnabled());
  const couldExpand = solidJs.createMemo(() => {
    var _a;
    return enableExpand() && ((_a = currentHunk()) == null ? void 0 : _a.splitInfo);
  });
  const lineSelector = solidJs.createMemo(() => `tr[data-line="${props.lineNumber}-hunk"]`);
  const checkCurrentShowExpandAll = () => {
    const hunk = currentHunk();
    return hunk && hunk.splitInfo && hunk.splitInfo.endHiddenIndex - hunk.splitInfo.startHiddenIndex < core.composeLen;
  };
  const [currentShowExpandAll, setCurrentShowExpandAll] = solidJs.createSignal(checkCurrentShowExpandAll());
  const checkCurrentIsShow = () => {
    const hunk = currentHunk();
    return hunk && hunk.splitInfo && hunk.splitInfo.startHiddenIndex < hunk.splitInfo.endHiddenIndex;
  };
  const [currentIsShow, setCurrentIsShow] = solidJs.createSignal(checkCurrentIsShow());
  const currentIsFirstLine = solidJs.createMemo(() => {
    const hunk = currentHunk();
    return hunk && hunk.isFirst;
  });
  const currentIsPureHunk = solidJs.createMemo(() => {
    const hunk = currentHunk();
    return hunk && props.diffFile._getIsPureDiffRender() && !hunk.splitInfo;
  });
  const currentIsLastLine = solidJs.createMemo(() => {
    const hunk = currentHunk();
    return hunk && hunk.isLast;
  });
  const currentSyncHeightSide = solidJs.createMemo(() => core.SplitSide[core.SplitSide.old]);
  const currentEnableSyncHeight = solidJs.createMemo(() => props.side === core.SplitSide.new && !!currentIsShow());
  solidJs.createEffect(() => {
    const init = () => {
      setCurrentIsShow(checkCurrentIsShow);
      setCurrentShowExpandAll(checkCurrentShowExpandAll);
    };
    init();
    const cb = props.diffFile.subscribe(init);
    solidJs.onCleanup(cb);
  });
  useSyncHeight({
    selector: lineSelector,
    wrapper: lineSelector,
    side: currentSyncHeightSide,
    enable: currentEnableSyncHeight
  });
  return solidJs.createComponent(solidJs.Show, {
    get when() {
      return currentIsShow() || currentIsPureHunk();
    },
    get children() {
      var _el$12 = _tmpl$11(), _el$13 = _el$12.firstChild, _el$14 = _el$13.nextSibling, _el$15 = _el$14.firstChild;
      `var(${hunkContentBGName})` != null ? _el$12.style.setProperty("background-color", `var(${hunkContentBGName})`) : _el$12.style.removeProperty("background-color");
      `var(${hunkLineNumberBGName})` != null ? _el$13.style.setProperty("background-color", `var(${hunkLineNumberBGName})`) : _el$13.style.removeProperty("background-color");
      `var(${plainLineNumberColorName})` != null ? _el$13.style.setProperty("color", `var(${plainLineNumberColorName})`) : _el$13.style.removeProperty("color");
      `var(${diffAsideWidthName})` != null ? _el$13.style.setProperty("width", `var(${diffAsideWidthName})`) : _el$13.style.removeProperty("width");
      `var(${diffAsideWidthName})` != null ? _el$13.style.setProperty("min-width", `var(${diffAsideWidthName})`) : _el$13.style.removeProperty("min-width");
      `var(${diffAsideWidthName})` != null ? _el$13.style.setProperty("max-width", `var(${diffAsideWidthName})`) : _el$13.style.removeProperty("max-width");
      insert(_el$13, (() => {
        var _c$3 = solidJs.createMemo(() => !!couldExpand());
        return () => _c$3() ? solidJs.createMemo(() => !!currentIsFirstLine())() ? (() => {
          var _el$16 = _tmpl$4$5();
          _el$16.$$click = () => props.diffFile.onSplitHunkExpand("up", props.index);
          insert(_el$16, solidJs.createComponent(ExpandUp, {
            "class": "fill-current"
          }));
          return _el$16;
        })() : solidJs.createMemo(() => !!currentIsLastLine())() ? (() => {
          var _el$17 = _tmpl$5$5();
          _el$17.$$click = () => props.diffFile.onSplitHunkExpand("down", props.index);
          insert(_el$17, solidJs.createComponent(ExpandDown, {
            "class": "fill-current"
          }));
          return _el$17;
        })() : solidJs.createMemo(() => !!currentShowExpandAll())() ? (() => {
          var _el$18 = _tmpl$6$3();
          _el$18.$$click = () => props.diffFile.onSplitHunkExpand("all", props.index);
          insert(_el$18, solidJs.createComponent(ExpandAll, {
            "class": "fill-current"
          }));
          return _el$18;
        })() : [(() => {
          var _el$19 = _tmpl$7$3();
          _el$19.$$click = () => props.diffFile.onSplitHunkExpand("down", props.index);
          insert(_el$19, solidJs.createComponent(ExpandDown, {
            "class": "fill-current"
          }));
          return _el$19;
        })(), (() => {
          var _el$20 = _tmpl$8$1();
          _el$20.$$click = () => props.diffFile.onSplitHunkExpand("up", props.index);
          insert(_el$20, solidJs.createComponent(ExpandUp, {
            "class": "fill-current"
          }));
          return _el$20;
        })()] : _tmpl$9();
      })());
      `var(${hunkContentBGName})` != null ? _el$14.style.setProperty("background-color", `var(${hunkContentBGName})`) : _el$14.style.removeProperty("background-color");
      `var(${hunkContentColorName})` != null ? _el$15.style.setProperty("color", `var(${hunkContentColorName})`) : _el$15.style.removeProperty("color");
      insert(_el$15, () => {
        var _a, _b, _c;
        return ((_b = (_a = currentHunk()) == null ? void 0 : _a.splitInfo) == null ? void 0 : _b.plainText) || ((_c = currentHunk()) == null ? void 0 : _c.text);
      });
      solidJs.createRenderEffect((_p$) => {
        var _v$3 = `${props.lineNumber}-hunk`, _v$4 = core.SplitSide[props.side];
        _v$3 !== _p$.e && setAttribute(_el$12, "data-line", _p$.e = _v$3);
        _v$4 !== _p$.t && setAttribute(_el$12, "data-side", _p$.t = _v$4);
        return _p$;
      }, {
        e: void 0,
        t: void 0
      });
      return _el$12;
    }
  });
};
const DiffSplitHunkLine$1 = (props) => {
  const mode = useMode();
  return solidJs.createComponent(solidJs.Show, {
    get when() {
      return mode() === exports.DiffModeEnum.SplitGitHub || mode() === exports.DiffModeEnum.Split;
    },
    get fallback() {
      return solidJs.createComponent(DiffSplitHunkLineGitLab$1, {
        get index() {
          return props.index;
        },
        get side() {
          return props.side;
        },
        get diffFile() {
          return props.diffFile;
        },
        get lineNumber() {
          return props.lineNumber;
        }
      });
    },
    get children() {
      return solidJs.createComponent(DiffSplitHunkLineGitHub$1, {
        get index() {
          return props.index;
        },
        get side() {
          return props.side;
        },
        get diffFile() {
          return props.diffFile;
        },
        get lineNumber() {
          return props.lineNumber;
        }
      });
    }
  });
};
delegateEvents(["click"]);
var _tmpl$$c = /* @__PURE__ */ template(`<tr data-state=widget class="diff-line diff-line-widget">`), _tmpl$2$8 = /* @__PURE__ */ template(`<td colspan=2><div class="diff-line-widget-wrapper sticky left-0 z-[1]">`), _tmpl$3$5 = /* @__PURE__ */ template(`<td colspan=2><div>`);
const DiffSplitWidgetLine$1 = (props) => {
  const renderWidget = useRenderWidget();
  const [widget, setWidget] = useDiffWidgetContext() || [];
  const oldLine = solidJs.createMemo(() => props.diffFile.getSplitLeftLine(props.index));
  const newLine = solidJs.createMemo(() => props.diffFile.getSplitRightLine(props.index));
  const oldLineWidget = solidJs.createMemo(() => {
    var _a, _b, _c, _d;
    return !!((_a = oldLine()) == null ? void 0 : _a.lineNumber) && ((_b = widget == null ? void 0 : widget()) == null ? void 0 : _b.side) === core.SplitSide.old && ((_c = widget == null ? void 0 : widget()) == null ? void 0 : _c.lineNumber) === ((_d = oldLine()) == null ? void 0 : _d.lineNumber);
  });
  const newLineWidget = solidJs.createMemo(() => {
    var _a, _b, _c, _d;
    return !!((_a = newLine()) == null ? void 0 : _a.lineNumber) && ((_b = widget == null ? void 0 : widget()) == null ? void 0 : _b.side) === core.SplitSide.new && ((_c = widget == null ? void 0 : widget()) == null ? void 0 : _c.lineNumber) === ((_d = newLine()) == null ? void 0 : _d.lineNumber);
  });
  const currentLine = solidJs.createMemo(() => props.side === core.SplitSide.old ? oldLine() : newLine());
  const currentIsHidden = solidJs.createMemo(() => {
    var _a;
    return (_a = currentLine()) == null ? void 0 : _a.isHidden;
  });
  const lineSelector = solidJs.createMemo(() => `div[data-line="${props.lineNumber}-widget-content"]`);
  const lineWrapperSelector = solidJs.createMemo(() => `tr[data-line="${props.lineNumber}-widget"]`);
  const wrapperSelector = solidJs.createMemo(() => props.side === core.SplitSide.old ? ".old-diff-table-wrapper" : ".new-diff-table-wrapper");
  const currentWidget = solidJs.createMemo(() => props.side === core.SplitSide.old ? oldLineWidget() : newLineWidget());
  const observeSide = solidJs.createMemo(() => core.SplitSide[currentWidget() ? props.side : props.side === core.SplitSide.old ? core.SplitSide.new : core.SplitSide.old]);
  const currentIsShow = solidJs.createMemo(() => (!!oldLineWidget() || !!newLineWidget()) && !currentIsHidden() && !!renderWidget);
  const currentEnable = solidJs.createMemo(() => currentWidget() && !!currentIsShow());
  const onCloseWidget = () => setWidget == null ? void 0 : setWidget({});
  useSyncHeight({
    selector: lineSelector,
    wrapper: lineWrapperSelector,
    side: observeSide,
    enable: currentIsShow
  });
  const width = useDomWidth({
    selector: wrapperSelector,
    enable: currentEnable
  });
  return solidJs.createComponent(solidJs.Show, {
    get when() {
      return currentIsShow();
    },
    get children() {
      var _el$ = _tmpl$$c();
      insert(_el$, (() => {
        var _c$ = solidJs.createMemo(() => !!currentWidget());
        return () => _c$() ? (() => {
          var _el$2 = _tmpl$2$8(), _el$3 = _el$2.firstChild;
          insert(_el$3, (() => {
            var _c$2 = solidJs.createMemo(() => width() > 0);
            return () => {
              var _a, _b;
              return _c$2() && ((_b = renderWidget()) == null ? void 0 : _b({
                diffFile: props.diffFile,
                side: props.side,
                lineNumber: ((_a = currentLine()) == null ? void 0 : _a.lineNumber) || 0,
                onClose: onCloseWidget
              }));
            };
          })());
          solidJs.createRenderEffect((_p$) => {
            var _v$3 = `diff-line-widget-${core.SplitSide[props.side]}-content p-0`, _v$4 = `${props.lineNumber}-widget-content`, _v$5 = core.SplitSide[props.side], _v$6 = width() + "px";
            _v$3 !== _p$.e && className(_el$2, _p$.e = _v$3);
            _v$4 !== _p$.t && setAttribute(_el$3, "data-line", _p$.t = _v$4);
            _v$5 !== _p$.a && setAttribute(_el$3, "data-side", _p$.a = _v$5);
            _v$6 !== _p$.o && ((_p$.o = _v$6) != null ? _el$3.style.setProperty("width", _v$6) : _el$3.style.removeProperty("width"));
            return _p$;
          }, {
            e: void 0,
            t: void 0,
            a: void 0,
            o: void 0
          });
          return _el$2;
        })() : (() => {
          var _el$4 = _tmpl$3$5(), _el$5 = _el$4.firstChild;
          `var(${emptyBGName})` != null ? _el$4.style.setProperty("background-color", `var(${emptyBGName})`) : _el$4.style.removeProperty("background-color");
          solidJs.createRenderEffect((_p$) => {
            var _v$7 = `diff-line-widget-${core.SplitSide[props.side]}-placeholder select-none p-0`, _v$8 = `${props.lineNumber}-widget-content`, _v$9 = core.SplitSide[props.side];
            _v$7 !== _p$.e && className(_el$4, _p$.e = _v$7);
            _v$8 !== _p$.t && setAttribute(_el$5, "data-line", _p$.t = _v$8);
            _v$9 !== _p$.a && setAttribute(_el$5, "data-side", _p$.a = _v$9);
            return _p$;
          }, {
            e: void 0,
            t: void 0,
            a: void 0
          });
          return _el$4;
        })();
      })());
      solidJs.createRenderEffect((_p$) => {
        var _v$ = `${props.lineNumber}-widget`, _v$2 = core.SplitSide[props.side];
        _v$ !== _p$.e && setAttribute(_el$, "data-line", _p$.e = _v$);
        _v$2 !== _p$.t && setAttribute(_el$, "data-side", _p$.t = _v$2);
        return _p$;
      }, {
        e: void 0,
        t: void 0
      });
      return _el$;
    }
  });
};
var _tmpl$$b = /* @__PURE__ */ template(`<table><colgroup><col><col></colgroup><thead class=hidden><tr><th scope=col> line number</th><th scope=col> line content</th></tr></thead><tbody class="diff-table-body leading-[1.4]">`), _tmpl$2$7 = /* @__PURE__ */ template(`<div class="split-diff-view split-diff-view-normal flex w-full basis-[50%]"><style data-select-style></style><div class="old-diff-table-wrapper diff-table-scroll-container w-full overflow-x-auto overflow-y-hidden"></div><div class="diff-split-line w-[1.5px]"></div><div class="new-diff-table-wrapper diff-table-scroll-container w-full overflow-x-auto overflow-y-hidden">`);
const DiffSplitViewTable = (props) => {
  const className$1 = solidJs.createMemo(() => props.side === core.SplitSide.new ? "new-diff-table" : "old-diff-table");
  const getAllLines = () => core.getSplitContentLines(props.diffFile);
  const [lines, setLines] = solidJs.createSignal(getAllLines());
  const selectState = props.selectState;
  solidJs.createEffect(() => {
    const init = () => setLines(getAllLines);
    init();
    const cb = props.diffFile.subscribe(init);
    solidJs.onCleanup(cb);
  });
  const onMouseDown = (e) => {
    var _a, _b;
    let ele = e.target;
    if (ele && (ele == null ? void 0 : ele.nodeName) === "BUTTON") {
      removeAllSelection();
      return;
    }
    while (ele && ele instanceof HTMLElement) {
      const state = ele.getAttribute("data-state");
      if (state) {
        if (state === "extend" || state === "hunk" || state === "widget") {
          if (selectState.current !== void 0) {
            selectState.current = void 0;
            (_a = props.onSelect) == null ? void 0 : _a.call(props, void 0);
            removeAllSelection();
          }
        } else {
          if (selectState.current !== props.side) {
            selectState.current = props.side;
            (_b = props.onSelect) == null ? void 0 : _b.call(props, props.side);
            removeAllSelection();
          }
        }
        return;
      }
      ele = ele.parentElement;
    }
  };
  return (() => {
    var _el$ = _tmpl$$b(), _el$2 = _el$.firstChild, _el$3 = _el$2.firstChild, _el$4 = _el$3.nextSibling, _el$5 = _el$2.nextSibling, _el$6 = _el$5.firstChild, _el$7 = _el$6.firstChild, _el$8 = _el$7.firstChild, _el$9 = _el$7.nextSibling, _el$10 = _el$9.firstChild, _el$11 = _el$5.nextSibling;
    insert(_el$7, () => core.SplitSide[props.side], _el$8);
    insert(_el$9, () => core.SplitSide[props.side], _el$10);
    _el$11.$$mousedown = onMouseDown;
    insert(_el$11, solidJs.createComponent(solidJs.For, {
      get each() {
        return lines();
      },
      children: (item) => [solidJs.createComponent(DiffSplitHunkLine$1, {
        get index() {
          return item.index;
        },
        get side() {
          return props.side;
        },
        get lineNumber() {
          return item.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        }
      }), solidJs.createComponent(DiffSplitContentLine$1, {
        get index() {
          return item.index;
        },
        get side() {
          return props.side;
        },
        get lineNumber() {
          return item.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        }
      }), solidJs.createComponent(DiffSplitWidgetLine$1, {
        get index() {
          return item.index;
        },
        get side() {
          return props.side;
        },
        get lineNumber() {
          return item.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        }
      }), solidJs.createComponent(DiffSplitExtendLine$1, {
        get index() {
          return item.index;
        },
        get side() {
          return props.side;
        },
        get lineNumber() {
          return item.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        }
      })]
    }), null);
    insert(_el$11, solidJs.createComponent(DiffSplitHunkLine$1, {
      get side() {
        return props.side;
      },
      get index() {
        return props.diffFile.splitLineLength;
      },
      get lineNumber() {
        return props.diffFile.splitLineLength;
      },
      get diffFile() {
        return props.diffFile;
      }
    }), null);
    solidJs.createRenderEffect((_p$) => {
      var _v$ = `${className$1()} w-full border-collapse border-spacing-0`, _v$2 = core.SplitSide[props.side], _v$3 = `diff-table-${core.SplitSide[props.side]}-num-col`, _v$4 = `diff-table-${core.SplitSide[props.side]}-content-col`;
      _v$ !== _p$.e && className(_el$, _p$.e = _v$);
      _v$2 !== _p$.t && setAttribute(_el$, "data-mode", _p$.t = _v$2);
      _v$3 !== _p$.a && className(_el$3, _p$.a = _v$3);
      _v$4 !== _p$.o && className(_el$4, _p$.o = _v$4);
      return _p$;
    }, {
      e: void 0,
      t: void 0,
      a: void 0,
      o: void 0
    });
    return _el$;
  })();
};
const DiffSplitViewNormal = (props) => {
  const isMounted = useIsMounted();
  const [ref1, setRef1] = solidJs.createSignal(null);
  const [ref2, setRef2] = solidJs.createSignal(null);
  const [styleRef, setStyleRef] = solidJs.createSignal(null);
  const maxText = solidJs.createMemo(() => Math.max(props.diffFile.fileLineLength, props.diffFile.splitLineLength).toString());
  const initSyncScroll = () => {
    if (!isMounted()) return;
    const left = ref1();
    const right = ref2();
    if (!left || !right) return;
    const clean = syncScroll(left, right);
    solidJs.onCleanup(clean);
  };
  solidJs.createEffect(initSyncScroll);
  const selectState = {
    current: void 0
  };
  const onSelect = (side) => {
    const ele = styleRef();
    if (!ele) return;
    if (!side) {
      ele.textContent = "";
    } else {
      ele.textContent = `#${getId()} [data-state="extend"] {user-select: none} 
#${getId()} [data-state="hunk"] {user-select: none} 
#${getId()} [data-state="widget"] {user-select: none}`;
    }
  };
  const fontSize = useFontSize();
  const font = solidJs.createMemo(() => ({
    fontSize: `${fontSize() || 14}px`,
    fontFamily: "Menlo, Consolas, monospace"
  }));
  const width = useTextWidth({
    text: maxText,
    font
  });
  const computedWidth = solidJs.createMemo(() => Math.max(40, width() + 25));
  const getId = () => `diff-split-view-${props.diffFile.getId()}`;
  return (() => {
    var _el$12 = _tmpl$2$7(), _el$13 = _el$12.firstChild, _el$14 = _el$13.nextSibling, _el$15 = _el$14.nextSibling, _el$16 = _el$15.nextSibling;
    use(setStyleRef, _el$13);
    use(setRef1, _el$14);
    _el$14.style.setProperty("overscroll-behavior-x", "none");
    _el$14.style.setProperty("font-family", "Menlo, Consolas, monospace");
    `var(${diffFontSizeName})` != null ? _el$14.style.setProperty("font-size", `var(${diffFontSizeName})`) : _el$14.style.removeProperty("font-size");
    insert(_el$14, solidJs.createComponent(DiffSplitViewTable, {
      get side() {
        return core.SplitSide.old;
      },
      get diffFile() {
        return props.diffFile;
      },
      onSelect,
      selectState
    }));
    `var(${borderColorName})` != null ? _el$15.style.setProperty("background-color", `var(${borderColorName})`) : _el$15.style.removeProperty("background-color");
    use(setRef2, _el$16);
    _el$16.style.setProperty("overscroll-behavior-x", "none");
    _el$16.style.setProperty("font-family", "Menlo, Consolas, monospace");
    `var(${diffFontSizeName})` != null ? _el$16.style.setProperty("font-size", `var(${diffFontSizeName})`) : _el$16.style.removeProperty("font-size");
    insert(_el$16, solidJs.createComponent(DiffSplitViewTable, {
      get side() {
        return core.SplitSide.new;
      },
      get diffFile() {
        return props.diffFile;
      },
      onSelect,
      selectState
    }));
    solidJs.createRenderEffect((_p$) => {
      var _v$5 = {
        [diffAsideWidthName]: `${Math.round(computedWidth())}px`
      }, _v$6 = {
        [diffAsideWidthName]: `${Math.round(computedWidth())}px`
      };
      _p$.e = style(_el$14, _v$5, _p$.e);
      _p$.t = style(_el$16, _v$6, _p$.t);
      return _p$;
    }, {
      e: void 0,
      t: void 0
    });
    return _el$12;
  })();
};
delegateEvents(["mousedown"]);
var _tmpl$$a = /* @__PURE__ */ template(`<tr class=diff-line>`), _tmpl$2$6 = /* @__PURE__ */ template(`<td class="diff-line-old-num group relative w-[1%] min-w-[40px] select-none pl-[10px] pr-[10px] text-right align-top"><span>`), _tmpl$3$4 = /* @__PURE__ */ template(`<td class="diff-line-old-content group relative pr-[10px] align-top">`), _tmpl$4$4 = /* @__PURE__ */ template(`<td class="diff-line-old-placeholder select-none"colspan=2><span>&ensp;`), _tmpl$5$4 = /* @__PURE__ */ template(`<td class="diff-line-new-num group relative w-[1%] min-w-[40px] select-none border-l-[1px] pl-[10px] pr-[10px] text-right align-top"><span>`), _tmpl$6$2 = /* @__PURE__ */ template(`<td class="diff-line-new-content group relative pr-[10px] align-top">`), _tmpl$7$2 = /* @__PURE__ */ template(`<td class="diff-line-new-placeholder select-none border-l-[1px]"colspan=2><span>&ensp;`);
const DiffSplitContentLine = (props) => {
  const [_, setWidget] = useDiffWidgetContext() || [];
  const enableAddWidget = useEnableAddWidget();
  const enableHighlight = useEnableHighlight();
  const onAddWidgetClick = useOnAddWidgetClick();
  const oldLine = solidJs.createMemo(() => props.diffFile.getSplitLeftLine(props.index));
  const newLine = solidJs.createMemo(() => props.diffFile.getSplitRightLine(props.index));
  const getOldSyntaxLine = () => {
    var _a;
    return props.diffFile.getOldSyntaxLine(((_a = oldLine()) == null ? void 0 : _a.lineNumber) || 0);
  };
  const getNewSyntaxLine = () => {
    var _a;
    return props.diffFile.getNewSyntaxLine(((_a = newLine()) == null ? void 0 : _a.lineNumber) || 0);
  };
  const [oldSyntaxLine, setOldSyntaxLine] = solidJs.createSignal(getOldSyntaxLine());
  const [newSyntaxLine, setNewSyntaxLine] = solidJs.createSignal(getNewSyntaxLine());
  const getOldPlainLine = () => {
    var _a;
    return props.diffFile.getOldPlainLine(((_a = oldLine()) == null ? void 0 : _a.lineNumber) || 0);
  };
  const getNewPlainLine = () => {
    var _a;
    return props.diffFile.getNewPlainLine(((_a = newLine()) == null ? void 0 : _a.lineNumber) || 0);
  };
  const [oldPlainLine, setOldPlainLine] = solidJs.createSignal(getOldPlainLine());
  const [newPlainLine, setNewPlainLine] = solidJs.createSignal(getNewPlainLine());
  const checkHasDiff = () => {
    var _a, _b;
    return !!((_a = oldLine()) == null ? void 0 : _a.diff) || !!((_b = newLine()) == null ? void 0 : _b.diff);
  };
  const hasDiff = solidJs.createMemo(checkHasDiff);
  const checkHasChange = () => {
    var _a, _b;
    return core.checkDiffLineIncludeChange((_a = oldLine()) == null ? void 0 : _a.diff) || core.checkDiffLineIncludeChange((_b = newLine()) == null ? void 0 : _b.diff);
  };
  const hasChange = solidJs.createMemo(checkHasChange);
  const checkHasHidden = () => {
    var _a, _b;
    return ((_a = oldLine()) == null ? void 0 : _a.isHidden) && ((_b = newLine()) == null ? void 0 : _b.isHidden);
  };
  const hasHidden = solidJs.createMemo(checkHasHidden);
  const oldLineIsDelete = () => {
    var _a, _b;
    return ((_b = (_a = oldLine()) == null ? void 0 : _a.diff) == null ? void 0 : _b.type) === core.DiffLineType.Delete;
  };
  const newLineIsAdded = () => {
    var _a, _b;
    return ((_b = (_a = newLine()) == null ? void 0 : _a.diff) == null ? void 0 : _b.type) === core.DiffLineType.Add;
  };
  solidJs.createEffect(() => {
    const init = () => {
      setOldSyntaxLine(getOldSyntaxLine);
      setNewSyntaxLine(getNewSyntaxLine);
      setOldPlainLine(getOldPlainLine);
      setNewPlainLine(getNewPlainLine);
    };
    init();
    const cb = props.diffFile.subscribe(init);
    solidJs.onCleanup(cb);
  });
  const onOpenAddWidget = (lineNumber, side) => setWidget == null ? void 0 : setWidget({
    side,
    lineNumber
  });
  return solidJs.createComponent(solidJs.Show, {
    get when() {
      return !hasHidden();
    },
    get children() {
      var _el$ = _tmpl$$a();
      insert(_el$, (() => {
        var _c$ = solidJs.createMemo(() => {
          var _a;
          return !!((_a = oldLine()) == null ? void 0 : _a.lineNumber);
        });
        return () => _c$() ? [(() => {
          var _el$2 = _tmpl$2$6(), _el$3 = _el$2.firstChild;
          insert(_el$2, (() => {
            var _c$3 = solidJs.createMemo(() => !!(hasDiff() && enableAddWidget()));
            return () => _c$3() && solidJs.createComponent(DiffSplitAddWidget, {
              get index() {
                return props.index;
              },
              get lineNumber() {
                var _a;
                return ((_a = oldLine()) == null ? void 0 : _a.lineNumber) || 0;
              },
              get side() {
                return core.SplitSide.old;
              },
              get diffFile() {
                return props.diffFile;
              },
              onWidgetClick: onAddWidgetClick,
              "class": "absolute left-[100%] z-[1] translate-x-[-50%]",
              onOpenAddWidget
            });
          })(), _el$3);
          insert(_el$3, () => {
            var _a;
            return (_a = oldLine()) == null ? void 0 : _a.lineNumber;
          });
          solidJs.createRenderEffect((_p$) => {
            var _a;
            var _v$3 = getLineNumberBG(false, oldLineIsDelete(), hasDiff()), _v$4 = `var(${hasDiff() ? plainLineNumberColorName : expandLineNumberColorName})`, _v$5 = (_a = oldLine()) == null ? void 0 : _a.lineNumber, _v$6 = hasChange() ? void 0 : 0.5;
            _v$3 !== _p$.e && ((_p$.e = _v$3) != null ? _el$2.style.setProperty("background-color", _v$3) : _el$2.style.removeProperty("background-color"));
            _v$4 !== _p$.t && ((_p$.t = _v$4) != null ? _el$2.style.setProperty("color", _v$4) : _el$2.style.removeProperty("color"));
            _v$5 !== _p$.a && setAttribute(_el$3, "data-line-num", _p$.a = _v$5);
            _v$6 !== _p$.o && ((_p$.o = _v$6) != null ? _el$3.style.setProperty("opacity", _v$6) : _el$3.style.removeProperty("opacity"));
            return _p$;
          }, {
            e: void 0,
            t: void 0,
            a: void 0,
            o: void 0
          });
          return _el$2;
        })(), (() => {
          var _el$4 = _tmpl$3$4();
          insert(_el$4, (() => {
            var _c$4 = solidJs.createMemo(() => !!(hasDiff() && enableAddWidget()));
            return () => _c$4() && solidJs.createComponent(DiffSplitAddWidget, {
              get index() {
                return props.index;
              },
              get lineNumber() {
                var _a;
                return ((_a = oldLine()) == null ? void 0 : _a.lineNumber) || 0;
              },
              get side() {
                return core.SplitSide.old;
              },
              get diffFile() {
                return props.diffFile;
              },
              onWidgetClick: onAddWidgetClick,
              "class": "absolute right-[100%] z-[1] translate-x-[50%]",
              onOpenAddWidget
            });
          })(), null);
          insert(_el$4, solidJs.createComponent(DiffContent, {
            enableWrap: true,
            get diffFile() {
              return props.diffFile;
            },
            get rawLine() {
              var _a;
              return ((_a = oldLine()) == null ? void 0 : _a.value) || "";
            },
            get diffLine() {
              var _a;
              return (_a = oldLine()) == null ? void 0 : _a.diff;
            },
            get plainLine() {
              return oldPlainLine();
            },
            get syntaxLine() {
              return oldSyntaxLine();
            },
            get enableHighlight() {
              return !!enableHighlight();
            }
          }), null);
          solidJs.createRenderEffect((_p$) => {
            var _v$7 = getContentBG(false, oldLineIsDelete(), hasDiff()), _v$8 = core.SplitSide[core.SplitSide.old];
            _v$7 !== _p$.e && ((_p$.e = _v$7) != null ? _el$4.style.setProperty("background-color", _v$7) : _el$4.style.removeProperty("background-color"));
            _v$8 !== _p$.t && setAttribute(_el$4, "data-side", _p$.t = _v$8);
            return _p$;
          }, {
            e: void 0,
            t: void 0
          });
          return _el$4;
        })()] : (() => {
          var _el$5 = _tmpl$4$4();
          `var(${emptyBGName})` != null ? _el$5.style.setProperty("background-color", `var(${emptyBGName})`) : _el$5.style.removeProperty("background-color");
          return _el$5;
        })();
      })(), null);
      insert(_el$, (() => {
        var _c$2 = solidJs.createMemo(() => {
          var _a;
          return !!((_a = newLine()) == null ? void 0 : _a.lineNumber);
        });
        return () => _c$2() ? [(() => {
          var _el$6 = _tmpl$5$4(), _el$7 = _el$6.firstChild;
          `var(${borderColorName})` != null ? _el$6.style.setProperty("border-left-color", `var(${borderColorName})`) : _el$6.style.removeProperty("border-left-color");
          _el$6.style.setProperty("border-left-style", "solid");
          insert(_el$6, (() => {
            var _c$5 = solidJs.createMemo(() => !!(hasDiff() && enableAddWidget()));
            return () => _c$5() && solidJs.createComponent(DiffSplitAddWidget, {
              get index() {
                return props.index;
              },
              get lineNumber() {
                var _a;
                return ((_a = newLine()) == null ? void 0 : _a.lineNumber) || 0;
              },
              get side() {
                return core.SplitSide.new;
              },
              get diffFile() {
                return props.diffFile;
              },
              onWidgetClick: onAddWidgetClick,
              "class": "absolute left-[100%] z-[1] translate-x-[-50%]",
              onOpenAddWidget
            });
          })(), _el$7);
          insert(_el$7, () => {
            var _a;
            return (_a = newLine()) == null ? void 0 : _a.lineNumber;
          });
          solidJs.createRenderEffect((_p$) => {
            var _a;
            var _v$9 = getLineNumberBG(newLineIsAdded(), false, hasDiff()), _v$10 = `var(${hasDiff() ? plainLineNumberColorName : expandLineNumberColorName})`, _v$11 = (_a = newLine()) == null ? void 0 : _a.lineNumber, _v$12 = hasChange() ? void 0 : 0.5;
            _v$9 !== _p$.e && ((_p$.e = _v$9) != null ? _el$6.style.setProperty("background-color", _v$9) : _el$6.style.removeProperty("background-color"));
            _v$10 !== _p$.t && ((_p$.t = _v$10) != null ? _el$6.style.setProperty("color", _v$10) : _el$6.style.removeProperty("color"));
            _v$11 !== _p$.a && setAttribute(_el$7, "data-line-num", _p$.a = _v$11);
            _v$12 !== _p$.o && ((_p$.o = _v$12) != null ? _el$7.style.setProperty("opacity", _v$12) : _el$7.style.removeProperty("opacity"));
            return _p$;
          }, {
            e: void 0,
            t: void 0,
            a: void 0,
            o: void 0
          });
          return _el$6;
        })(), (() => {
          var _el$8 = _tmpl$6$2();
          insert(_el$8, (() => {
            var _c$6 = solidJs.createMemo(() => !!(hasDiff() && enableAddWidget()));
            return () => _c$6() && solidJs.createComponent(DiffSplitAddWidget, {
              get index() {
                return props.index;
              },
              get lineNumber() {
                var _a;
                return ((_a = newLine()) == null ? void 0 : _a.lineNumber) || 0;
              },
              get side() {
                return core.SplitSide.new;
              },
              get diffFile() {
                return props.diffFile;
              },
              onWidgetClick: onAddWidgetClick,
              "class": "absolute right-[100%] z-[1] translate-x-[50%]",
              onOpenAddWidget
            });
          })(), null);
          insert(_el$8, solidJs.createComponent(DiffContent, {
            enableWrap: true,
            get diffFile() {
              return props.diffFile;
            },
            get rawLine() {
              var _a;
              return ((_a = newLine()) == null ? void 0 : _a.value) || "";
            },
            get diffLine() {
              var _a;
              return (_a = newLine()) == null ? void 0 : _a.diff;
            },
            get plainLine() {
              return newPlainLine();
            },
            get syntaxLine() {
              return newSyntaxLine();
            },
            get enableHighlight() {
              return !!enableHighlight();
            }
          }), null);
          solidJs.createRenderEffect((_p$) => {
            var _v$13 = getContentBG(newLineIsAdded(), false, hasDiff()), _v$14 = core.SplitSide[core.SplitSide.new];
            _v$13 !== _p$.e && ((_p$.e = _v$13) != null ? _el$8.style.setProperty("background-color", _v$13) : _el$8.style.removeProperty("background-color"));
            _v$14 !== _p$.t && setAttribute(_el$8, "data-side", _p$.t = _v$14);
            return _p$;
          }, {
            e: void 0,
            t: void 0
          });
          return _el$8;
        })()] : (() => {
          var _el$9 = _tmpl$7$2();
          `var(${emptyBGName})` != null ? _el$9.style.setProperty("background-color", `var(${emptyBGName})`) : _el$9.style.removeProperty("background-color");
          `var(${borderColorName})` != null ? _el$9.style.setProperty("border-left-color", `var(${borderColorName})`) : _el$9.style.removeProperty("border-left-color");
          _el$9.style.setProperty("border-left-style", "solid");
          return _el$9;
        })();
      })(), null);
      solidJs.createRenderEffect((_p$) => {
        var _v$ = props.lineNumber, _v$2 = hasDiff() ? "diff" : "plain";
        _v$ !== _p$.e && setAttribute(_el$, "data-line", _p$.e = _v$);
        _v$2 !== _p$.t && setAttribute(_el$, "data-state", _p$.t = _v$2);
        return _p$;
      }, {
        e: void 0,
        t: void 0
      });
      return _el$;
    }
  });
};
var _tmpl$$9 = /* @__PURE__ */ template(`<tr data-state=extend class="diff-line diff-line-extend">`), _tmpl$2$5 = /* @__PURE__ */ template(`<td class="diff-line-extend-old-content p-0"colspan=2><div class=diff-line-extend-wrapper>`), _tmpl$3$3 = /* @__PURE__ */ template(`<td class="diff-line-extend-old-placeholder select-none p-0"colspan=2>`), _tmpl$4$3 = /* @__PURE__ */ template(`<td class="diff-line-extend-new-content border-l-[1px] p-0"colspan=2><div class=diff-line-extend-wrapper>`), _tmpl$5$3 = /* @__PURE__ */ template(`<td class="diff-line-extend-new-placeholder select-none border-l-[1px] p-0"colspan=2>`);
const DiffSplitExtendLine = (props) => {
  const extendData = useExtendData();
  const renderExtend = useRenderExtend();
  const oldLine = solidJs.createMemo(() => props.diffFile.getSplitLeftLine(props.index));
  const newLine = solidJs.createMemo(() => props.diffFile.getSplitRightLine(props.index));
  const enableExpand = solidJs.createMemo(() => props.diffFile.getExpandEnabled());
  const oldLineExtend = solidJs.createMemo(() => {
    var _a, _b, _c;
    return (_c = (_a = extendData()) == null ? void 0 : _a.oldFile) == null ? void 0 : _c[((_b = oldLine()) == null ? void 0 : _b.lineNumber) || ""];
  });
  const newLineExtend = solidJs.createMemo(() => {
    var _a, _b, _c;
    return (_c = (_a = extendData()) == null ? void 0 : _a.newFile) == null ? void 0 : _c[((_b = newLine()) == null ? void 0 : _b.lineNumber) || ""];
  });
  const checkIsShow = () => {
    const oldExtend = oldLineExtend();
    const newExtend = newLineExtend();
    const oldLineData = oldLine();
    const newLineData = newLine();
    const enableExpandValue = enableExpand();
    const renderExtendValue = renderExtend();
    return (oldExtend || newExtend) && (!(oldLineData == null ? void 0 : oldLineData.isHidden) && !(newLineData == null ? void 0 : newLineData.isHidden) || enableExpandValue) && renderExtendValue;
  };
  const currentIsShow = solidJs.createMemo(checkIsShow);
  return solidJs.createComponent(solidJs.Show, {
    get when() {
      return currentIsShow();
    },
    get children() {
      var _el$ = _tmpl$$9();
      insert(_el$, (() => {
        var _c$ = solidJs.createMemo(() => !!(renderExtend() && oldLineExtend()));
        return () => _c$() ? (() => {
          var _el$2 = _tmpl$2$5(), _el$3 = _el$2.firstChild;
          insert(_el$3, () => {
            var _a, _b, _c;
            return (_c = renderExtend()) == null ? void 0 : _c({
              diffFile: props.diffFile,
              side: core.SplitSide.old,
              lineNumber: ((_a = oldLine()) == null ? void 0 : _a.lineNumber) || 0,
              data: (_b = oldLineExtend()) == null ? void 0 : _b.data,
              onUpdate: props.diffFile.notifyAll
            });
          });
          return _el$2;
        })() : (() => {
          var _el$4 = _tmpl$3$3();
          `var(${emptyBGName})` != null ? _el$4.style.setProperty("background-color", `var(${emptyBGName})`) : _el$4.style.removeProperty("background-color");
          return _el$4;
        })();
      })(), null);
      insert(_el$, (() => {
        var _c$2 = solidJs.createMemo(() => !!(renderExtend() && newLineExtend()));
        return () => _c$2() ? (() => {
          var _el$5 = _tmpl$4$3(), _el$6 = _el$5.firstChild;
          `var(${borderColorName})` != null ? _el$5.style.setProperty("border-left-color", `var(${borderColorName})`) : _el$5.style.removeProperty("border-left-color");
          _el$5.style.setProperty("border-left-style", "solid");
          insert(_el$6, () => {
            var _a, _b, _c;
            return (_c = renderExtend()) == null ? void 0 : _c({
              diffFile: props.diffFile,
              side: core.SplitSide.new,
              lineNumber: ((_a = newLine()) == null ? void 0 : _a.lineNumber) || 0,
              data: (_b = newLineExtend()) == null ? void 0 : _b.data,
              onUpdate: props.diffFile.notifyAll
            });
          });
          return _el$5;
        })() : (() => {
          var _el$7 = _tmpl$5$3();
          `var(${emptyBGName})` != null ? _el$7.style.setProperty("background-color", `var(${emptyBGName})`) : _el$7.style.removeProperty("background-color");
          `var(${borderColorName})` != null ? _el$7.style.setProperty("border-left-color", `var(${borderColorName})`) : _el$7.style.removeProperty("border-left-color");
          _el$7.style.setProperty("border-left-style", "solid");
          return _el$7;
        })();
      })(), null);
      solidJs.createRenderEffect(() => setAttribute(_el$, "data-line", `${props.lineNumber}-extend`));
      return _el$;
    }
  });
};
var _tmpl$$8 = /* @__PURE__ */ template(`<tr data-state=hunk class="diff-line diff-line-hunk"><td class="diff-line-hunk-action relative w-[1%] min-w-[40px] select-none p-[1px]"></td><td class="diff-line-hunk-content pr-[10px] align-middle"colspan=3><div class=pl-[1.5em]>`), _tmpl$2$4 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[6px]"title="Expand Up"data-title="Expand Up">`), _tmpl$3$2 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[6px]"title="Expand Down"data-title="Expand Down">`), _tmpl$4$2 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[6px]"title="Expand All"data-title="Expand All">`), _tmpl$5$2 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[2px]"title="Expand Down"data-title="Expand Down">`), _tmpl$6$1 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[2px]"title="Expand Up"data-title="Expand Up">`), _tmpl$7$1 = /* @__PURE__ */ template(`<div class=min-h-[28px]>&ensp;`), _tmpl$8 = /* @__PURE__ */ template(`<tr data-state=hunk class="diff-line diff-line-hunk"><td class="diff-line-hunk-action relative w-[1%] min-w-[40px] select-none p-[1px]"></td><td class="diff-line-hunk-content pr-[10px] align-middle"><div class=pl-[1.5em]></div></td><td class="diff-line-hunk-action relative z-[1] w-[1%] min-w-[40px] select-none border-l-[1px] p-[1px]"></td><td class="diff-line-hunk-content relative pr-[10px] align-middle"><div class=pl-[1.5em]>`);
const DiffSplitHunkLineGitHub = (props) => {
  const currentHunk = solidJs.createMemo(() => props.diffFile.getSplitHunkLine(props.index));
  const enableExpand = solidJs.createMemo(() => props.diffFile.getExpandEnabled());
  const couldExpand = solidJs.createMemo(() => {
    var _a;
    return enableExpand() && ((_a = currentHunk()) == null ? void 0 : _a.splitInfo);
  });
  const checkCurrentShowExpandAll = () => {
    const hunk = currentHunk();
    return hunk && hunk.splitInfo && hunk.splitInfo.endHiddenIndex - hunk.splitInfo.startHiddenIndex < core.composeLen;
  };
  const [currentShowExpandAll, setCurrentShowExpandAll] = solidJs.createSignal(checkCurrentShowExpandAll());
  const checkCurrentIsShow = () => {
    const hunk = currentHunk();
    return hunk && hunk.splitInfo && hunk.splitInfo.startHiddenIndex < hunk.splitInfo.endHiddenIndex;
  };
  const [currentIsShow, setCurrentIsShow] = solidJs.createSignal(checkCurrentIsShow());
  const currentIsFirstLine = solidJs.createMemo(() => {
    const hunk = currentHunk();
    return hunk && hunk.isFirst;
  });
  const currentIsPureHunk = solidJs.createMemo(() => {
    const hunk = currentHunk();
    return hunk && props.diffFile._getIsPureDiffRender() && !hunk.splitInfo;
  });
  const currentIsLastLine = solidJs.createMemo(() => {
    const hunk = currentHunk();
    return hunk && hunk.isLast;
  });
  solidJs.createEffect(() => {
    const init = () => {
      setCurrentIsShow(checkCurrentIsShow);
      setCurrentShowExpandAll(checkCurrentShowExpandAll);
    };
    init();
    const cb = props.diffFile.subscribe(init);
    solidJs.onCleanup(cb);
  });
  return solidJs.createComponent(solidJs.Show, {
    get when() {
      return currentIsShow() || currentIsPureHunk();
    },
    get children() {
      var _el$ = _tmpl$$8(), _el$2 = _el$.firstChild, _el$3 = _el$2.nextSibling, _el$4 = _el$3.firstChild;
      `var(${hunkLineNumberBGName})` != null ? _el$2.style.setProperty("background-color", `var(${hunkLineNumberBGName})`) : _el$2.style.removeProperty("background-color");
      `var(${plainLineNumberColorName})` != null ? _el$2.style.setProperty("color", `var(${plainLineNumberColorName})`) : _el$2.style.removeProperty("color");
      insert(_el$2, (() => {
        var _c$ = solidJs.createMemo(() => !!couldExpand());
        return () => _c$() ? solidJs.createMemo(() => !!currentIsFirstLine())() ? (() => {
          var _el$5 = _tmpl$2$4();
          _el$5.$$click = () => props.diffFile.onSplitHunkExpand("up", props.index);
          insert(_el$5, solidJs.createComponent(ExpandUp, {
            "class": "fill-current"
          }));
          return _el$5;
        })() : solidJs.createMemo(() => !!currentIsLastLine())() ? (() => {
          var _el$6 = _tmpl$3$2();
          _el$6.$$click = () => props.diffFile.onSplitHunkExpand("down", props.index);
          insert(_el$6, solidJs.createComponent(ExpandDown, {
            "class": "fill-current"
          }));
          return _el$6;
        })() : solidJs.createMemo(() => !!currentShowExpandAll())() ? (() => {
          var _el$7 = _tmpl$4$2();
          _el$7.$$click = () => props.diffFile.onSplitHunkExpand("all", props.index);
          insert(_el$7, solidJs.createComponent(ExpandAll, {
            "class": "fill-current"
          }));
          return _el$7;
        })() : [(() => {
          var _el$8 = _tmpl$5$2();
          _el$8.$$click = () => props.diffFile.onSplitHunkExpand("down", props.index);
          insert(_el$8, solidJs.createComponent(ExpandDown, {
            "class": "fill-current"
          }));
          return _el$8;
        })(), (() => {
          var _el$9 = _tmpl$6$1();
          _el$9.$$click = () => props.diffFile.onSplitHunkExpand("up", props.index);
          insert(_el$9, solidJs.createComponent(ExpandUp, {
            "class": "fill-current"
          }));
          return _el$9;
        })()] : _tmpl$7$1();
      })());
      `var(${hunkContentBGName})` != null ? _el$3.style.setProperty("background-color", `var(${hunkContentBGName})`) : _el$3.style.removeProperty("background-color");
      `var(${hunkContentColorName})` != null ? _el$4.style.setProperty("color", `var(${hunkContentColorName})`) : _el$4.style.removeProperty("color");
      insert(_el$4, () => {
        var _a, _b, _c;
        return ((_b = (_a = currentHunk()) == null ? void 0 : _a.splitInfo) == null ? void 0 : _b.plainText) || ((_c = currentHunk()) == null ? void 0 : _c.text);
      });
      solidJs.createRenderEffect(() => setAttribute(_el$, "data-line", `${props.lineNumber}-hunk`));
      return _el$;
    }
  });
};
const DiffSplitHunkLineGitLab = (props) => {
  const currentHunk = solidJs.createMemo(() => props.diffFile.getSplitHunkLine(props.index));
  const enableExpand = solidJs.createMemo(() => props.diffFile.getExpandEnabled());
  const couldExpand = solidJs.createMemo(() => {
    var _a;
    return enableExpand() && ((_a = currentHunk()) == null ? void 0 : _a.splitInfo);
  });
  const checkCurrentShowExpandAll = () => {
    const hunk = currentHunk();
    return hunk && hunk.splitInfo && hunk.splitInfo.endHiddenIndex - hunk.splitInfo.startHiddenIndex < core.composeLen;
  };
  const [currentShowExpandAll, setCurrentShowExpandAll] = solidJs.createSignal(checkCurrentShowExpandAll());
  const checkCurrentIsShow = () => {
    const hunk = currentHunk();
    return hunk && hunk.splitInfo && hunk.splitInfo.startHiddenIndex < hunk.splitInfo.endHiddenIndex;
  };
  const [currentIsShow, setCurrentIsShow] = solidJs.createSignal(checkCurrentIsShow());
  const currentIsFirstLine = solidJs.createMemo(() => {
    const hunk = currentHunk();
    return hunk && hunk.isFirst;
  });
  const currentIsPureHunk = solidJs.createMemo(() => {
    const hunk = currentHunk();
    return hunk && props.diffFile._getIsPureDiffRender() && !hunk.splitInfo;
  });
  const currentIsLastLine = solidJs.createMemo(() => {
    const hunk = currentHunk();
    return hunk && hunk.isLast;
  });
  solidJs.createEffect(() => {
    const init = () => {
      setCurrentShowExpandAll(checkCurrentShowExpandAll());
      setCurrentIsShow(checkCurrentIsShow());
    };
    init();
    const cb = props.diffFile.subscribe(init);
    solidJs.onCleanup(cb);
  });
  return solidJs.createComponent(solidJs.Show, {
    get when() {
      return currentIsShow() || currentIsPureHunk();
    },
    get children() {
      var _el$11 = _tmpl$8(), _el$12 = _el$11.firstChild, _el$13 = _el$12.nextSibling, _el$14 = _el$13.firstChild, _el$15 = _el$13.nextSibling, _el$16 = _el$15.nextSibling, _el$17 = _el$16.firstChild;
      `var(${hunkLineNumberBGName})` != null ? _el$12.style.setProperty("background-color", `var(${hunkLineNumberBGName})`) : _el$12.style.removeProperty("background-color");
      `var(${plainLineNumberColorName})` != null ? _el$12.style.setProperty("color", `var(${plainLineNumberColorName})`) : _el$12.style.removeProperty("color");
      insert(_el$12, (() => {
        var _c$2 = solidJs.createMemo(() => !!couldExpand());
        return () => _c$2() ? solidJs.createMemo(() => !!currentIsFirstLine())() ? (() => {
          var _el$18 = _tmpl$2$4();
          _el$18.$$click = () => props.diffFile.onSplitHunkExpand("up", props.index);
          insert(_el$18, solidJs.createComponent(ExpandUp, {
            "class": "fill-current"
          }));
          return _el$18;
        })() : solidJs.createMemo(() => !!currentIsLastLine())() ? (() => {
          var _el$19 = _tmpl$3$2();
          _el$19.$$click = () => props.diffFile.onSplitHunkExpand("down", props.index);
          insert(_el$19, solidJs.createComponent(ExpandDown, {
            "class": "fill-current"
          }));
          return _el$19;
        })() : solidJs.createMemo(() => !!currentShowExpandAll())() ? (() => {
          var _el$20 = _tmpl$4$2();
          _el$20.$$click = () => props.diffFile.onSplitHunkExpand("all", props.index);
          insert(_el$20, solidJs.createComponent(ExpandAll, {
            "class": "fill-current"
          }));
          return _el$20;
        })() : [(() => {
          var _el$21 = _tmpl$5$2();
          _el$21.$$click = () => props.diffFile.onSplitHunkExpand("down", props.index);
          insert(_el$21, solidJs.createComponent(ExpandDown, {
            "class": "fill-current"
          }));
          return _el$21;
        })(), (() => {
          var _el$22 = _tmpl$6$1();
          _el$22.$$click = () => props.diffFile.onSplitHunkExpand("up", props.index);
          insert(_el$22, solidJs.createComponent(ExpandUp, {
            "class": "fill-current"
          }));
          return _el$22;
        })()] : _tmpl$7$1();
      })());
      `var(${hunkContentBGName})` != null ? _el$13.style.setProperty("background-color", `var(${hunkContentBGName})`) : _el$13.style.removeProperty("background-color");
      `var(${hunkContentColorName})` != null ? _el$14.style.setProperty("color", `var(${hunkContentColorName})`) : _el$14.style.removeProperty("color");
      insert(_el$14, () => {
        var _a, _b, _c;
        return ((_b = (_a = currentHunk()) == null ? void 0 : _a.splitInfo) == null ? void 0 : _b.plainText) || ((_c = currentHunk()) == null ? void 0 : _c.text);
      });
      `var(${hunkLineNumberBGName})` != null ? _el$15.style.setProperty("background-color", `var(${hunkLineNumberBGName})`) : _el$15.style.removeProperty("background-color");
      `var(${plainLineNumberColorName})` != null ? _el$15.style.setProperty("color", `var(${plainLineNumberColorName})`) : _el$15.style.removeProperty("color");
      `var(${borderColorName})` != null ? _el$15.style.setProperty("border-left-color", `var(${borderColorName})`) : _el$15.style.removeProperty("border-left-color");
      _el$15.style.setProperty("border-left-style", "solid");
      insert(_el$15, (() => {
        var _c$3 = solidJs.createMemo(() => !!couldExpand());
        return () => _c$3() ? solidJs.createMemo(() => !!currentIsFirstLine())() ? (() => {
          var _el$24 = _tmpl$2$4();
          _el$24.$$click = () => props.diffFile.onSplitHunkExpand("up", props.index);
          insert(_el$24, solidJs.createComponent(ExpandUp, {
            "class": "fill-current"
          }));
          return _el$24;
        })() : solidJs.createMemo(() => !!currentIsLastLine())() ? (() => {
          var _el$25 = _tmpl$3$2();
          _el$25.$$click = () => props.diffFile.onSplitHunkExpand("down", props.index);
          insert(_el$25, solidJs.createComponent(ExpandDown, {
            "class": "fill-current"
          }));
          return _el$25;
        })() : solidJs.createMemo(() => !!currentShowExpandAll())() ? (() => {
          var _el$26 = _tmpl$4$2();
          _el$26.$$click = () => props.diffFile.onSplitHunkExpand("all", props.index);
          insert(_el$26, solidJs.createComponent(ExpandAll, {
            "class": "fill-current"
          }));
          return _el$26;
        })() : [(() => {
          var _el$27 = _tmpl$5$2();
          _el$27.$$click = () => props.diffFile.onSplitHunkExpand("down", props.index);
          insert(_el$27, solidJs.createComponent(ExpandDown, {
            "class": "fill-current"
          }));
          return _el$27;
        })(), (() => {
          var _el$28 = _tmpl$6$1();
          _el$28.$$click = () => props.diffFile.onSplitHunkExpand("up", props.index);
          insert(_el$28, solidJs.createComponent(ExpandUp, {
            "class": "fill-current"
          }));
          return _el$28;
        })()] : _tmpl$7$1();
      })());
      `var(${hunkContentBGName})` != null ? _el$16.style.setProperty("background-color", `var(${hunkContentBGName})`) : _el$16.style.removeProperty("background-color");
      `var(${hunkContentColorName})` != null ? _el$17.style.setProperty("color", `var(${hunkContentColorName})`) : _el$17.style.removeProperty("color");
      insert(_el$17, () => {
        var _a, _b, _c;
        return ((_b = (_a = currentHunk()) == null ? void 0 : _a.splitInfo) == null ? void 0 : _b.plainText) || ((_c = currentHunk()) == null ? void 0 : _c.text);
      });
      solidJs.createRenderEffect(() => setAttribute(_el$11, "data-line", `${props.lineNumber}-hunk`));
      return _el$11;
    }
  });
};
const DiffSplitHunkLine = (props) => {
  const mode = useMode();
  return solidJs.createComponent(solidJs.Show, {
    get when() {
      return mode() === exports.DiffModeEnum.SplitGitHub || mode() === exports.DiffModeEnum.Split;
    },
    get fallback() {
      return solidJs.createComponent(DiffSplitHunkLineGitLab, {
        get index() {
          return props.index;
        },
        get diffFile() {
          return props.diffFile;
        },
        get lineNumber() {
          return props.lineNumber;
        }
      });
    },
    get children() {
      return solidJs.createComponent(DiffSplitHunkLineGitHub, {
        get index() {
          return props.index;
        },
        get diffFile() {
          return props.diffFile;
        },
        get lineNumber() {
          return props.lineNumber;
        }
      });
    }
  });
};
delegateEvents(["click"]);
var _tmpl$$7 = /* @__PURE__ */ template(`<tr data-state=widget class="diff-line diff-line-widget">`), _tmpl$2$3 = /* @__PURE__ */ template(`<td class="diff-line-widget-old-content p-0"colspan=2><div class=diff-line-widget-wrapper>`), _tmpl$3$1 = /* @__PURE__ */ template(`<td class="diff-line-widget-old-placeholder select-none p-0"colspan=2>`), _tmpl$4$1 = /* @__PURE__ */ template(`<td class="diff-line-widget-new-content border-l-[1px] p-0"colspan=2><div class=diff-line-widget-wrapper>`), _tmpl$5$1 = /* @__PURE__ */ template(`<td class="diff-line-widget-new-placeholder select-none border-l-[1px] p-0"colspan=2>`);
const DiffSplitWidgetLine = (props) => {
  const renderWidget = useRenderWidget();
  const [widget, setWidget] = useDiffWidgetContext() || [];
  const oldLine = solidJs.createMemo(() => props.diffFile.getSplitLeftLine(props.index));
  const newLine = solidJs.createMemo(() => props.diffFile.getSplitRightLine(props.index));
  const oldLineWidget = solidJs.createMemo(() => {
    var _a, _b, _c, _d;
    return ((_a = oldLine()) == null ? void 0 : _a.lineNumber) && ((_b = widget == null ? void 0 : widget()) == null ? void 0 : _b.side) === core.SplitSide.old && ((_c = widget == null ? void 0 : widget()) == null ? void 0 : _c.lineNumber) === ((_d = oldLine()) == null ? void 0 : _d.lineNumber);
  });
  const newLineWidget = solidJs.createMemo(() => {
    var _a, _b, _c, _d;
    return ((_a = newLine()) == null ? void 0 : _a.lineNumber) && ((_b = widget == null ? void 0 : widget()) == null ? void 0 : _b.side) === core.SplitSide.new && ((_c = widget == null ? void 0 : widget()) == null ? void 0 : _c.lineNumber) === ((_d = newLine()) == null ? void 0 : _d.lineNumber);
  });
  const currentIsShow = solidJs.createMemo(() => {
    var _a, _b;
    return (!!oldLineWidget() || !!newLineWidget()) && !((_a = oldLine()) == null ? void 0 : _a.isHidden) && !((_b = newLine()) == null ? void 0 : _b.isHidden) && !!renderWidget;
  });
  const onCloseWidget = () => setWidget == null ? void 0 : setWidget({});
  return solidJs.createComponent(solidJs.Show, {
    get when() {
      return currentIsShow();
    },
    get children() {
      var _el$ = _tmpl$$7();
      insert(_el$, (() => {
        var _c$ = solidJs.createMemo(() => !!(oldLineWidget() && renderWidget()));
        return () => _c$() ? (() => {
          var _el$2 = _tmpl$2$3(), _el$3 = _el$2.firstChild;
          insert(_el$3, () => {
            var _a, _b;
            return (_b = renderWidget == null ? void 0 : renderWidget()) == null ? void 0 : _b({
              diffFile: props.diffFile,
              side: core.SplitSide.old,
              lineNumber: ((_a = oldLine()) == null ? void 0 : _a.lineNumber) || 0,
              onClose: onCloseWidget
            });
          });
          return _el$2;
        })() : (() => {
          var _el$4 = _tmpl$3$1();
          `var(${emptyBGName})` != null ? _el$4.style.setProperty("background-color", `var(${emptyBGName})`) : _el$4.style.removeProperty("background-color");
          return _el$4;
        })();
      })(), null);
      insert(_el$, (() => {
        var _c$2 = solidJs.createMemo(() => !!(newLineWidget() && renderWidget()));
        return () => _c$2() ? (() => {
          var _el$5 = _tmpl$4$1(), _el$6 = _el$5.firstChild;
          `var(${borderColorName})` != null ? _el$5.style.setProperty("border-left-color", `var(${borderColorName})`) : _el$5.style.removeProperty("border-left-color");
          _el$5.style.setProperty("border-left-style", "solid");
          insert(_el$6, () => {
            var _a, _b;
            return (_b = renderWidget == null ? void 0 : renderWidget()) == null ? void 0 : _b({
              diffFile: props.diffFile,
              side: core.SplitSide.new,
              lineNumber: ((_a = newLine()) == null ? void 0 : _a.lineNumber) || 0,
              onClose: onCloseWidget
            });
          });
          return _el$5;
        })() : (() => {
          var _el$7 = _tmpl$5$1();
          `var(${emptyBGName})` != null ? _el$7.style.setProperty("background-color", `var(${emptyBGName})`) : _el$7.style.removeProperty("background-color");
          `var(${borderColorName})` != null ? _el$7.style.setProperty("border-left-color", `var(${borderColorName})`) : _el$7.style.removeProperty("border-left-color");
          _el$7.style.setProperty("border-left-style", "solid");
          return _el$7;
        })();
      })(), null);
      solidJs.createRenderEffect(() => setAttribute(_el$, "data-line", `${props.lineNumber}-widget`));
      return _el$;
    }
  });
};
var _tmpl$$6 = /* @__PURE__ */ template(`<div class="split-diff-view split-diff-view-warp w-full"><div class="diff-table-wrapper w-full"><style data-select-style></style><table class="diff-table w-full table-fixed border-collapse border-spacing-0"><colgroup><col class=diff-table-old-num-col><col class=diff-table-old-content-col><col class=diff-table-new-num-col><col class=diff-table-new-content-col></colgroup><thead class=hidden><tr><th scope=col>old line number</th><th scope=col>old line content</th><th scope=col>new line number</th><th scope=col>new line content</th></tr></thead><tbody class="diff-table-body leading-[1.4]">`);
const DiffSplitViewWrap = (props) => {
  const getAllLines = () => core.getSplitContentLines(props.diffFile);
  const [lines, setLines] = solidJs.createSignal(getAllLines());
  const [styleRef, setStyleRef] = solidJs.createSignal();
  const maxText = solidJs.createMemo(() => Math.max(props.diffFile.splitLineLength, props.diffFile.fileLineLength).toString());
  const fontSize = useFontSize();
  const font = solidJs.createMemo(() => ({
    fontSize: `${fontSize() || 14}px`,
    fontFamily: "Menlo, Consolas, monospace"
  }));
  solidJs.createEffect(() => {
    const init = () => setLines(getAllLines);
    init();
    const cb = props.diffFile.subscribe(init);
    solidJs.onCleanup(cb);
  });
  const selectState = {
    current: void 0
  };
  const onSelect = (side) => {
    const ele = styleRef();
    if (!ele) return;
    if (side) {
      const targetSide = side === core.SplitSide.old ? core.SplitSide.new : core.SplitSide.old;
      ele.textContent = `#diff-root${props.diffFile.getId()} [data-side="${core.SplitSide[targetSide]}"] {user-select: none} 
#diff-root${props.diffFile.getId()} [data-state="extend"] {user-select: none} 
#diff-root${props.diffFile.getId()} [data-state="hunk"] {user-select: none} 
#diff-root${props.diffFile.getId()} [data-state="widget"] {user-select: none}`;
    } else {
      ele.textContent = "";
    }
  };
  const onMouseDown = (e) => {
    let ele = e.target;
    if (ele && ele instanceof HTMLElement && ele.nodeName === "BUTTON") {
      removeAllSelection();
      return;
    }
    while (ele && ele instanceof HTMLElement) {
      const state = ele.getAttribute("data-state");
      const side = ele.getAttribute("data-side");
      if (side) {
        if (selectState.current !== core.SplitSide[side]) {
          selectState.current = core.SplitSide[side];
          onSelect(core.SplitSide[side]);
          removeAllSelection();
        }
      }
      if (state) {
        if (state === "extend" || state === "hunk" || state === "widget") {
          if (selectState.current !== void 0) {
            selectState.current = void 0;
            onSelect(void 0);
            removeAllSelection();
          }
          return;
        } else {
          return;
        }
      }
      ele = ele.parentElement;
    }
  };
  const width = useTextWidth({
    text: maxText,
    font
  });
  const memoWidth = solidJs.createMemo(() => Math.max(40, width() + 25));
  return (() => {
    var _el$ = _tmpl$$6(), _el$2 = _el$.firstChild, _el$3 = _el$2.firstChild, _el$4 = _el$3.nextSibling, _el$5 = _el$4.firstChild, _el$6 = _el$5.firstChild, _el$7 = _el$6.nextSibling, _el$8 = _el$7.nextSibling, _el$9 = _el$5.nextSibling, _el$10 = _el$9.nextSibling;
    _el$2.style.setProperty("font-family", "Menlo, Consolas, monospace");
    `var(${diffFontSizeName})` != null ? _el$2.style.setProperty("font-size", `var(${diffFontSizeName})`) : _el$2.style.removeProperty("font-size");
    use(setStyleRef, _el$3);
    _el$10.$$mousedown = onMouseDown;
    insert(_el$10, solidJs.createComponent(solidJs.For, {
      get each() {
        return lines();
      },
      children: (item) => [solidJs.createComponent(DiffSplitHunkLine, {
        get index() {
          return item.index;
        },
        get lineNumber() {
          return item.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        }
      }), solidJs.createComponent(DiffSplitContentLine, {
        get index() {
          return item.index;
        },
        get lineNumber() {
          return item.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        }
      }), solidJs.createComponent(DiffSplitWidgetLine, {
        get index() {
          return item.index;
        },
        get lineNumber() {
          return item.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        }
      }), solidJs.createComponent(DiffSplitExtendLine, {
        get index() {
          return item.index;
        },
        get lineNumber() {
          return item.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        }
      })]
    }), null);
    insert(_el$10, solidJs.createComponent(DiffSplitHunkLine, {
      get index() {
        return props.diffFile.splitLineLength;
      },
      get lineNumber() {
        return props.diffFile.splitLineLength;
      },
      get diffFile() {
        return props.diffFile;
      }
    }), null);
    solidJs.createRenderEffect((_p$) => {
      var _v$ = {
        [diffAsideWidthName]: `${Math.round(memoWidth())}px`
      }, _v$2 = Math.round(memoWidth()), _v$3 = Math.round(memoWidth());
      _p$.e = style(_el$2, _v$, _p$.e);
      _v$2 !== _p$.t && setAttribute(_el$6, "width", _p$.t = _v$2);
      _v$3 !== _p$.a && setAttribute(_el$8, "width", _p$.a = _v$3);
      return _p$;
    }, {
      e: void 0,
      t: void 0,
      a: void 0
    });
    return _el$;
  })();
};
delegateEvents(["mousedown"]);
const DiffSplitView = (props) => {
  const enableWrap = useEnableWrap();
  return solidJs.createComponent(solidJs.Show, {
    get when() {
      return enableWrap();
    },
    get fallback() {
      return solidJs.createComponent(DiffSplitViewNormal, {
        get diffFile() {
          return props.diffFile;
        }
      });
    },
    get children() {
      return solidJs.createComponent(DiffSplitViewWrap, {
        get diffFile() {
          return props.diffFile;
        }
      });
    }
  });
};
var _tmpl$$5 = /* @__PURE__ */ template(`<tr data-state=diff class="diff-line group"><td class="diff-line-num sticky left-0 z-[1] w-[1%] min-w-[100px] select-none whitespace-nowrap pl-[10px] pr-[10px] text-right align-top"><div class=flex><span class="inline-block w-[50%]"></span><span class="w-[10px] shrink-0"></span><span class="inline-block w-[50%]"></span></div></td><td class="diff-line-content pr-[10px] align-top">`), _tmpl$2$2 = /* @__PURE__ */ template(`<tr class="diff-line group"><td class="diff-line-num sticky left-0 z-[1] w-[1%] min-w-[100px] select-none whitespace-nowrap pl-[10px] pr-[10px] text-right align-top"><div class="flex opacity-[0.5]"><span class="inline-block w-[50%]"></span><span class="w-[10px] shrink-0"></span><span class="inline-block w-[50%]"></span></div></td><td class="diff-line-content pr-[10px] align-top">`);
const DiffUnifiedOldLine = (props) => {
  return (() => {
    var _el$ = _tmpl$$5(), _el$2 = _el$.firstChild, _el$3 = _el$2.firstChild, _el$4 = _el$3.firstChild, _el$5 = _el$2.nextSibling;
    `var(${plainLineNumberColorName})` != null ? _el$2.style.setProperty("color", `var(${plainLineNumberColorName})`) : _el$2.style.removeProperty("color");
    `var(${delLineNumberBGName})` != null ? _el$2.style.setProperty("background-color", `var(${delLineNumberBGName})`) : _el$2.style.removeProperty("background-color");
    `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)` != null ? _el$2.style.setProperty("width", `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)`) : _el$2.style.removeProperty("width");
    `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)` != null ? _el$2.style.setProperty("max-width", `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)`) : _el$2.style.removeProperty("max-width");
    `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)` != null ? _el$2.style.setProperty("min-width", `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)`) : _el$2.style.removeProperty("min-width");
    insert(_el$2, (() => {
      var _c$ = solidJs.createMemo(() => !!props.enableAddWidget);
      return () => _c$() && solidJs.createComponent(DiffUnifiedAddWidget, {
        get index() {
          return props.index - 1;
        },
        get lineNumber() {
          return props.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        },
        get side() {
          return core.SplitSide.old;
        },
        get onWidgetClick() {
          return props.onAddWidgetClick;
        },
        get onOpenAddWidget() {
          return props.onOpenAddWidget;
        }
      });
    })(), _el$3);
    insert(_el$4, () => props.lineNumber);
    `var(${delContentBGName})` != null ? _el$5.style.setProperty("background-color", `var(${delContentBGName})`) : _el$5.style.removeProperty("background-color");
    insert(_el$5, solidJs.createComponent(DiffContent, {
      get enableWrap() {
        return props.enableWrap;
      },
      get diffFile() {
        return props.diffFile;
      },
      get enableHighlight() {
        return props.enableHighlight;
      },
      get rawLine() {
        return props.rawLine;
      },
      get diffLine() {
        return props.diffLine;
      },
      get plainLine() {
        return props.plainLine;
      },
      get syntaxLine() {
        return props.syntaxLine;
      }
    }));
    solidJs.createRenderEffect((_p$) => {
      var _v$ = props.index, _v$2 = props.lineNumber;
      _v$ !== _p$.e && setAttribute(_el$, "data-line", _p$.e = _v$);
      _v$2 !== _p$.t && setAttribute(_el$4, "data-line-old-num", _p$.t = _v$2);
      return _p$;
    }, {
      e: void 0,
      t: void 0
    });
    return _el$;
  })();
};
const DiffUnifiedNewLine = (props) => {
  return (() => {
    var _el$6 = _tmpl$$5(), _el$7 = _el$6.firstChild, _el$8 = _el$7.firstChild, _el$9 = _el$8.firstChild, _el$10 = _el$9.nextSibling, _el$11 = _el$10.nextSibling, _el$12 = _el$7.nextSibling;
    `var(${plainLineNumberColorName})` != null ? _el$7.style.setProperty("color", `var(${plainLineNumberColorName})`) : _el$7.style.removeProperty("color");
    `var(${addLineNumberBGName})` != null ? _el$7.style.setProperty("background-color", `var(${addLineNumberBGName})`) : _el$7.style.removeProperty("background-color");
    `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)` != null ? _el$7.style.setProperty("width", `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)`) : _el$7.style.removeProperty("width");
    `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)` != null ? _el$7.style.setProperty("max-width", `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)`) : _el$7.style.removeProperty("max-width");
    `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)` != null ? _el$7.style.setProperty("min-width", `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)`) : _el$7.style.removeProperty("min-width");
    insert(_el$7, (() => {
      var _c$2 = solidJs.createMemo(() => !!props.enableAddWidget);
      return () => _c$2() && solidJs.createComponent(DiffUnifiedAddWidget, {
        get index() {
          return props.index - 1;
        },
        get lineNumber() {
          return props.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        },
        get side() {
          return core.SplitSide.new;
        },
        get onWidgetClick() {
          return props.onAddWidgetClick;
        },
        get onOpenAddWidget() {
          return props.onOpenAddWidget;
        }
      });
    })(), _el$8);
    insert(_el$11, () => props.lineNumber);
    `var(${addContentBGName})` != null ? _el$12.style.setProperty("background-color", `var(${addContentBGName})`) : _el$12.style.removeProperty("background-color");
    insert(_el$12, solidJs.createComponent(DiffContent, {
      get enableWrap() {
        return props.enableWrap;
      },
      get diffFile() {
        return props.diffFile;
      },
      get enableHighlight() {
        return props.enableHighlight;
      },
      get rawLine() {
        return props.rawLine;
      },
      get diffLine() {
        return props.diffLine;
      },
      get plainLine() {
        return props.plainLine;
      },
      get syntaxLine() {
        return props.syntaxLine;
      }
    }));
    solidJs.createRenderEffect((_p$) => {
      var _v$3 = props.index, _v$4 = props.lineNumber;
      _v$3 !== _p$.e && setAttribute(_el$6, "data-line", _p$.e = _v$3);
      _v$4 !== _p$.t && setAttribute(_el$11, "data-line-new-num", _p$.t = _v$4);
      return _p$;
    }, {
      e: void 0,
      t: void 0
    });
    return _el$6;
  })();
};
const DiffUnifiedContentLine = (props) => {
  const unifiedItem = solidJs.createMemo(() => props.diffFile.getUnifiedLine(props.index));
  const enableWrap = useEnableWrap();
  const [_, setWidget] = useDiffWidgetContext() || [];
  const onAddWidgetClick = useOnAddWidgetClick();
  const enableHighlight = useEnableHighlight();
  const enableAddWidget = useEnableAddWidget();
  const currentItemHasHidden = solidJs.createMemo(() => {
    var _a;
    return (_a = unifiedItem()) == null ? void 0 : _a.isHidden;
  });
  const currentItemHasChange = solidJs.createMemo(() => {
    var _a;
    return core.checkDiffLineIncludeChange((_a = unifiedItem()) == null ? void 0 : _a.diff);
  });
  const getCurrentSyntaxLine = () => {
    var _a, _b, _c, _d;
    return ((_a = unifiedItem()) == null ? void 0 : _a.newLineNumber) ? props.diffFile.getNewSyntaxLine(((_b = unifiedItem()) == null ? void 0 : _b.newLineNumber) || 0) : ((_c = unifiedItem()) == null ? void 0 : _c.oldLineNumber) ? props.diffFile.getOldSyntaxLine(((_d = unifiedItem()) == null ? void 0 : _d.oldLineNumber) || 0) : void 0;
  };
  const [currentSyntaxLine, setCurrentSyntaxLine] = solidJs.createSignal(getCurrentSyntaxLine());
  const getCurrentPlainLine = () => {
    var _a, _b, _c, _d;
    return ((_a = unifiedItem()) == null ? void 0 : _a.newLineNumber) ? props.diffFile.getNewPlainLine(((_b = unifiedItem()) == null ? void 0 : _b.newLineNumber) || 0) : ((_c = unifiedItem()) == null ? void 0 : _c.oldLineNumber) ? props.diffFile.getOldPlainLine(((_d = unifiedItem()) == null ? void 0 : _d.oldLineNumber) || 0) : void 0;
  };
  const [currentPlainLine, setCurrentPlainLine] = solidJs.createSignal(getCurrentPlainLine());
  solidJs.createEffect(() => {
    const init = () => {
      setCurrentSyntaxLine(getCurrentSyntaxLine);
      setCurrentPlainLine(getCurrentPlainLine);
    };
    init();
    const unsubscribe = props.diffFile.subscribe(init);
    solidJs.onCleanup(unsubscribe);
  });
  const onClickAddWidget = (lineNumber, side) => setWidget == null ? void 0 : setWidget({
    side,
    lineNumber
  });
  return solidJs.createComponent(solidJs.Show, {
    get when() {
      return !currentItemHasHidden();
    },
    get children() {
      return solidJs.createComponent(solidJs.Show, {
        get when() {
          return currentItemHasChange();
        },
        get fallback() {
          return (() => {
            var _el$13 = _tmpl$2$2(), _el$14 = _el$13.firstChild, _el$15 = _el$14.firstChild, _el$16 = _el$15.firstChild, _el$17 = _el$16.nextSibling, _el$18 = _el$17.nextSibling, _el$19 = _el$14.nextSibling;
            `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)` != null ? _el$14.style.setProperty("width", `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)`) : _el$14.style.removeProperty("width");
            `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)` != null ? _el$14.style.setProperty("max-width", `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)`) : _el$14.style.removeProperty("max-width");
            `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)` != null ? _el$14.style.setProperty("min-width", `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)`) : _el$14.style.removeProperty("min-width");
            insert(_el$14, (() => {
              var _c$3 = solidJs.createMemo(() => {
                var _a;
                return !!(enableAddWidget() && ((_a = unifiedItem()) == null ? void 0 : _a.diff));
              });
              return () => _c$3() && solidJs.createComponent(DiffUnifiedAddWidget, {
                get index() {
                  return props.index;
                },
                get diffFile() {
                  return props.diffFile;
                },
                get lineNumber() {
                  var _a;
                  return ((_a = unifiedItem()) == null ? void 0 : _a.newLineNumber) || 0;
                },
                get side() {
                  return core.SplitSide.new;
                },
                onOpenAddWidget: onClickAddWidget,
                onWidgetClick: onAddWidgetClick
              });
            })(), _el$15);
            insert(_el$16, () => {
              var _a;
              return ((_a = unifiedItem()) == null ? void 0 : _a.oldLineNumber) || 0;
            });
            insert(_el$18, () => {
              var _a;
              return ((_a = unifiedItem()) == null ? void 0 : _a.newLineNumber) || 0;
            });
            insert(_el$19, solidJs.createComponent(DiffContent, {
              get enableWrap() {
                return !!enableWrap();
              },
              get diffFile() {
                return props.diffFile;
              },
              get enableHighlight() {
                return !!enableHighlight();
              },
              get rawLine() {
                var _a;
                return ((_a = unifiedItem()) == null ? void 0 : _a.value) || "";
              },
              get diffLine() {
                var _a;
                return (_a = unifiedItem()) == null ? void 0 : _a.diff;
              },
              get plainLine() {
                return currentPlainLine();
              },
              get syntaxLine() {
                return currentSyntaxLine();
              }
            }));
            solidJs.createRenderEffect((_p$) => {
              var _a, _b, _c, _d, _e, _f;
              var _v$5 = props.lineNumber, _v$6 = ((_a = unifiedItem()) == null ? void 0 : _a.diff) ? "diff" : "plain", _v$7 = `var(${((_b = unifiedItem()) == null ? void 0 : _b.diff) ? plainLineNumberColorName : expandLineNumberColorName})`, _v$8 = ((_c = unifiedItem()) == null ? void 0 : _c.diff) ? `var(${plainLineNumberBGName})` : `var(${expandContentBGName})`, _v$9 = ((_d = unifiedItem()) == null ? void 0 : _d.oldLineNumber) || 0, _v$10 = ((_e = unifiedItem()) == null ? void 0 : _e.newLineNumber) || 0, _v$11 = ((_f = unifiedItem()) == null ? void 0 : _f.diff) ? `var(${plainContentBGName})` : `var(${expandContentBGName})`;
              _v$5 !== _p$.e && setAttribute(_el$13, "data-line", _p$.e = _v$5);
              _v$6 !== _p$.t && setAttribute(_el$13, "data-state", _p$.t = _v$6);
              _v$7 !== _p$.a && ((_p$.a = _v$7) != null ? _el$14.style.setProperty("color", _v$7) : _el$14.style.removeProperty("color"));
              _v$8 !== _p$.o && ((_p$.o = _v$8) != null ? _el$14.style.setProperty("background-color", _v$8) : _el$14.style.removeProperty("background-color"));
              _v$9 !== _p$.i && setAttribute(_el$16, "data-line-old-num", _p$.i = _v$9);
              _v$10 !== _p$.n && setAttribute(_el$18, "data-line-new-num", _p$.n = _v$10);
              _v$11 !== _p$.s && ((_p$.s = _v$11) != null ? _el$19.style.setProperty("background-color", _v$11) : _el$19.style.removeProperty("background-color"));
              return _p$;
            }, {
              e: void 0,
              t: void 0,
              a: void 0,
              o: void 0,
              i: void 0,
              n: void 0,
              s: void 0
            });
            return _el$13;
          })();
        },
        get children() {
          return solidJs.createComponent(solidJs.Show, {
            get when() {
              var _a;
              return (_a = unifiedItem()) == null ? void 0 : _a.oldLineNumber;
            },
            get fallback() {
              return solidJs.createComponent(DiffUnifiedNewLine, {
                get index() {
                  return props.lineNumber;
                },
                get enableWrap() {
                  return !!enableWrap();
                },
                get diffFile() {
                  return props.diffFile;
                },
                get rawLine() {
                  var _a;
                  return ((_a = unifiedItem()) == null ? void 0 : _a.value) || "";
                },
                get diffLine() {
                  var _a;
                  return (_a = unifiedItem()) == null ? void 0 : _a.diff;
                },
                get plainLine() {
                  return currentPlainLine();
                },
                get syntaxLine() {
                  return currentSyntaxLine();
                },
                get enableHighlight() {
                  return !!enableHighlight();
                },
                get enableAddWidget() {
                  return !!enableAddWidget();
                },
                get lineNumber() {
                  return unifiedItem().newLineNumber || 0;
                },
                onAddWidgetClick,
                onOpenAddWidget: onClickAddWidget
              });
            },
            get children() {
              return solidJs.createComponent(DiffUnifiedOldLine, {
                get index() {
                  return props.lineNumber;
                },
                get enableWrap() {
                  return !!enableWrap();
                },
                get diffFile() {
                  return props.diffFile;
                },
                get rawLine() {
                  var _a;
                  return ((_a = unifiedItem()) == null ? void 0 : _a.value) || "";
                },
                get diffLine() {
                  var _a;
                  return (_a = unifiedItem()) == null ? void 0 : _a.diff;
                },
                get plainLine() {
                  return currentPlainLine();
                },
                get syntaxLine() {
                  return currentSyntaxLine();
                },
                get enableHighlight() {
                  return !!enableHighlight();
                },
                get enableAddWidget() {
                  return !!enableAddWidget();
                },
                get lineNumber() {
                  return unifiedItem().oldLineNumber || 0;
                },
                onAddWidgetClick,
                onOpenAddWidget: onClickAddWidget
              });
            }
          });
        }
      });
    }
  });
};
var _tmpl$$4 = /* @__PURE__ */ template(`<tr data-state=extend class="diff-line diff-line-extend"><td class="diff-line-extend-content p-0 align-top"colspan=2><div class="diff-line-extend-wrapper sticky left-0 z-[1]">`);
const DiffUnifiedExtendLine = (props) => {
  const extendData = useExtendData();
  const renderExtend = useRenderExtend();
  const unifiedItem = solidJs.createMemo(() => props.diffFile.getUnifiedLine(props.index));
  const oldExtend = solidJs.createMemo(() => {
    var _a, _b, _c;
    return (_c = (_a = extendData()) == null ? void 0 : _a.oldFile) == null ? void 0 : _c[((_b = unifiedItem()) == null ? void 0 : _b.oldLineNumber) || 0];
  });
  const newExtend = solidJs.createMemo(() => {
    var _a, _b, _c;
    return (_c = (_a = extendData()) == null ? void 0 : _a.newFile) == null ? void 0 : _c[((_b = unifiedItem()) == null ? void 0 : _b.newLineNumber) || 0];
  });
  const currentIsHidden = solidJs.createMemo(() => {
    var _a;
    return (_a = unifiedItem()) == null ? void 0 : _a.isHidden;
  });
  const currentIsShow = solidJs.createMemo(() => Boolean((oldExtend() || newExtend()) && !currentIsHidden() && renderExtend()));
  const lineSelector = solidJs.createMemo(() => `.unified-diff-table-wrapper`);
  const width = useDomWidth({
    selector: lineSelector,
    enable: currentIsShow
  });
  return solidJs.createComponent(solidJs.Show, {
    get when() {
      return currentIsShow();
    },
    get children() {
      var _el$ = _tmpl$$4(), _el$2 = _el$.firstChild, _el$3 = _el$2.firstChild;
      insert(_el$3, (() => {
        var _c$ = solidJs.createMemo(() => !!(width() > 0 && oldExtend()));
        return () => {
          var _a, _b, _c;
          return _c$() && ((_c = renderExtend()) == null ? void 0 : _c({
            diffFile: props.diffFile,
            side: core.SplitSide.old,
            lineNumber: ((_a = unifiedItem()) == null ? void 0 : _a.oldLineNumber) || 0,
            data: (_b = oldExtend()) == null ? void 0 : _b.data,
            onUpdate: props.diffFile.notifyAll
          }));
        };
      })(), null);
      insert(_el$3, (() => {
        var _c$2 = solidJs.createMemo(() => !!(width() > 0 && newExtend()));
        return () => {
          var _a, _b;
          return _c$2() && ((_b = renderExtend()) == null ? void 0 : _b({
            diffFile: props.diffFile,
            side: core.SplitSide.new,
            lineNumber: unifiedItem().newLineNumber || 0,
            data: (_a = newExtend()) == null ? void 0 : _a.data,
            onUpdate: props.diffFile.notifyAll
          }));
        };
      })(), null);
      solidJs.createRenderEffect((_p$) => {
        var _v$ = `${props.lineNumber}-extend`, _v$2 = width() + "px";
        _v$ !== _p$.e && setAttribute(_el$, "data-line", _p$.e = _v$);
        _v$2 !== _p$.t && ((_p$.t = _v$2) != null ? _el$3.style.setProperty("width", _v$2) : _el$3.style.removeProperty("width"));
        return _p$;
      }, {
        e: void 0,
        t: void 0
      });
      return _el$;
    }
  });
};
var _tmpl$$3 = /* @__PURE__ */ template(`<tr data-state=hunk class="diff-line diff-line-hunk"><td class="diff-line-hunk-action sticky left-0 w-[1%] min-w-[100px] select-none"></td><td class="diff-line-hunk-content pr-[10px] align-middle"><div class=pl-[1.5em]>`), _tmpl$2$1 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[6px]"title="Expand Up"data-title="Expand Up">`), _tmpl$3 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[6px]"title="Expand Down"data-title="Expand Down">`), _tmpl$4 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[6px]"title="Expand All"data-title="Expand All">`), _tmpl$5 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[2px]"title="Expand Down"data-title="Expand Down">`), _tmpl$6 = /* @__PURE__ */ template(`<button class="diff-widget-tooltip flex w-full cursor-pointer items-center justify-center rounded-[2px] py-[2px]"title="Expand Up"data-title="Expand Up">`), _tmpl$7 = /* @__PURE__ */ template(`<div class=min-h-[28px]>&ensp;`);
const DiffUnifiedHunkLine = (props) => {
  const currentHunk = solidJs.createMemo(() => props.diffFile.getUnifiedHunkLine(props.index));
  const enableExpand = solidJs.createMemo(() => props.diffFile.getExpandEnabled());
  const couldExpand = solidJs.createMemo(() => {
    var _a;
    return enableExpand() && !!((_a = currentHunk()) == null ? void 0 : _a.unifiedInfo);
  });
  const enableWrap = useEnableWrap();
  const checkCurrentIsShow = () => {
    const hunk = currentHunk();
    return hunk && hunk.unifiedInfo && hunk.unifiedInfo.startHiddenIndex < hunk.unifiedInfo.endHiddenIndex;
  };
  const [currentIsShow, setCurrentIsShow] = solidJs.createSignal(checkCurrentIsShow());
  const checkCurrentIsEnableAll = () => {
    const hunk = currentHunk();
    return hunk && hunk.unifiedInfo && hunk.unifiedInfo.endHiddenIndex - hunk.unifiedInfo.startHiddenIndex < core.composeLen;
  };
  const [currentIsEnableAll, setCurrentIsEnableAll] = solidJs.createSignal(checkCurrentIsEnableAll());
  const currentIsFirstLine = solidJs.createMemo(() => {
    var _a;
    return (_a = currentHunk()) == null ? void 0 : _a.isFirst;
  });
  const currentIsLastLine = solidJs.createMemo(() => {
    var _a;
    return (_a = currentHunk()) == null ? void 0 : _a.isLast;
  });
  const currentIsPureHunk = solidJs.createMemo(() => {
    var _a;
    return currentHunk() && props.diffFile._getIsPureDiffRender() && !((_a = currentHunk()) == null ? void 0 : _a.unifiedInfo);
  });
  solidJs.createEffect(() => {
    const init = () => {
      setCurrentIsShow(checkCurrentIsShow);
      setCurrentIsEnableAll(checkCurrentIsEnableAll);
    };
    init();
    const unsubscribe = props.diffFile.subscribe(init);
    solidJs.onCleanup(unsubscribe);
  });
  return solidJs.createComponent(solidJs.Show, {
    get when() {
      return currentIsShow() || currentIsPureHunk();
    },
    get children() {
      var _el$ = _tmpl$$3(), _el$2 = _el$.firstChild, _el$3 = _el$2.nextSibling, _el$4 = _el$3.firstChild;
      `var(${hunkLineNumberBGName})` != null ? _el$2.style.setProperty("background-color", `var(${hunkLineNumberBGName})`) : _el$2.style.removeProperty("background-color");
      `var(${plainLineNumberColorName})` != null ? _el$2.style.setProperty("color", `var(${plainLineNumberColorName})`) : _el$2.style.removeProperty("color");
      `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)` != null ? _el$2.style.setProperty("width", `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)`) : _el$2.style.removeProperty("width");
      `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)` != null ? _el$2.style.setProperty("max-width", `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)`) : _el$2.style.removeProperty("max-width");
      `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)` != null ? _el$2.style.setProperty("min-width", `calc(calc(var(${diffAsideWidthName}) + 5px) * 2)`) : _el$2.style.removeProperty("min-width");
      insert(_el$2, (() => {
        var _c$ = solidJs.createMemo(() => !!couldExpand());
        return () => _c$() ? solidJs.createMemo(() => !!currentIsFirstLine())() ? (() => {
          var _el$5 = _tmpl$2$1();
          _el$5.$$click = () => props.diffFile.onUnifiedHunkExpand("up", props.index);
          insert(_el$5, solidJs.createComponent(ExpandUp, {
            "class": "fill-current"
          }));
          return _el$5;
        })() : solidJs.createMemo(() => !!currentIsLastLine())() ? (() => {
          var _el$6 = _tmpl$3();
          _el$6.$$click = () => props.diffFile.onUnifiedHunkExpand("down", props.index);
          insert(_el$6, solidJs.createComponent(ExpandDown, {
            "class": "fill-current"
          }));
          return _el$6;
        })() : solidJs.createMemo(() => !!currentIsEnableAll())() ? (() => {
          var _el$7 = _tmpl$4();
          _el$7.$$click = () => props.diffFile.onUnifiedHunkExpand("all", props.index);
          insert(_el$7, solidJs.createComponent(ExpandAll, {
            "class": "fill-current"
          }));
          return _el$7;
        })() : [(() => {
          var _el$8 = _tmpl$5();
          _el$8.$$click = () => props.diffFile.onUnifiedHunkExpand("down", props.index);
          insert(_el$8, solidJs.createComponent(ExpandDown, {
            "class": "fill-current"
          }));
          return _el$8;
        })(), (() => {
          var _el$9 = _tmpl$6();
          _el$9.$$click = () => props.diffFile.onUnifiedHunkExpand("up", props.index);
          insert(_el$9, solidJs.createComponent(ExpandUp, {
            "class": "fill-current"
          }));
          return _el$9;
        })()] : _tmpl$7();
      })());
      `var(${hunkContentBGName})` != null ? _el$3.style.setProperty("background-color", `var(${hunkContentBGName})`) : _el$3.style.removeProperty("background-color");
      `var(${hunkContentColorName})` != null ? _el$4.style.setProperty("color", `var(${hunkContentColorName})`) : _el$4.style.removeProperty("color");
      insert(_el$4, () => {
        var _a, _b, _c;
        return ((_b = (_a = currentHunk()) == null ? void 0 : _a.unifiedInfo) == null ? void 0 : _b.plainText) || ((_c = currentHunk()) == null ? void 0 : _c.text);
      });
      solidJs.createRenderEffect((_p$) => {
        var _v$ = `${props.lineNumber}-hunk`, _v$2 = enableWrap() ? "pre-wrap" : "pre", _v$3 = enableWrap() ? "break-all" : "initial";
        _v$ !== _p$.e && setAttribute(_el$, "data-line", _p$.e = _v$);
        _v$2 !== _p$.t && ((_p$.t = _v$2) != null ? _el$4.style.setProperty("white-space", _v$2) : _el$4.style.removeProperty("white-space"));
        _v$3 !== _p$.a && ((_p$.a = _v$3) != null ? _el$4.style.setProperty("word-break", _v$3) : _el$4.style.removeProperty("word-break"));
        return _p$;
      }, {
        e: void 0,
        t: void 0,
        a: void 0
      });
      return _el$;
    }
  });
};
delegateEvents(["click"]);
var _tmpl$$2 = /* @__PURE__ */ template(`<tr data-state=widget class="diff-line diff-line-widget"><td class="diff-line-widget-content p-0"colspan=2><div class="diff-line-widget-wrapper sticky left-0 z-[1]">`);
const DiffUnifiedWidgetLine = (props) => {
  const renderWidget = useRenderWidget();
  const [widget, setWidget] = useDiffWidgetContext() || [];
  const unifiedItem = solidJs.createMemo(() => props.diffFile.getUnifiedLine(props.index));
  const oldWidget = solidJs.createMemo(() => {
    var _a, _b, _c, _d;
    return ((_a = unifiedItem()) == null ? void 0 : _a.oldLineNumber) && ((_b = widget == null ? void 0 : widget()) == null ? void 0 : _b.side) === core.SplitSide.old && ((_c = widget == null ? void 0 : widget()) == null ? void 0 : _c.lineNumber) === ((_d = unifiedItem()) == null ? void 0 : _d.oldLineNumber);
  });
  const newWidget = solidJs.createMemo(() => {
    var _a, _b, _c, _d;
    return ((_a = unifiedItem()) == null ? void 0 : _a.newLineNumber) && ((_b = widget == null ? void 0 : widget()) == null ? void 0 : _b.side) === core.SplitSide.new && ((_c = widget == null ? void 0 : widget()) == null ? void 0 : _c.lineNumber) === ((_d = unifiedItem()) == null ? void 0 : _d.newLineNumber);
  });
  const currentIsHidden = solidJs.createMemo(() => {
    var _a;
    return (_a = unifiedItem()) == null ? void 0 : _a.isHidden;
  });
  const currenIsShow = solidJs.createMemo(() => !!(oldWidget() || newWidget()) && !currentIsHidden() && !!renderWidget());
  const onCloseWidget = () => setWidget == null ? void 0 : setWidget({});
  const lineSelector = solidJs.createMemo(() => `.unified-diff-table-wrapper`);
  const width = useDomWidth({
    selector: lineSelector,
    enable: currenIsShow
  });
  return solidJs.createComponent(solidJs.Show, {
    get when() {
      return currenIsShow();
    },
    get children() {
      var _el$ = _tmpl$$2(), _el$2 = _el$.firstChild, _el$3 = _el$2.firstChild;
      insert(_el$3, (() => {
        var _c$ = solidJs.createMemo(() => !!(width() > 0 && oldWidget()));
        return () => {
          var _a, _b;
          return _c$() && ((_b = renderWidget()) == null ? void 0 : _b({
            diffFile: props.diffFile,
            side: core.SplitSide.old,
            lineNumber: ((_a = unifiedItem()) == null ? void 0 : _a.oldLineNumber) || 0,
            onClose: onCloseWidget
          }));
        };
      })(), null);
      insert(_el$3, (() => {
        var _c$2 = solidJs.createMemo(() => !!(width() > 0 && newWidget()));
        return () => {
          var _a;
          return _c$2() && ((_a = renderWidget()) == null ? void 0 : _a({
            diffFile: props.diffFile,
            side: core.SplitSide.new,
            lineNumber: unifiedItem().newLineNumber || 0,
            onClose: onCloseWidget
          }));
        };
      })(), null);
      solidJs.createRenderEffect((_p$) => {
        var _v$ = `${props.lineNumber}-widget`, _v$2 = width() + "px";
        _v$ !== _p$.e && setAttribute(_el$, "data-line", _p$.e = _v$);
        _v$2 !== _p$.t && ((_p$.t = _v$2) != null ? _el$3.style.setProperty("width", _v$2) : _el$3.style.removeProperty("width"));
        return _p$;
      }, {
        e: void 0,
        t: void 0
      });
      return _el$;
    }
  });
};
var _tmpl$$1 = /* @__PURE__ */ template(`<div><style data-select-style></style><div class="unified-diff-table-wrapper diff-table-scroll-container w-full overflow-x-auto overflow-y-hidden"><table><colgroup><col class=unified-diff-table-num-col><col class=unified-diff-table-content-col></colgroup><thead class=hidden><tr><th scope=col>line number</th><th scope=col>line content</th></tr></thead><tbody class="diff-table-body leading-[1.4]">`);
const DiffUnifiedView = (props) => {
  const getAllLines = () => core.getUnifiedContentLine(props.diffFile);
  const [lines, setLines] = solidJs.createSignal(getAllLines());
  const maxText = solidJs.createMemo(() => Math.max(props.diffFile.unifiedLineLength, props.diffFile.fileLineLength).toString());
  solidJs.createEffect(() => {
    const init = () => setLines(getAllLines);
    init();
    const unsubscribe = props.diffFile.subscribe(init);
    solidJs.onCleanup(unsubscribe);
  });
  const fontSize = useFontSize();
  const enableWrap = useEnableWrap();
  const [styleRef, setStyleRef] = solidJs.createSignal();
  const selectState = {
    current: void 0
  };
  const onSelect = (s) => {
    const ele = styleRef();
    if (!ele) return;
    if (s) {
      ele.textContent = `#${getId()} [data-state="extend"] {user-select: none} 
#${getId()} [data-state="hunk"] {user-select: none} 
#${getId()} [data-state="widget"] {user-select: none}`;
    } else {
      ele.textContent = "";
    }
  };
  const onMouseDown = (e) => {
    let ele = e.target;
    if (ele && (ele == null ? void 0 : ele.nodeName) === "BUTTON") {
      removeAllSelection();
      return;
    }
    while (ele && ele instanceof HTMLElement) {
      const state = ele.getAttribute("data-state");
      if (state) {
        if (state === "extend" || state === "hunk" || state === "widget") {
          if (selectState.current !== false) {
            selectState.current = false;
            onSelect(false);
            removeAllSelection();
          }
        } else {
          if (selectState.current !== true) {
            selectState.current = true;
            onSelect(true);
            removeAllSelection();
          }
        }
        return;
      }
      ele = ele.parentElement;
    }
  };
  const font = solidJs.createMemo(() => ({
    fontSize: fontSize() + "px",
    fontFamily: "Menlo, Consolas, monospace"
  }));
  const width = useTextWidth({
    text: maxText,
    font
  });
  const computedWidth = solidJs.createMemo(() => Math.max(40, width() + 10));
  const getId = () => `diff-root${props.diffFile.getId()}`;
  return (() => {
    var _el$ = _tmpl$$1(), _el$2 = _el$.firstChild, _el$3 = _el$2.nextSibling, _el$4 = _el$3.firstChild, _el$5 = _el$4.firstChild, _el$6 = _el$5.nextSibling, _el$7 = _el$6.nextSibling;
    use(setStyleRef, _el$2);
    _el$3.style.setProperty("font-family", "Menlo, Consolas, monospace");
    `var(${diffFontSizeName})` != null ? _el$3.style.setProperty("font-size", `var(${diffFontSizeName})`) : _el$3.style.removeProperty("font-size");
    _el$7.$$mousedown = onMouseDown;
    insert(_el$7, solidJs.createComponent(solidJs.For, {
      get each() {
        return lines();
      },
      children: (item) => [solidJs.createComponent(DiffUnifiedHunkLine, {
        get index() {
          return item.index;
        },
        get lineNumber() {
          return item.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        }
      }), solidJs.createComponent(DiffUnifiedContentLine, {
        get index() {
          return item.index;
        },
        get lineNumber() {
          return item.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        }
      }), solidJs.createComponent(DiffUnifiedWidgetLine, {
        get index() {
          return item.index;
        },
        get lineNumber() {
          return item.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        }
      }), solidJs.createComponent(DiffUnifiedExtendLine, {
        get index() {
          return item.index;
        },
        get lineNumber() {
          return item.lineNumber;
        },
        get diffFile() {
          return props.diffFile;
        }
      })]
    }), null);
    insert(_el$7, solidJs.createComponent(DiffUnifiedHunkLine, {
      get index() {
        return props.diffFile.unifiedLineLength;
      },
      get lineNumber() {
        return props.diffFile.unifiedLineLength;
      },
      get diffFile() {
        return props.diffFile;
      }
    }), null);
    solidJs.createRenderEffect((_p$) => {
      var _v$ = `unified-diff-view ${enableWrap() ? "unified-diff-view-wrap" : "unified-diff-view-normal"} w-full`, _v$2 = {
        [diffAsideWidthName]: `${Math.round(computedWidth())}px`
      }, _v$3 = `unified-diff-table w-full border-collapse border-spacing-0 ${enableWrap() ? "table-fixed" : ""}`;
      _v$ !== _p$.e && className(_el$, _p$.e = _v$);
      _p$.t = style(_el$3, _v$2, _p$.t);
      _v$3 !== _p$.a && className(_el$4, _p$.a = _v$3);
      return _p$;
    }, {
      e: void 0,
      t: void 0,
      a: void 0
    });
    return _el$;
  })();
};
delegateEvents(["mousedown"]);
/**
* @vue/shared v3.5.16
* (c) 2018-present Yuxi (Evan) You and Vue contributors
* @license MIT
**/
/*! #__NO_SIDE_EFFECTS__ */
// @__NO_SIDE_EFFECTS__
function makeMap(str) {
  const map = /* @__PURE__ */ Object.create(null);
  for (const key of str.split(",")) map[key] = 1;
  return (val) => val in map;
}
!!(process.env.NODE_ENV !== "production") ? Object.freeze({}) : {};
!!(process.env.NODE_ENV !== "production") ? Object.freeze([]) : [];
const extend = Object.assign;
const hasOwnProperty$1 = Object.prototype.hasOwnProperty;
const hasOwn = (val, key) => hasOwnProperty$1.call(val, key);
const isArray = Array.isArray;
const isMap = (val) => toTypeString(val) === "[object Map]";
const isSet = (val) => toTypeString(val) === "[object Set]";
const isFunction = (val) => typeof val === "function";
const isString = (val) => typeof val === "string";
const isSymbol = (val) => typeof val === "symbol";
const isObject = (val) => val !== null && typeof val === "object";
const isPromise = (val) => {
  return (isObject(val) || isFunction(val)) && isFunction(val.then) && isFunction(val.catch);
};
const objectToString = Object.prototype.toString;
const toTypeString = (value) => objectToString.call(value);
const toRawType = (value) => {
  return toTypeString(value).slice(8, -1);
};
const isPlainObject = (val) => toTypeString(val) === "[object Object]";
const isIntegerKey = (key) => isString(key) && key !== "NaN" && key[0] !== "-" && "" + parseInt(key, 10) === key;
const cacheStringFunction = (fn) => {
  const cache = /* @__PURE__ */ Object.create(null);
  return (str) => {
    const hit = cache[str];
    return hit || (cache[str] = fn(str));
  };
};
const capitalize = cacheStringFunction((str) => {
  return str.charAt(0).toUpperCase() + str.slice(1);
});
const hasChanged = (value, oldValue) => !Object.is(value, oldValue);
/**
* @vue/reactivity v3.5.16
* (c) 2018-present Yuxi (Evan) You and Vue contributors
* @license MIT
**/
function warn(msg, ...args) {
  console.warn(`[Vue warn] ${msg}`, ...args);
}
let activeEffectScope;
class EffectScope {
  constructor(detached = false) {
    this.detached = detached;
    this._active = true;
    this._on = 0;
    this.effects = [];
    this.cleanups = [];
    this._isPaused = false;
    this.parent = activeEffectScope;
    if (!detached && activeEffectScope) {
      this.index = (activeEffectScope.scopes || (activeEffectScope.scopes = [])).push(
        this
      ) - 1;
    }
  }
  get active() {
    return this._active;
  }
  pause() {
    if (this._active) {
      this._isPaused = true;
      let i, l;
      if (this.scopes) {
        for (i = 0, l = this.scopes.length; i < l; i++) {
          this.scopes[i].pause();
        }
      }
      for (i = 0, l = this.effects.length; i < l; i++) {
        this.effects[i].pause();
      }
    }
  }
  /**
   * Resumes the effect scope, including all child scopes and effects.
   */
  resume() {
    if (this._active) {
      if (this._isPaused) {
        this._isPaused = false;
        let i, l;
        if (this.scopes) {
          for (i = 0, l = this.scopes.length; i < l; i++) {
            this.scopes[i].resume();
          }
        }
        for (i = 0, l = this.effects.length; i < l; i++) {
          this.effects[i].resume();
        }
      }
    }
  }
  run(fn) {
    if (this._active) {
      const currentEffectScope = activeEffectScope;
      try {
        activeEffectScope = this;
        return fn();
      } finally {
        activeEffectScope = currentEffectScope;
      }
    } else if (!!(process.env.NODE_ENV !== "production")) {
      warn(`cannot run an inactive effect scope.`);
    }
  }
  /**
   * This should only be called on non-detached scopes
   * @internal
   */
  on() {
    if (++this._on === 1) {
      this.prevScope = activeEffectScope;
      activeEffectScope = this;
    }
  }
  /**
   * This should only be called on non-detached scopes
   * @internal
   */
  off() {
    if (this._on > 0 && --this._on === 0) {
      activeEffectScope = this.prevScope;
      this.prevScope = void 0;
    }
  }
  stop(fromParent) {
    if (this._active) {
      this._active = false;
      let i, l;
      for (i = 0, l = this.effects.length; i < l; i++) {
        this.effects[i].stop();
      }
      this.effects.length = 0;
      for (i = 0, l = this.cleanups.length; i < l; i++) {
        this.cleanups[i]();
      }
      this.cleanups.length = 0;
      if (this.scopes) {
        for (i = 0, l = this.scopes.length; i < l; i++) {
          this.scopes[i].stop(true);
        }
        this.scopes.length = 0;
      }
      if (!this.detached && this.parent && !fromParent) {
        const last = this.parent.scopes.pop();
        if (last && last !== this) {
          this.parent.scopes[this.index] = last;
          last.index = this.index;
        }
      }
      this.parent = void 0;
    }
  }
}
function effectScope(detached) {
  return new EffectScope(detached);
}
let activeSub;
const pausedQueueEffects = /* @__PURE__ */ new WeakSet();
class ReactiveEffect {
  constructor(fn) {
    this.fn = fn;
    this.deps = void 0;
    this.depsTail = void 0;
    this.flags = 1 | 4;
    this.next = void 0;
    this.cleanup = void 0;
    this.scheduler = void 0;
    if (activeEffectScope && activeEffectScope.active) {
      activeEffectScope.effects.push(this);
    }
  }
  pause() {
    this.flags |= 64;
  }
  resume() {
    if (this.flags & 64) {
      this.flags &= -65;
      if (pausedQueueEffects.has(this)) {
        pausedQueueEffects.delete(this);
        this.trigger();
      }
    }
  }
  /**
   * @internal
   */
  notify() {
    if (this.flags & 2 && !(this.flags & 32)) {
      return;
    }
    if (!(this.flags & 8)) {
      batch(this);
    }
  }
  run() {
    if (!(this.flags & 1)) {
      return this.fn();
    }
    this.flags |= 2;
    cleanupEffect(this);
    prepareDeps(this);
    const prevEffect = activeSub;
    const prevShouldTrack = shouldTrack;
    activeSub = this;
    shouldTrack = true;
    try {
      return this.fn();
    } finally {
      if (!!(process.env.NODE_ENV !== "production") && activeSub !== this) {
        warn(
          "Active effect was not restored correctly - this is likely a Vue internal bug."
        );
      }
      cleanupDeps(this);
      activeSub = prevEffect;
      shouldTrack = prevShouldTrack;
      this.flags &= -3;
    }
  }
  stop() {
    if (this.flags & 1) {
      for (let link = this.deps; link; link = link.nextDep) {
        removeSub(link);
      }
      this.deps = this.depsTail = void 0;
      cleanupEffect(this);
      this.onStop && this.onStop();
      this.flags &= -2;
    }
  }
  trigger() {
    if (this.flags & 64) {
      pausedQueueEffects.add(this);
    } else if (this.scheduler) {
      this.scheduler();
    } else {
      this.runIfDirty();
    }
  }
  /**
   * @internal
   */
  runIfDirty() {
    if (isDirty(this)) {
      this.run();
    }
  }
  get dirty() {
    return isDirty(this);
  }
}
let batchDepth = 0;
let batchedSub;
let batchedComputed;
function batch(sub, isComputed = false) {
  sub.flags |= 8;
  if (isComputed) {
    sub.next = batchedComputed;
    batchedComputed = sub;
    return;
  }
  sub.next = batchedSub;
  batchedSub = sub;
}
function startBatch() {
  batchDepth++;
}
function endBatch() {
  if (--batchDepth > 0) {
    return;
  }
  if (batchedComputed) {
    let e = batchedComputed;
    batchedComputed = void 0;
    while (e) {
      const next = e.next;
      e.next = void 0;
      e.flags &= -9;
      e = next;
    }
  }
  let error;
  while (batchedSub) {
    let e = batchedSub;
    batchedSub = void 0;
    while (e) {
      const next = e.next;
      e.next = void 0;
      e.flags &= -9;
      if (e.flags & 1) {
        try {
          ;
          e.trigger();
        } catch (err) {
          if (!error) error = err;
        }
      }
      e = next;
    }
  }
  if (error) throw error;
}
function prepareDeps(sub) {
  for (let link = sub.deps; link; link = link.nextDep) {
    link.version = -1;
    link.prevActiveLink = link.dep.activeLink;
    link.dep.activeLink = link;
  }
}
function cleanupDeps(sub) {
  let head;
  let tail = sub.depsTail;
  let link = tail;
  while (link) {
    const prev = link.prevDep;
    if (link.version === -1) {
      if (link === tail) tail = prev;
      removeSub(link);
      removeDep(link);
    } else {
      head = link;
    }
    link.dep.activeLink = link.prevActiveLink;
    link.prevActiveLink = void 0;
    link = prev;
  }
  sub.deps = head;
  sub.depsTail = tail;
}
function isDirty(sub) {
  for (let link = sub.deps; link; link = link.nextDep) {
    if (link.dep.version !== link.version || link.dep.computed && (refreshComputed(link.dep.computed) || link.dep.version !== link.version)) {
      return true;
    }
  }
  if (sub._dirty) {
    return true;
  }
  return false;
}
function refreshComputed(computed) {
  if (computed.flags & 4 && !(computed.flags & 16)) {
    return;
  }
  computed.flags &= -17;
  if (computed.globalVersion === globalVersion) {
    return;
  }
  computed.globalVersion = globalVersion;
  if (!computed.isSSR && computed.flags & 128 && (!computed.deps && !computed._dirty || !isDirty(computed))) {
    return;
  }
  computed.flags |= 2;
  const dep = computed.dep;
  const prevSub = activeSub;
  const prevShouldTrack = shouldTrack;
  activeSub = computed;
  shouldTrack = true;
  try {
    prepareDeps(computed);
    const value = computed.fn(computed._value);
    if (dep.version === 0 || hasChanged(value, computed._value)) {
      computed.flags |= 128;
      computed._value = value;
      dep.version++;
    }
  } catch (err) {
    dep.version++;
    throw err;
  } finally {
    activeSub = prevSub;
    shouldTrack = prevShouldTrack;
    cleanupDeps(computed);
    computed.flags &= -3;
  }
}
function removeSub(link, soft = false) {
  const { dep, prevSub, nextSub } = link;
  if (prevSub) {
    prevSub.nextSub = nextSub;
    link.prevSub = void 0;
  }
  if (nextSub) {
    nextSub.prevSub = prevSub;
    link.nextSub = void 0;
  }
  if (!!(process.env.NODE_ENV !== "production") && dep.subsHead === link) {
    dep.subsHead = nextSub;
  }
  if (dep.subs === link) {
    dep.subs = prevSub;
    if (!prevSub && dep.computed) {
      dep.computed.flags &= -5;
      for (let l = dep.computed.deps; l; l = l.nextDep) {
        removeSub(l, true);
      }
    }
  }
  if (!soft && !--dep.sc && dep.map) {
    dep.map.delete(dep.key);
  }
}
function removeDep(link) {
  const { prevDep, nextDep } = link;
  if (prevDep) {
    prevDep.nextDep = nextDep;
    link.prevDep = void 0;
  }
  if (nextDep) {
    nextDep.prevDep = prevDep;
    link.nextDep = void 0;
  }
}
let shouldTrack = true;
const trackStack = [];
function pauseTracking() {
  trackStack.push(shouldTrack);
  shouldTrack = false;
}
function resetTracking() {
  const last = trackStack.pop();
  shouldTrack = last === void 0 ? true : last;
}
function cleanupEffect(e) {
  const { cleanup } = e;
  e.cleanup = void 0;
  if (cleanup) {
    const prevSub = activeSub;
    activeSub = void 0;
    try {
      cleanup();
    } finally {
      activeSub = prevSub;
    }
  }
}
let globalVersion = 0;
class Link {
  constructor(sub, dep) {
    this.sub = sub;
    this.dep = dep;
    this.version = dep.version;
    this.nextDep = this.prevDep = this.nextSub = this.prevSub = this.prevActiveLink = void 0;
  }
}
class Dep {
  constructor(computed) {
    this.computed = computed;
    this.version = 0;
    this.activeLink = void 0;
    this.subs = void 0;
    this.map = void 0;
    this.key = void 0;
    this.sc = 0;
    if (!!(process.env.NODE_ENV !== "production")) {
      this.subsHead = void 0;
    }
  }
  track(debugInfo) {
    if (!activeSub || !shouldTrack || activeSub === this.computed) {
      return;
    }
    let link = this.activeLink;
    if (link === void 0 || link.sub !== activeSub) {
      link = this.activeLink = new Link(activeSub, this);
      if (!activeSub.deps) {
        activeSub.deps = activeSub.depsTail = link;
      } else {
        link.prevDep = activeSub.depsTail;
        activeSub.depsTail.nextDep = link;
        activeSub.depsTail = link;
      }
      addSub(link);
    } else if (link.version === -1) {
      link.version = this.version;
      if (link.nextDep) {
        const next = link.nextDep;
        next.prevDep = link.prevDep;
        if (link.prevDep) {
          link.prevDep.nextDep = next;
        }
        link.prevDep = activeSub.depsTail;
        link.nextDep = void 0;
        activeSub.depsTail.nextDep = link;
        activeSub.depsTail = link;
        if (activeSub.deps === link) {
          activeSub.deps = next;
        }
      }
    }
    if (!!(process.env.NODE_ENV !== "production") && activeSub.onTrack) {
      activeSub.onTrack(
        extend(
          {
            effect: activeSub
          },
          debugInfo
        )
      );
    }
    return link;
  }
  trigger(debugInfo) {
    this.version++;
    globalVersion++;
    this.notify(debugInfo);
  }
  notify(debugInfo) {
    startBatch();
    try {
      if (!!(process.env.NODE_ENV !== "production")) {
        for (let head = this.subsHead; head; head = head.nextSub) {
          if (head.sub.onTrigger && !(head.sub.flags & 8)) {
            head.sub.onTrigger(
              extend(
                {
                  effect: head.sub
                },
                debugInfo
              )
            );
          }
        }
      }
      for (let link = this.subs; link; link = link.prevSub) {
        if (link.sub.notify()) {
          ;
          link.sub.dep.notify();
        }
      }
    } finally {
      endBatch();
    }
  }
}
function addSub(link) {
  link.dep.sc++;
  if (link.sub.flags & 4) {
    const computed = link.dep.computed;
    if (computed && !link.dep.subs) {
      computed.flags |= 4 | 16;
      for (let l = computed.deps; l; l = l.nextDep) {
        addSub(l);
      }
    }
    const currentTail = link.dep.subs;
    if (currentTail !== link) {
      link.prevSub = currentTail;
      if (currentTail) currentTail.nextSub = link;
    }
    if (!!(process.env.NODE_ENV !== "production") && link.dep.subsHead === void 0) {
      link.dep.subsHead = link;
    }
    link.dep.subs = link;
  }
}
const targetMap = /* @__PURE__ */ new WeakMap();
const ITERATE_KEY = Symbol(
  !!(process.env.NODE_ENV !== "production") ? "Object iterate" : ""
);
const MAP_KEY_ITERATE_KEY = Symbol(
  !!(process.env.NODE_ENV !== "production") ? "Map keys iterate" : ""
);
const ARRAY_ITERATE_KEY = Symbol(
  !!(process.env.NODE_ENV !== "production") ? "Array iterate" : ""
);
function track(target, type, key) {
  if (shouldTrack && activeSub) {
    let depsMap = targetMap.get(target);
    if (!depsMap) {
      targetMap.set(target, depsMap = /* @__PURE__ */ new Map());
    }
    let dep = depsMap.get(key);
    if (!dep) {
      depsMap.set(key, dep = new Dep());
      dep.map = depsMap;
      dep.key = key;
    }
    if (!!(process.env.NODE_ENV !== "production")) {
      dep.track({
        target,
        type,
        key
      });
    } else {
      dep.track();
    }
  }
}
function trigger(target, type, key, newValue, oldValue, oldTarget) {
  const depsMap = targetMap.get(target);
  if (!depsMap) {
    globalVersion++;
    return;
  }
  const run = (dep) => {
    if (dep) {
      if (!!(process.env.NODE_ENV !== "production")) {
        dep.trigger({
          target,
          type,
          key,
          newValue,
          oldValue,
          oldTarget
        });
      } else {
        dep.trigger();
      }
    }
  };
  startBatch();
  if (type === "clear") {
    depsMap.forEach(run);
  } else {
    const targetIsArray = isArray(target);
    const isArrayIndex = targetIsArray && isIntegerKey(key);
    if (targetIsArray && key === "length") {
      const newLength = Number(newValue);
      depsMap.forEach((dep, key2) => {
        if (key2 === "length" || key2 === ARRAY_ITERATE_KEY || !isSymbol(key2) && key2 >= newLength) {
          run(dep);
        }
      });
    } else {
      if (key !== void 0 || depsMap.has(void 0)) {
        run(depsMap.get(key));
      }
      if (isArrayIndex) {
        run(depsMap.get(ARRAY_ITERATE_KEY));
      }
      switch (type) {
        case "add":
          if (!targetIsArray) {
            run(depsMap.get(ITERATE_KEY));
            if (isMap(target)) {
              run(depsMap.get(MAP_KEY_ITERATE_KEY));
            }
          } else if (isArrayIndex) {
            run(depsMap.get("length"));
          }
          break;
        case "delete":
          if (!targetIsArray) {
            run(depsMap.get(ITERATE_KEY));
            if (isMap(target)) {
              run(depsMap.get(MAP_KEY_ITERATE_KEY));
            }
          }
          break;
        case "set":
          if (isMap(target)) {
            run(depsMap.get(ITERATE_KEY));
          }
          break;
      }
    }
  }
  endBatch();
}
function reactiveReadArray(array) {
  const raw = toRaw(array);
  if (raw === array) return raw;
  track(raw, "iterate", ARRAY_ITERATE_KEY);
  return isShallow(array) ? raw : raw.map(toReactive);
}
function shallowReadArray(arr) {
  track(arr = toRaw(arr), "iterate", ARRAY_ITERATE_KEY);
  return arr;
}
const arrayInstrumentations = {
  __proto__: null,
  [Symbol.iterator]() {
    return iterator(this, Symbol.iterator, toReactive);
  },
  concat(...args) {
    return reactiveReadArray(this).concat(
      ...args.map((x) => isArray(x) ? reactiveReadArray(x) : x)
    );
  },
  entries() {
    return iterator(this, "entries", (value) => {
      value[1] = toReactive(value[1]);
      return value;
    });
  },
  every(fn, thisArg) {
    return apply(this, "every", fn, thisArg, void 0, arguments);
  },
  filter(fn, thisArg) {
    return apply(this, "filter", fn, thisArg, (v) => v.map(toReactive), arguments);
  },
  find(fn, thisArg) {
    return apply(this, "find", fn, thisArg, toReactive, arguments);
  },
  findIndex(fn, thisArg) {
    return apply(this, "findIndex", fn, thisArg, void 0, arguments);
  },
  findLast(fn, thisArg) {
    return apply(this, "findLast", fn, thisArg, toReactive, arguments);
  },
  findLastIndex(fn, thisArg) {
    return apply(this, "findLastIndex", fn, thisArg, void 0, arguments);
  },
  // flat, flatMap could benefit from ARRAY_ITERATE but are not straight-forward to implement
  forEach(fn, thisArg) {
    return apply(this, "forEach", fn, thisArg, void 0, arguments);
  },
  includes(...args) {
    return searchProxy(this, "includes", args);
  },
  indexOf(...args) {
    return searchProxy(this, "indexOf", args);
  },
  join(separator) {
    return reactiveReadArray(this).join(separator);
  },
  // keys() iterator only reads `length`, no optimisation required
  lastIndexOf(...args) {
    return searchProxy(this, "lastIndexOf", args);
  },
  map(fn, thisArg) {
    return apply(this, "map", fn, thisArg, void 0, arguments);
  },
  pop() {
    return noTracking(this, "pop");
  },
  push(...args) {
    return noTracking(this, "push", args);
  },
  reduce(fn, ...args) {
    return reduce(this, "reduce", fn, args);
  },
  reduceRight(fn, ...args) {
    return reduce(this, "reduceRight", fn, args);
  },
  shift() {
    return noTracking(this, "shift");
  },
  // slice could use ARRAY_ITERATE but also seems to beg for range tracking
  some(fn, thisArg) {
    return apply(this, "some", fn, thisArg, void 0, arguments);
  },
  splice(...args) {
    return noTracking(this, "splice", args);
  },
  toReversed() {
    return reactiveReadArray(this).toReversed();
  },
  toSorted(comparer) {
    return reactiveReadArray(this).toSorted(comparer);
  },
  toSpliced(...args) {
    return reactiveReadArray(this).toSpliced(...args);
  },
  unshift(...args) {
    return noTracking(this, "unshift", args);
  },
  values() {
    return iterator(this, "values", toReactive);
  }
};
function iterator(self, method, wrapValue) {
  const arr = shallowReadArray(self);
  const iter = arr[method]();
  if (arr !== self && !isShallow(self)) {
    iter._next = iter.next;
    iter.next = () => {
      const result = iter._next();
      if (result.value) {
        result.value = wrapValue(result.value);
      }
      return result;
    };
  }
  return iter;
}
const arrayProto = Array.prototype;
function apply(self, method, fn, thisArg, wrappedRetFn, args) {
  const arr = shallowReadArray(self);
  const needsWrap = arr !== self && !isShallow(self);
  const methodFn = arr[method];
  if (methodFn !== arrayProto[method]) {
    const result2 = methodFn.apply(self, args);
    return needsWrap ? toReactive(result2) : result2;
  }
  let wrappedFn = fn;
  if (arr !== self) {
    if (needsWrap) {
      wrappedFn = function(item, index) {
        return fn.call(this, toReactive(item), index, self);
      };
    } else if (fn.length > 2) {
      wrappedFn = function(item, index) {
        return fn.call(this, item, index, self);
      };
    }
  }
  const result = methodFn.call(arr, wrappedFn, thisArg);
  return needsWrap && wrappedRetFn ? wrappedRetFn(result) : result;
}
function reduce(self, method, fn, args) {
  const arr = shallowReadArray(self);
  let wrappedFn = fn;
  if (arr !== self) {
    if (!isShallow(self)) {
      wrappedFn = function(acc, item, index) {
        return fn.call(this, acc, toReactive(item), index, self);
      };
    } else if (fn.length > 3) {
      wrappedFn = function(acc, item, index) {
        return fn.call(this, acc, item, index, self);
      };
    }
  }
  return arr[method](wrappedFn, ...args);
}
function searchProxy(self, method, args) {
  const arr = toRaw(self);
  track(arr, "iterate", ARRAY_ITERATE_KEY);
  const res = arr[method](...args);
  if ((res === -1 || res === false) && isProxy(args[0])) {
    args[0] = toRaw(args[0]);
    return arr[method](...args);
  }
  return res;
}
function noTracking(self, method, args = []) {
  pauseTracking();
  startBatch();
  const res = toRaw(self)[method].apply(self, args);
  endBatch();
  resetTracking();
  return res;
}
const isNonTrackableKeys = /* @__PURE__ */ makeMap(`__proto__,__v_isRef,__isVue`);
const builtInSymbols = new Set(
  /* @__PURE__ */ Object.getOwnPropertyNames(Symbol).filter((key) => key !== "arguments" && key !== "caller").map((key) => Symbol[key]).filter(isSymbol)
);
function hasOwnProperty(key) {
  if (!isSymbol(key)) key = String(key);
  const obj = toRaw(this);
  track(obj, "has", key);
  return obj.hasOwnProperty(key);
}
class BaseReactiveHandler {
  constructor(_isReadonly = false, _isShallow = false) {
    this._isReadonly = _isReadonly;
    this._isShallow = _isShallow;
  }
  get(target, key, receiver) {
    if (key === "__v_skip") return target["__v_skip"];
    const isReadonly2 = this._isReadonly, isShallow2 = this._isShallow;
    if (key === "__v_isReactive") {
      return !isReadonly2;
    } else if (key === "__v_isReadonly") {
      return isReadonly2;
    } else if (key === "__v_isShallow") {
      return isShallow2;
    } else if (key === "__v_raw") {
      if (receiver === (isReadonly2 ? isShallow2 ? shallowReadonlyMap : readonlyMap : isShallow2 ? shallowReactiveMap : reactiveMap).get(target) || // receiver is not the reactive proxy, but has the same prototype
      // this means the receiver is a user proxy of the reactive proxy
      Object.getPrototypeOf(target) === Object.getPrototypeOf(receiver)) {
        return target;
      }
      return;
    }
    const targetIsArray = isArray(target);
    if (!isReadonly2) {
      let fn;
      if (targetIsArray && (fn = arrayInstrumentations[key])) {
        return fn;
      }
      if (key === "hasOwnProperty") {
        return hasOwnProperty;
      }
    }
    const res = Reflect.get(
      target,
      key,
      // if this is a proxy wrapping a ref, return methods using the raw ref
      // as receiver so that we don't have to call `toRaw` on the ref in all
      // its class methods
      isRef(target) ? target : receiver
    );
    if (isSymbol(key) ? builtInSymbols.has(key) : isNonTrackableKeys(key)) {
      return res;
    }
    if (!isReadonly2) {
      track(target, "get", key);
    }
    if (isShallow2) {
      return res;
    }
    if (isRef(res)) {
      return targetIsArray && isIntegerKey(key) ? res : res.value;
    }
    if (isObject(res)) {
      return isReadonly2 ? readonly(res) : reactive(res);
    }
    return res;
  }
}
class MutableReactiveHandler extends BaseReactiveHandler {
  constructor(isShallow2 = false) {
    super(false, isShallow2);
  }
  set(target, key, value, receiver) {
    let oldValue = target[key];
    if (!this._isShallow) {
      const isOldValueReadonly = isReadonly(oldValue);
      if (!isShallow(value) && !isReadonly(value)) {
        oldValue = toRaw(oldValue);
        value = toRaw(value);
      }
      if (!isArray(target) && isRef(oldValue) && !isRef(value)) {
        if (isOldValueReadonly) {
          return false;
        } else {
          oldValue.value = value;
          return true;
        }
      }
    }
    const hadKey = isArray(target) && isIntegerKey(key) ? Number(key) < target.length : hasOwn(target, key);
    const result = Reflect.set(
      target,
      key,
      value,
      isRef(target) ? target : receiver
    );
    if (target === toRaw(receiver)) {
      if (!hadKey) {
        trigger(target, "add", key, value);
      } else if (hasChanged(value, oldValue)) {
        trigger(target, "set", key, value, oldValue);
      }
    }
    return result;
  }
  deleteProperty(target, key) {
    const hadKey = hasOwn(target, key);
    const oldValue = target[key];
    const result = Reflect.deleteProperty(target, key);
    if (result && hadKey) {
      trigger(target, "delete", key, void 0, oldValue);
    }
    return result;
  }
  has(target, key) {
    const result = Reflect.has(target, key);
    if (!isSymbol(key) || !builtInSymbols.has(key)) {
      track(target, "has", key);
    }
    return result;
  }
  ownKeys(target) {
    track(
      target,
      "iterate",
      isArray(target) ? "length" : ITERATE_KEY
    );
    return Reflect.ownKeys(target);
  }
}
class ReadonlyReactiveHandler extends BaseReactiveHandler {
  constructor(isShallow2 = false) {
    super(true, isShallow2);
  }
  set(target, key) {
    if (!!(process.env.NODE_ENV !== "production")) {
      warn(
        `Set operation on key "${String(key)}" failed: target is readonly.`,
        target
      );
    }
    return true;
  }
  deleteProperty(target, key) {
    if (!!(process.env.NODE_ENV !== "production")) {
      warn(
        `Delete operation on key "${String(key)}" failed: target is readonly.`,
        target
      );
    }
    return true;
  }
}
const mutableHandlers = /* @__PURE__ */ new MutableReactiveHandler();
const readonlyHandlers = /* @__PURE__ */ new ReadonlyReactiveHandler();
const toShallow = (value) => value;
const getProto = (v) => Reflect.getPrototypeOf(v);
function createIterableMethod(method, isReadonly2, isShallow2) {
  return function(...args) {
    const target = this["__v_raw"];
    const rawTarget = toRaw(target);
    const targetIsMap = isMap(rawTarget);
    const isPair = method === "entries" || method === Symbol.iterator && targetIsMap;
    const isKeyOnly = method === "keys" && targetIsMap;
    const innerIterator = target[method](...args);
    const wrap = isShallow2 ? toShallow : isReadonly2 ? toReadonly : toReactive;
    !isReadonly2 && track(
      rawTarget,
      "iterate",
      isKeyOnly ? MAP_KEY_ITERATE_KEY : ITERATE_KEY
    );
    return {
      // iterator protocol
      next() {
        const { value, done } = innerIterator.next();
        return done ? { value, done } : {
          value: isPair ? [wrap(value[0]), wrap(value[1])] : wrap(value),
          done
        };
      },
      // iterable protocol
      [Symbol.iterator]() {
        return this;
      }
    };
  };
}
function createReadonlyMethod(type) {
  return function(...args) {
    if (!!(process.env.NODE_ENV !== "production")) {
      const key = args[0] ? `on key "${args[0]}" ` : ``;
      warn(
        `${capitalize(type)} operation ${key}failed: target is readonly.`,
        toRaw(this)
      );
    }
    return type === "delete" ? false : type === "clear" ? void 0 : this;
  };
}
function createInstrumentations(readonly2, shallow) {
  const instrumentations = {
    get(key) {
      const target = this["__v_raw"];
      const rawTarget = toRaw(target);
      const rawKey = toRaw(key);
      if (!readonly2) {
        if (hasChanged(key, rawKey)) {
          track(rawTarget, "get", key);
        }
        track(rawTarget, "get", rawKey);
      }
      const { has } = getProto(rawTarget);
      const wrap = shallow ? toShallow : readonly2 ? toReadonly : toReactive;
      if (has.call(rawTarget, key)) {
        return wrap(target.get(key));
      } else if (has.call(rawTarget, rawKey)) {
        return wrap(target.get(rawKey));
      } else if (target !== rawTarget) {
        target.get(key);
      }
    },
    get size() {
      const target = this["__v_raw"];
      !readonly2 && track(toRaw(target), "iterate", ITERATE_KEY);
      return Reflect.get(target, "size", target);
    },
    has(key) {
      const target = this["__v_raw"];
      const rawTarget = toRaw(target);
      const rawKey = toRaw(key);
      if (!readonly2) {
        if (hasChanged(key, rawKey)) {
          track(rawTarget, "has", key);
        }
        track(rawTarget, "has", rawKey);
      }
      return key === rawKey ? target.has(key) : target.has(key) || target.has(rawKey);
    },
    forEach(callback, thisArg) {
      const observed = this;
      const target = observed["__v_raw"];
      const rawTarget = toRaw(target);
      const wrap = shallow ? toShallow : readonly2 ? toReadonly : toReactive;
      !readonly2 && track(rawTarget, "iterate", ITERATE_KEY);
      return target.forEach((value, key) => {
        return callback.call(thisArg, wrap(value), wrap(key), observed);
      });
    }
  };
  extend(
    instrumentations,
    readonly2 ? {
      add: createReadonlyMethod("add"),
      set: createReadonlyMethod("set"),
      delete: createReadonlyMethod("delete"),
      clear: createReadonlyMethod("clear")
    } : {
      add(value) {
        if (!shallow && !isShallow(value) && !isReadonly(value)) {
          value = toRaw(value);
        }
        const target = toRaw(this);
        const proto = getProto(target);
        const hadKey = proto.has.call(target, value);
        if (!hadKey) {
          target.add(value);
          trigger(target, "add", value, value);
        }
        return this;
      },
      set(key, value) {
        if (!shallow && !isShallow(value) && !isReadonly(value)) {
          value = toRaw(value);
        }
        const target = toRaw(this);
        const { has, get } = getProto(target);
        let hadKey = has.call(target, key);
        if (!hadKey) {
          key = toRaw(key);
          hadKey = has.call(target, key);
        } else if (!!(process.env.NODE_ENV !== "production")) {
          checkIdentityKeys(target, has, key);
        }
        const oldValue = get.call(target, key);
        target.set(key, value);
        if (!hadKey) {
          trigger(target, "add", key, value);
        } else if (hasChanged(value, oldValue)) {
          trigger(target, "set", key, value, oldValue);
        }
        return this;
      },
      delete(key) {
        const target = toRaw(this);
        const { has, get } = getProto(target);
        let hadKey = has.call(target, key);
        if (!hadKey) {
          key = toRaw(key);
          hadKey = has.call(target, key);
        } else if (!!(process.env.NODE_ENV !== "production")) {
          checkIdentityKeys(target, has, key);
        }
        const oldValue = get ? get.call(target, key) : void 0;
        const result = target.delete(key);
        if (hadKey) {
          trigger(target, "delete", key, void 0, oldValue);
        }
        return result;
      },
      clear() {
        const target = toRaw(this);
        const hadItems = target.size !== 0;
        const oldTarget = !!(process.env.NODE_ENV !== "production") ? isMap(target) ? new Map(target) : new Set(target) : void 0;
        const result = target.clear();
        if (hadItems) {
          trigger(
            target,
            "clear",
            void 0,
            void 0,
            oldTarget
          );
        }
        return result;
      }
    }
  );
  const iteratorMethods = [
    "keys",
    "values",
    "entries",
    Symbol.iterator
  ];
  iteratorMethods.forEach((method) => {
    instrumentations[method] = createIterableMethod(method, readonly2, shallow);
  });
  return instrumentations;
}
function createInstrumentationGetter(isReadonly2, shallow) {
  const instrumentations = createInstrumentations(isReadonly2, shallow);
  return (target, key, receiver) => {
    if (key === "__v_isReactive") {
      return !isReadonly2;
    } else if (key === "__v_isReadonly") {
      return isReadonly2;
    } else if (key === "__v_raw") {
      return target;
    }
    return Reflect.get(
      hasOwn(instrumentations, key) && key in target ? instrumentations : target,
      key,
      receiver
    );
  };
}
const mutableCollectionHandlers = {
  get: /* @__PURE__ */ createInstrumentationGetter(false, false)
};
const readonlyCollectionHandlers = {
  get: /* @__PURE__ */ createInstrumentationGetter(true, false)
};
function checkIdentityKeys(target, has, key) {
  const rawKey = toRaw(key);
  if (rawKey !== key && has.call(target, rawKey)) {
    const type = toRawType(target);
    warn(
      `Reactive ${type} contains both the raw and reactive versions of the same object${type === `Map` ? ` as keys` : ``}, which can lead to inconsistencies. Avoid differentiating between the raw and reactive versions of an object and only use the reactive version if possible.`
    );
  }
}
const reactiveMap = /* @__PURE__ */ new WeakMap();
const shallowReactiveMap = /* @__PURE__ */ new WeakMap();
const readonlyMap = /* @__PURE__ */ new WeakMap();
const shallowReadonlyMap = /* @__PURE__ */ new WeakMap();
function targetTypeMap(rawType) {
  switch (rawType) {
    case "Object":
    case "Array":
      return 1;
    case "Map":
    case "Set":
    case "WeakMap":
    case "WeakSet":
      return 2;
    default:
      return 0;
  }
}
function getTargetType(value) {
  return value["__v_skip"] || !Object.isExtensible(value) ? 0 : targetTypeMap(toRawType(value));
}
function reactive(target) {
  if (isReadonly(target)) {
    return target;
  }
  return createReactiveObject(
    target,
    false,
    mutableHandlers,
    mutableCollectionHandlers,
    reactiveMap
  );
}
function readonly(target) {
  return createReactiveObject(
    target,
    true,
    readonlyHandlers,
    readonlyCollectionHandlers,
    readonlyMap
  );
}
function createReactiveObject(target, isReadonly2, baseHandlers, collectionHandlers, proxyMap) {
  if (!isObject(target)) {
    if (!!(process.env.NODE_ENV !== "production")) {
      warn(
        `value cannot be made ${isReadonly2 ? "readonly" : "reactive"}: ${String(
          target
        )}`
      );
    }
    return target;
  }
  if (target["__v_raw"] && !(isReadonly2 && target["__v_isReactive"])) {
    return target;
  }
  const targetType = getTargetType(target);
  if (targetType === 0) {
    return target;
  }
  const existingProxy = proxyMap.get(target);
  if (existingProxy) {
    return existingProxy;
  }
  const proxy = new Proxy(
    target,
    targetType === 2 ? collectionHandlers : baseHandlers
  );
  proxyMap.set(target, proxy);
  return proxy;
}
function isReactive(value) {
  if (isReadonly(value)) {
    return isReactive(value["__v_raw"]);
  }
  return !!(value && value["__v_isReactive"]);
}
function isReadonly(value) {
  return !!(value && value["__v_isReadonly"]);
}
function isShallow(value) {
  return !!(value && value["__v_isShallow"]);
}
function isProxy(value) {
  return value ? !!value["__v_raw"] : false;
}
function toRaw(observed) {
  const raw = observed && observed["__v_raw"];
  return raw ? toRaw(raw) : observed;
}
const toReactive = (value) => isObject(value) ? reactive(value) : value;
const toReadonly = (value) => isObject(value) ? readonly(value) : value;
function isRef(r) {
  return r ? r["__v_isRef"] === true : false;
}
function ref(value) {
  return createRef(value, false);
}
function createRef(rawValue, shallow) {
  if (isRef(rawValue)) {
    return rawValue;
  }
  return new RefImpl(rawValue, shallow);
}
class RefImpl {
  constructor(value, isShallow2) {
    this.dep = new Dep();
    this["__v_isRef"] = true;
    this["__v_isShallow"] = false;
    this._rawValue = isShallow2 ? value : toRaw(value);
    this._value = isShallow2 ? value : toReactive(value);
    this["__v_isShallow"] = isShallow2;
  }
  get value() {
    if (!!(process.env.NODE_ENV !== "production")) {
      this.dep.track({
        target: this,
        type: "get",
        key: "value"
      });
    } else {
      this.dep.track();
    }
    return this._value;
  }
  set value(newValue) {
    const oldValue = this._rawValue;
    const useDirectValue = this["__v_isShallow"] || isShallow(newValue) || isReadonly(newValue);
    newValue = useDirectValue ? newValue : toRaw(newValue);
    if (hasChanged(newValue, oldValue)) {
      this._rawValue = newValue;
      this._value = useDirectValue ? newValue : toReactive(newValue);
      if (!!(process.env.NODE_ENV !== "production")) {
        this.dep.trigger({
          target: this,
          type: "set",
          key: "value",
          newValue,
          oldValue
        });
      } else {
        this.dep.trigger();
      }
    }
  }
}
const ReactiveFlags = {
  "SKIP": "__v_skip"
};
var react = { exports: {} };
var react_production = {};
/**
 * @license React
 * react.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
var hasRequiredReact_production;
function requireReact_production() {
  if (hasRequiredReact_production) return react_production;
  hasRequiredReact_production = 1;
  var REACT_ELEMENT_TYPE = Symbol.for("react.transitional.element"), REACT_PORTAL_TYPE = Symbol.for("react.portal"), REACT_FRAGMENT_TYPE = Symbol.for("react.fragment"), REACT_STRICT_MODE_TYPE = Symbol.for("react.strict_mode"), REACT_PROFILER_TYPE = Symbol.for("react.profiler"), REACT_CONSUMER_TYPE = Symbol.for("react.consumer"), REACT_CONTEXT_TYPE = Symbol.for("react.context"), REACT_FORWARD_REF_TYPE = Symbol.for("react.forward_ref"), REACT_SUSPENSE_TYPE = Symbol.for("react.suspense"), REACT_MEMO_TYPE = Symbol.for("react.memo"), REACT_LAZY_TYPE = Symbol.for("react.lazy"), MAYBE_ITERATOR_SYMBOL = Symbol.iterator;
  function getIteratorFn(maybeIterable) {
    if (null === maybeIterable || "object" !== typeof maybeIterable) return null;
    maybeIterable = MAYBE_ITERATOR_SYMBOL && maybeIterable[MAYBE_ITERATOR_SYMBOL] || maybeIterable["@@iterator"];
    return "function" === typeof maybeIterable ? maybeIterable : null;
  }
  var ReactNoopUpdateQueue = {
    isMounted: function() {
      return false;
    },
    enqueueForceUpdate: function() {
    },
    enqueueReplaceState: function() {
    },
    enqueueSetState: function() {
    }
  }, assign = Object.assign, emptyObject = {};
  function Component(props, context, updater) {
    this.props = props;
    this.context = context;
    this.refs = emptyObject;
    this.updater = updater || ReactNoopUpdateQueue;
  }
  Component.prototype.isReactComponent = {};
  Component.prototype.setState = function(partialState, callback) {
    if ("object" !== typeof partialState && "function" !== typeof partialState && null != partialState)
      throw Error(
        "takes an object of state variables to update or a function which returns an object of state variables."
      );
    this.updater.enqueueSetState(this, partialState, callback, "setState");
  };
  Component.prototype.forceUpdate = function(callback) {
    this.updater.enqueueForceUpdate(this, callback, "forceUpdate");
  };
  function ComponentDummy() {
  }
  ComponentDummy.prototype = Component.prototype;
  function PureComponent(props, context, updater) {
    this.props = props;
    this.context = context;
    this.refs = emptyObject;
    this.updater = updater || ReactNoopUpdateQueue;
  }
  var pureComponentPrototype = PureComponent.prototype = new ComponentDummy();
  pureComponentPrototype.constructor = PureComponent;
  assign(pureComponentPrototype, Component.prototype);
  pureComponentPrototype.isPureReactComponent = true;
  var isArrayImpl = Array.isArray, ReactSharedInternals = { H: null, A: null, T: null, S: null }, hasOwnProperty2 = Object.prototype.hasOwnProperty;
  function ReactElement(type, key, self, source, owner, props) {
    self = props.ref;
    return {
      $$typeof: REACT_ELEMENT_TYPE,
      type,
      key,
      ref: void 0 !== self ? self : null,
      props
    };
  }
  function cloneAndReplaceKey(oldElement, newKey) {
    return ReactElement(
      oldElement.type,
      newKey,
      void 0,
      void 0,
      void 0,
      oldElement.props
    );
  }
  function isValidElement(object) {
    return "object" === typeof object && null !== object && object.$$typeof === REACT_ELEMENT_TYPE;
  }
  function escape(key) {
    var escaperLookup = { "=": "=0", ":": "=2" };
    return "$" + key.replace(/[=:]/g, function(match) {
      return escaperLookup[match];
    });
  }
  var userProvidedKeyEscapeRegex = /\/+/g;
  function getElementKey(element, index) {
    return "object" === typeof element && null !== element && null != element.key ? escape("" + element.key) : index.toString(36);
  }
  function noop$1() {
  }
  function resolveThenable(thenable) {
    switch (thenable.status) {
      case "fulfilled":
        return thenable.value;
      case "rejected":
        throw thenable.reason;
      default:
        switch ("string" === typeof thenable.status ? thenable.then(noop$1, noop$1) : (thenable.status = "pending", thenable.then(
          function(fulfilledValue) {
            "pending" === thenable.status && (thenable.status = "fulfilled", thenable.value = fulfilledValue);
          },
          function(error) {
            "pending" === thenable.status && (thenable.status = "rejected", thenable.reason = error);
          }
        )), thenable.status) {
          case "fulfilled":
            return thenable.value;
          case "rejected":
            throw thenable.reason;
        }
    }
    throw thenable;
  }
  function mapIntoArray(children, array, escapedPrefix, nameSoFar, callback) {
    var type = typeof children;
    if ("undefined" === type || "boolean" === type) children = null;
    var invokeCallback = false;
    if (null === children) invokeCallback = true;
    else
      switch (type) {
        case "bigint":
        case "string":
        case "number":
          invokeCallback = true;
          break;
        case "object":
          switch (children.$$typeof) {
            case REACT_ELEMENT_TYPE:
            case REACT_PORTAL_TYPE:
              invokeCallback = true;
              break;
            case REACT_LAZY_TYPE:
              return invokeCallback = children._init, mapIntoArray(
                invokeCallback(children._payload),
                array,
                escapedPrefix,
                nameSoFar,
                callback
              );
          }
      }
    if (invokeCallback)
      return callback = callback(children), invokeCallback = "" === nameSoFar ? "." + getElementKey(children, 0) : nameSoFar, isArrayImpl(callback) ? (escapedPrefix = "", null != invokeCallback && (escapedPrefix = invokeCallback.replace(userProvidedKeyEscapeRegex, "$&/") + "/"), mapIntoArray(callback, array, escapedPrefix, "", function(c) {
        return c;
      })) : null != callback && (isValidElement(callback) && (callback = cloneAndReplaceKey(
        callback,
        escapedPrefix + (null == callback.key || children && children.key === callback.key ? "" : ("" + callback.key).replace(
          userProvidedKeyEscapeRegex,
          "$&/"
        ) + "/") + invokeCallback
      )), array.push(callback)), 1;
    invokeCallback = 0;
    var nextNamePrefix = "" === nameSoFar ? "." : nameSoFar + ":";
    if (isArrayImpl(children))
      for (var i = 0; i < children.length; i++)
        nameSoFar = children[i], type = nextNamePrefix + getElementKey(nameSoFar, i), invokeCallback += mapIntoArray(
          nameSoFar,
          array,
          escapedPrefix,
          type,
          callback
        );
    else if (i = getIteratorFn(children), "function" === typeof i)
      for (children = i.call(children), i = 0; !(nameSoFar = children.next()).done; )
        nameSoFar = nameSoFar.value, type = nextNamePrefix + getElementKey(nameSoFar, i++), invokeCallback += mapIntoArray(
          nameSoFar,
          array,
          escapedPrefix,
          type,
          callback
        );
    else if ("object" === type) {
      if ("function" === typeof children.then)
        return mapIntoArray(
          resolveThenable(children),
          array,
          escapedPrefix,
          nameSoFar,
          callback
        );
      array = String(children);
      throw Error(
        "Objects are not valid as a React child (found: " + ("[object Object]" === array ? "object with keys {" + Object.keys(children).join(", ") + "}" : array) + "). If you meant to render a collection of children, use an array instead."
      );
    }
    return invokeCallback;
  }
  function mapChildren(children, func, context) {
    if (null == children) return children;
    var result = [], count = 0;
    mapIntoArray(children, result, "", "", function(child) {
      return func.call(context, child, count++);
    });
    return result;
  }
  function lazyInitializer(payload) {
    if (-1 === payload._status) {
      var ctor = payload._result;
      ctor = ctor();
      ctor.then(
        function(moduleObject) {
          if (0 === payload._status || -1 === payload._status)
            payload._status = 1, payload._result = moduleObject;
        },
        function(error) {
          if (0 === payload._status || -1 === payload._status)
            payload._status = 2, payload._result = error;
        }
      );
      -1 === payload._status && (payload._status = 0, payload._result = ctor);
    }
    if (1 === payload._status) return payload._result.default;
    throw payload._result;
  }
  var reportGlobalError = "function" === typeof reportError ? reportError : function(error) {
    if ("object" === typeof window && "function" === typeof window.ErrorEvent) {
      var event = new window.ErrorEvent("error", {
        bubbles: true,
        cancelable: true,
        message: "object" === typeof error && null !== error && "string" === typeof error.message ? String(error.message) : String(error),
        error
      });
      if (!window.dispatchEvent(event)) return;
    } else if ("object" === typeof process && "function" === typeof process.emit) {
      process.emit("uncaughtException", error);
      return;
    }
    console.error(error);
  };
  function noop() {
  }
  react_production.Children = {
    map: mapChildren,
    forEach: function(children, forEachFunc, forEachContext) {
      mapChildren(
        children,
        function() {
          forEachFunc.apply(this, arguments);
        },
        forEachContext
      );
    },
    count: function(children) {
      var n = 0;
      mapChildren(children, function() {
        n++;
      });
      return n;
    },
    toArray: function(children) {
      return mapChildren(children, function(child) {
        return child;
      }) || [];
    },
    only: function(children) {
      if (!isValidElement(children))
        throw Error(
          "React.Children.only expected to receive a single React element child."
        );
      return children;
    }
  };
  react_production.Component = Component;
  react_production.Fragment = REACT_FRAGMENT_TYPE;
  react_production.Profiler = REACT_PROFILER_TYPE;
  react_production.PureComponent = PureComponent;
  react_production.StrictMode = REACT_STRICT_MODE_TYPE;
  react_production.Suspense = REACT_SUSPENSE_TYPE;
  react_production.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE = ReactSharedInternals;
  react_production.act = function() {
    throw Error("act(...) is not supported in production builds of React.");
  };
  react_production.cache = function(fn) {
    return function() {
      return fn.apply(null, arguments);
    };
  };
  react_production.cloneElement = function(element, config, children) {
    if (null === element || void 0 === element)
      throw Error(
        "The argument must be a React element, but you passed " + element + "."
      );
    var props = assign({}, element.props), key = element.key, owner = void 0;
    if (null != config)
      for (propName in void 0 !== config.ref && (owner = void 0), void 0 !== config.key && (key = "" + config.key), config)
        !hasOwnProperty2.call(config, propName) || "key" === propName || "__self" === propName || "__source" === propName || "ref" === propName && void 0 === config.ref || (props[propName] = config[propName]);
    var propName = arguments.length - 2;
    if (1 === propName) props.children = children;
    else if (1 < propName) {
      for (var childArray = Array(propName), i = 0; i < propName; i++)
        childArray[i] = arguments[i + 2];
      props.children = childArray;
    }
    return ReactElement(element.type, key, void 0, void 0, owner, props);
  };
  react_production.createContext = function(defaultValue) {
    defaultValue = {
      $$typeof: REACT_CONTEXT_TYPE,
      _currentValue: defaultValue,
      _currentValue2: defaultValue,
      _threadCount: 0,
      Provider: null,
      Consumer: null
    };
    defaultValue.Provider = defaultValue;
    defaultValue.Consumer = {
      $$typeof: REACT_CONSUMER_TYPE,
      _context: defaultValue
    };
    return defaultValue;
  };
  react_production.createElement = function(type, config, children) {
    var propName, props = {}, key = null;
    if (null != config)
      for (propName in void 0 !== config.key && (key = "" + config.key), config)
        hasOwnProperty2.call(config, propName) && "key" !== propName && "__self" !== propName && "__source" !== propName && (props[propName] = config[propName]);
    var childrenLength = arguments.length - 2;
    if (1 === childrenLength) props.children = children;
    else if (1 < childrenLength) {
      for (var childArray = Array(childrenLength), i = 0; i < childrenLength; i++)
        childArray[i] = arguments[i + 2];
      props.children = childArray;
    }
    if (type && type.defaultProps)
      for (propName in childrenLength = type.defaultProps, childrenLength)
        void 0 === props[propName] && (props[propName] = childrenLength[propName]);
    return ReactElement(type, key, void 0, void 0, null, props);
  };
  react_production.createRef = function() {
    return { current: null };
  };
  react_production.forwardRef = function(render) {
    return { $$typeof: REACT_FORWARD_REF_TYPE, render };
  };
  react_production.isValidElement = isValidElement;
  react_production.lazy = function(ctor) {
    return {
      $$typeof: REACT_LAZY_TYPE,
      _payload: { _status: -1, _result: ctor },
      _init: lazyInitializer
    };
  };
  react_production.memo = function(type, compare) {
    return {
      $$typeof: REACT_MEMO_TYPE,
      type,
      compare: void 0 === compare ? null : compare
    };
  };
  react_production.startTransition = function(scope) {
    var prevTransition = ReactSharedInternals.T, currentTransition = {};
    ReactSharedInternals.T = currentTransition;
    try {
      var returnValue = scope(), onStartTransitionFinish = ReactSharedInternals.S;
      null !== onStartTransitionFinish && onStartTransitionFinish(currentTransition, returnValue);
      "object" === typeof returnValue && null !== returnValue && "function" === typeof returnValue.then && returnValue.then(noop, reportGlobalError);
    } catch (error) {
      reportGlobalError(error);
    } finally {
      ReactSharedInternals.T = prevTransition;
    }
  };
  react_production.unstable_useCacheRefresh = function() {
    return ReactSharedInternals.H.useCacheRefresh();
  };
  react_production.use = function(usable) {
    return ReactSharedInternals.H.use(usable);
  };
  react_production.useActionState = function(action, initialState, permalink) {
    return ReactSharedInternals.H.useActionState(action, initialState, permalink);
  };
  react_production.useCallback = function(callback, deps) {
    return ReactSharedInternals.H.useCallback(callback, deps);
  };
  react_production.useContext = function(Context) {
    return ReactSharedInternals.H.useContext(Context);
  };
  react_production.useDebugValue = function() {
  };
  react_production.useDeferredValue = function(value, initialValue) {
    return ReactSharedInternals.H.useDeferredValue(value, initialValue);
  };
  react_production.useEffect = function(create, deps) {
    return ReactSharedInternals.H.useEffect(create, deps);
  };
  react_production.useId = function() {
    return ReactSharedInternals.H.useId();
  };
  react_production.useImperativeHandle = function(ref2, create, deps) {
    return ReactSharedInternals.H.useImperativeHandle(ref2, create, deps);
  };
  react_production.useInsertionEffect = function(create, deps) {
    return ReactSharedInternals.H.useInsertionEffect(create, deps);
  };
  react_production.useLayoutEffect = function(create, deps) {
    return ReactSharedInternals.H.useLayoutEffect(create, deps);
  };
  react_production.useMemo = function(create, deps) {
    return ReactSharedInternals.H.useMemo(create, deps);
  };
  react_production.useOptimistic = function(passthrough, reducer) {
    return ReactSharedInternals.H.useOptimistic(passthrough, reducer);
  };
  react_production.useReducer = function(reducer, initialArg, init) {
    return ReactSharedInternals.H.useReducer(reducer, initialArg, init);
  };
  react_production.useRef = function(initialValue) {
    return ReactSharedInternals.H.useRef(initialValue);
  };
  react_production.useState = function(initialState) {
    return ReactSharedInternals.H.useState(initialState);
  };
  react_production.useSyncExternalStore = function(subscribe, getSnapshot, getServerSnapshot) {
    return ReactSharedInternals.H.useSyncExternalStore(
      subscribe,
      getSnapshot,
      getServerSnapshot
    );
  };
  react_production.useTransition = function() {
    return ReactSharedInternals.H.useTransition();
  };
  react_production.version = "19.0.0";
  return react_production;
}
var react_development = { exports: {} };
/**
 * @license React
 * react.development.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
react_development.exports;
var hasRequiredReact_development;
function requireReact_development() {
  if (hasRequiredReact_development) return react_development.exports;
  hasRequiredReact_development = 1;
  (function(module2, exports2) {
    "production" !== process.env.NODE_ENV && function() {
      function defineDeprecationWarning(methodName, info) {
        Object.defineProperty(Component.prototype, methodName, {
          get: function() {
            console.warn(
              "%s(...) is deprecated in plain JavaScript React classes. %s",
              info[0],
              info[1]
            );
          }
        });
      }
      function getIteratorFn(maybeIterable) {
        if (null === maybeIterable || "object" !== typeof maybeIterable)
          return null;
        maybeIterable = MAYBE_ITERATOR_SYMBOL && maybeIterable[MAYBE_ITERATOR_SYMBOL] || maybeIterable["@@iterator"];
        return "function" === typeof maybeIterable ? maybeIterable : null;
      }
      function warnNoop(publicInstance, callerName) {
        publicInstance = (publicInstance = publicInstance.constructor) && (publicInstance.displayName || publicInstance.name) || "ReactClass";
        var warningKey = publicInstance + "." + callerName;
        didWarnStateUpdateForUnmountedComponent[warningKey] || (console.error(
          "Can't call %s on a component that is not yet mounted. This is a no-op, but it might indicate a bug in your application. Instead, assign to `this.state` directly or define a `state = {};` class property with the desired state in the %s component.",
          callerName,
          publicInstance
        ), didWarnStateUpdateForUnmountedComponent[warningKey] = true);
      }
      function Component(props, context, updater) {
        this.props = props;
        this.context = context;
        this.refs = emptyObject;
        this.updater = updater || ReactNoopUpdateQueue;
      }
      function ComponentDummy() {
      }
      function PureComponent(props, context, updater) {
        this.props = props;
        this.context = context;
        this.refs = emptyObject;
        this.updater = updater || ReactNoopUpdateQueue;
      }
      function testStringCoercion(value) {
        return "" + value;
      }
      function checkKeyStringCoercion(value) {
        try {
          testStringCoercion(value);
          var JSCompiler_inline_result = false;
        } catch (e) {
          JSCompiler_inline_result = true;
        }
        if (JSCompiler_inline_result) {
          JSCompiler_inline_result = console;
          var JSCompiler_temp_const = JSCompiler_inline_result.error;
          var JSCompiler_inline_result$jscomp$0 = "function" === typeof Symbol && Symbol.toStringTag && value[Symbol.toStringTag] || value.constructor.name || "Object";
          JSCompiler_temp_const.call(
            JSCompiler_inline_result,
            "The provided key is an unsupported type %s. This value must be coerced to a string before using it here.",
            JSCompiler_inline_result$jscomp$0
          );
          return testStringCoercion(value);
        }
      }
      function getComponentNameFromType(type) {
        if (null == type) return null;
        if ("function" === typeof type)
          return type.$$typeof === REACT_CLIENT_REFERENCE$2 ? null : type.displayName || type.name || null;
        if ("string" === typeof type) return type;
        switch (type) {
          case REACT_FRAGMENT_TYPE:
            return "Fragment";
          case REACT_PORTAL_TYPE:
            return "Portal";
          case REACT_PROFILER_TYPE:
            return "Profiler";
          case REACT_STRICT_MODE_TYPE:
            return "StrictMode";
          case REACT_SUSPENSE_TYPE:
            return "Suspense";
          case REACT_SUSPENSE_LIST_TYPE:
            return "SuspenseList";
        }
        if ("object" === typeof type)
          switch ("number" === typeof type.tag && console.error(
            "Received an unexpected object in getComponentNameFromType(). This is likely a bug in React. Please file an issue."
          ), type.$$typeof) {
            case REACT_CONTEXT_TYPE:
              return (type.displayName || "Context") + ".Provider";
            case REACT_CONSUMER_TYPE:
              return (type._context.displayName || "Context") + ".Consumer";
            case REACT_FORWARD_REF_TYPE:
              var innerType = type.render;
              type = type.displayName;
              type || (type = innerType.displayName || innerType.name || "", type = "" !== type ? "ForwardRef(" + type + ")" : "ForwardRef");
              return type;
            case REACT_MEMO_TYPE:
              return innerType = type.displayName || null, null !== innerType ? innerType : getComponentNameFromType(type.type) || "Memo";
            case REACT_LAZY_TYPE:
              innerType = type._payload;
              type = type._init;
              try {
                return getComponentNameFromType(type(innerType));
              } catch (x) {
              }
          }
        return null;
      }
      function isValidElementType(type) {
        return "string" === typeof type || "function" === typeof type || type === REACT_FRAGMENT_TYPE || type === REACT_PROFILER_TYPE || type === REACT_STRICT_MODE_TYPE || type === REACT_SUSPENSE_TYPE || type === REACT_SUSPENSE_LIST_TYPE || type === REACT_OFFSCREEN_TYPE || "object" === typeof type && null !== type && (type.$$typeof === REACT_LAZY_TYPE || type.$$typeof === REACT_MEMO_TYPE || type.$$typeof === REACT_CONTEXT_TYPE || type.$$typeof === REACT_CONSUMER_TYPE || type.$$typeof === REACT_FORWARD_REF_TYPE || type.$$typeof === REACT_CLIENT_REFERENCE$1 || void 0 !== type.getModuleId) ? true : false;
      }
      function disabledLog() {
      }
      function disableLogs() {
        if (0 === disabledDepth) {
          prevLog = console.log;
          prevInfo = console.info;
          prevWarn = console.warn;
          prevError = console.error;
          prevGroup = console.group;
          prevGroupCollapsed = console.groupCollapsed;
          prevGroupEnd = console.groupEnd;
          var props = {
            configurable: true,
            enumerable: true,
            value: disabledLog,
            writable: true
          };
          Object.defineProperties(console, {
            info: props,
            log: props,
            warn: props,
            error: props,
            group: props,
            groupCollapsed: props,
            groupEnd: props
          });
        }
        disabledDepth++;
      }
      function reenableLogs() {
        disabledDepth--;
        if (0 === disabledDepth) {
          var props = { configurable: true, enumerable: true, writable: true };
          Object.defineProperties(console, {
            log: assign({}, props, { value: prevLog }),
            info: assign({}, props, { value: prevInfo }),
            warn: assign({}, props, { value: prevWarn }),
            error: assign({}, props, { value: prevError }),
            group: assign({}, props, { value: prevGroup }),
            groupCollapsed: assign({}, props, { value: prevGroupCollapsed }),
            groupEnd: assign({}, props, { value: prevGroupEnd })
          });
        }
        0 > disabledDepth && console.error(
          "disabledDepth fell below zero. This is a bug in React. Please file an issue."
        );
      }
      function describeBuiltInComponentFrame(name) {
        if (void 0 === prefix)
          try {
            throw Error();
          } catch (x) {
            var match = x.stack.trim().match(/\n( *(at )?)/);
            prefix = match && match[1] || "";
            suffix = -1 < x.stack.indexOf("\n    at") ? " (<anonymous>)" : -1 < x.stack.indexOf("@") ? "@unknown:0:0" : "";
          }
        return "\n" + prefix + name + suffix;
      }
      function describeNativeComponentFrame(fn, construct) {
        if (!fn || reentry) return "";
        var frame = componentFrameCache.get(fn);
        if (void 0 !== frame) return frame;
        reentry = true;
        frame = Error.prepareStackTrace;
        Error.prepareStackTrace = void 0;
        var previousDispatcher = null;
        previousDispatcher = ReactSharedInternals.H;
        ReactSharedInternals.H = null;
        disableLogs();
        try {
          var RunInRootFrame = {
            DetermineComponentFrameRoot: function() {
              try {
                if (construct) {
                  var Fake = function() {
                    throw Error();
                  };
                  Object.defineProperty(Fake.prototype, "props", {
                    set: function() {
                      throw Error();
                    }
                  });
                  if ("object" === typeof Reflect && Reflect.construct) {
                    try {
                      Reflect.construct(Fake, []);
                    } catch (x) {
                      var control = x;
                    }
                    Reflect.construct(fn, [], Fake);
                  } else {
                    try {
                      Fake.call();
                    } catch (x$0) {
                      control = x$0;
                    }
                    fn.call(Fake.prototype);
                  }
                } else {
                  try {
                    throw Error();
                  } catch (x$1) {
                    control = x$1;
                  }
                  (Fake = fn()) && "function" === typeof Fake.catch && Fake.catch(function() {
                  });
                }
              } catch (sample) {
                if (sample && control && "string" === typeof sample.stack)
                  return [sample.stack, control.stack];
              }
              return [null, null];
            }
          };
          RunInRootFrame.DetermineComponentFrameRoot.displayName = "DetermineComponentFrameRoot";
          var namePropDescriptor = Object.getOwnPropertyDescriptor(
            RunInRootFrame.DetermineComponentFrameRoot,
            "name"
          );
          namePropDescriptor && namePropDescriptor.configurable && Object.defineProperty(
            RunInRootFrame.DetermineComponentFrameRoot,
            "name",
            { value: "DetermineComponentFrameRoot" }
          );
          var _RunInRootFrame$Deter = RunInRootFrame.DetermineComponentFrameRoot(), sampleStack = _RunInRootFrame$Deter[0], controlStack = _RunInRootFrame$Deter[1];
          if (sampleStack && controlStack) {
            var sampleLines = sampleStack.split("\n"), controlLines = controlStack.split("\n");
            for (_RunInRootFrame$Deter = namePropDescriptor = 0; namePropDescriptor < sampleLines.length && !sampleLines[namePropDescriptor].includes(
              "DetermineComponentFrameRoot"
            ); )
              namePropDescriptor++;
            for (; _RunInRootFrame$Deter < controlLines.length && !controlLines[_RunInRootFrame$Deter].includes(
              "DetermineComponentFrameRoot"
            ); )
              _RunInRootFrame$Deter++;
            if (namePropDescriptor === sampleLines.length || _RunInRootFrame$Deter === controlLines.length)
              for (namePropDescriptor = sampleLines.length - 1, _RunInRootFrame$Deter = controlLines.length - 1; 1 <= namePropDescriptor && 0 <= _RunInRootFrame$Deter && sampleLines[namePropDescriptor] !== controlLines[_RunInRootFrame$Deter]; )
                _RunInRootFrame$Deter--;
            for (; 1 <= namePropDescriptor && 0 <= _RunInRootFrame$Deter; namePropDescriptor--, _RunInRootFrame$Deter--)
              if (sampleLines[namePropDescriptor] !== controlLines[_RunInRootFrame$Deter]) {
                if (1 !== namePropDescriptor || 1 !== _RunInRootFrame$Deter) {
                  do
                    if (namePropDescriptor--, _RunInRootFrame$Deter--, 0 > _RunInRootFrame$Deter || sampleLines[namePropDescriptor] !== controlLines[_RunInRootFrame$Deter]) {
                      var _frame = "\n" + sampleLines[namePropDescriptor].replace(
                        " at new ",
                        " at "
                      );
                      fn.displayName && _frame.includes("<anonymous>") && (_frame = _frame.replace("<anonymous>", fn.displayName));
                      "function" === typeof fn && componentFrameCache.set(fn, _frame);
                      return _frame;
                    }
                  while (1 <= namePropDescriptor && 0 <= _RunInRootFrame$Deter);
                }
                break;
              }
          }
        } finally {
          reentry = false, ReactSharedInternals.H = previousDispatcher, reenableLogs(), Error.prepareStackTrace = frame;
        }
        sampleLines = (sampleLines = fn ? fn.displayName || fn.name : "") ? describeBuiltInComponentFrame(sampleLines) : "";
        "function" === typeof fn && componentFrameCache.set(fn, sampleLines);
        return sampleLines;
      }
      function describeUnknownElementTypeFrameInDEV(type) {
        if (null == type) return "";
        if ("function" === typeof type) {
          var prototype = type.prototype;
          return describeNativeComponentFrame(
            type,
            !(!prototype || !prototype.isReactComponent)
          );
        }
        if ("string" === typeof type) return describeBuiltInComponentFrame(type);
        switch (type) {
          case REACT_SUSPENSE_TYPE:
            return describeBuiltInComponentFrame("Suspense");
          case REACT_SUSPENSE_LIST_TYPE:
            return describeBuiltInComponentFrame("SuspenseList");
        }
        if ("object" === typeof type)
          switch (type.$$typeof) {
            case REACT_FORWARD_REF_TYPE:
              return type = describeNativeComponentFrame(type.render, false), type;
            case REACT_MEMO_TYPE:
              return describeUnknownElementTypeFrameInDEV(type.type);
            case REACT_LAZY_TYPE:
              prototype = type._payload;
              type = type._init;
              try {
                return describeUnknownElementTypeFrameInDEV(type(prototype));
              } catch (x) {
              }
          }
        return "";
      }
      function getOwner() {
        var dispatcher = ReactSharedInternals.A;
        return null === dispatcher ? null : dispatcher.getOwner();
      }
      function hasValidKey(config) {
        if (hasOwnProperty2.call(config, "key")) {
          var getter = Object.getOwnPropertyDescriptor(config, "key").get;
          if (getter && getter.isReactWarning) return false;
        }
        return void 0 !== config.key;
      }
      function defineKeyPropWarningGetter(props, displayName) {
        function warnAboutAccessingKey() {
          specialPropKeyWarningShown || (specialPropKeyWarningShown = true, console.error(
            "%s: `key` is not a prop. Trying to access it will result in `undefined` being returned. If you need to access the same value within the child component, you should pass it as a different prop. (https://react.dev/link/special-props)",
            displayName
          ));
        }
        warnAboutAccessingKey.isReactWarning = true;
        Object.defineProperty(props, "key", {
          get: warnAboutAccessingKey,
          configurable: true
        });
      }
      function elementRefGetterWithDeprecationWarning() {
        var componentName = getComponentNameFromType(this.type);
        didWarnAboutElementRef[componentName] || (didWarnAboutElementRef[componentName] = true, console.error(
          "Accessing element.ref was removed in React 19. ref is now a regular prop. It will be removed from the JSX Element type in a future release."
        ));
        componentName = this.props.ref;
        return void 0 !== componentName ? componentName : null;
      }
      function ReactElement(type, key, self, source, owner, props) {
        self = props.ref;
        type = {
          $$typeof: REACT_ELEMENT_TYPE,
          type,
          key,
          props,
          _owner: owner
        };
        null !== (void 0 !== self ? self : null) ? Object.defineProperty(type, "ref", {
          enumerable: false,
          get: elementRefGetterWithDeprecationWarning
        }) : Object.defineProperty(type, "ref", { enumerable: false, value: null });
        type._store = {};
        Object.defineProperty(type._store, "validated", {
          configurable: false,
          enumerable: false,
          writable: true,
          value: 0
        });
        Object.defineProperty(type, "_debugInfo", {
          configurable: false,
          enumerable: false,
          writable: true,
          value: null
        });
        Object.freeze && (Object.freeze(type.props), Object.freeze(type));
        return type;
      }
      function cloneAndReplaceKey(oldElement, newKey) {
        newKey = ReactElement(
          oldElement.type,
          newKey,
          void 0,
          void 0,
          oldElement._owner,
          oldElement.props
        );
        newKey._store.validated = oldElement._store.validated;
        return newKey;
      }
      function validateChildKeys(node, parentType) {
        if ("object" === typeof node && node && node.$$typeof !== REACT_CLIENT_REFERENCE) {
          if (isArrayImpl(node))
            for (var i = 0; i < node.length; i++) {
              var child = node[i];
              isValidElement(child) && validateExplicitKey(child, parentType);
            }
          else if (isValidElement(node))
            node._store && (node._store.validated = 1);
          else if (i = getIteratorFn(node), "function" === typeof i && i !== node.entries && (i = i.call(node), i !== node))
            for (; !(node = i.next()).done; )
              isValidElement(node.value) && validateExplicitKey(node.value, parentType);
        }
      }
      function isValidElement(object) {
        return "object" === typeof object && null !== object && object.$$typeof === REACT_ELEMENT_TYPE;
      }
      function validateExplicitKey(element, parentType) {
        if (element._store && !element._store.validated && null == element.key && (element._store.validated = 1, parentType = getCurrentComponentErrorInfo(parentType), !ownerHasKeyUseWarning[parentType])) {
          ownerHasKeyUseWarning[parentType] = true;
          var childOwner = "";
          element && null != element._owner && element._owner !== getOwner() && (childOwner = null, "number" === typeof element._owner.tag ? childOwner = getComponentNameFromType(element._owner.type) : "string" === typeof element._owner.name && (childOwner = element._owner.name), childOwner = " It was passed a child from " + childOwner + ".");
          var prevGetCurrentStack = ReactSharedInternals.getCurrentStack;
          ReactSharedInternals.getCurrentStack = function() {
            var stack = describeUnknownElementTypeFrameInDEV(element.type);
            prevGetCurrentStack && (stack += prevGetCurrentStack() || "");
            return stack;
          };
          console.error(
            'Each child in a list should have a unique "key" prop.%s%s See https://react.dev/link/warning-keys for more information.',
            parentType,
            childOwner
          );
          ReactSharedInternals.getCurrentStack = prevGetCurrentStack;
        }
      }
      function getCurrentComponentErrorInfo(parentType) {
        var info = "", owner = getOwner();
        owner && (owner = getComponentNameFromType(owner.type)) && (info = "\n\nCheck the render method of `" + owner + "`.");
        info || (parentType = getComponentNameFromType(parentType)) && (info = "\n\nCheck the top-level render call using <" + parentType + ">.");
        return info;
      }
      function escape(key) {
        var escaperLookup = { "=": "=0", ":": "=2" };
        return "$" + key.replace(/[=:]/g, function(match) {
          return escaperLookup[match];
        });
      }
      function getElementKey(element, index) {
        return "object" === typeof element && null !== element && null != element.key ? (checkKeyStringCoercion(element.key), escape("" + element.key)) : index.toString(36);
      }
      function noop$1() {
      }
      function resolveThenable(thenable) {
        switch (thenable.status) {
          case "fulfilled":
            return thenable.value;
          case "rejected":
            throw thenable.reason;
          default:
            switch ("string" === typeof thenable.status ? thenable.then(noop$1, noop$1) : (thenable.status = "pending", thenable.then(
              function(fulfilledValue) {
                "pending" === thenable.status && (thenable.status = "fulfilled", thenable.value = fulfilledValue);
              },
              function(error) {
                "pending" === thenable.status && (thenable.status = "rejected", thenable.reason = error);
              }
            )), thenable.status) {
              case "fulfilled":
                return thenable.value;
              case "rejected":
                throw thenable.reason;
            }
        }
        throw thenable;
      }
      function mapIntoArray(children, array, escapedPrefix, nameSoFar, callback) {
        var type = typeof children;
        if ("undefined" === type || "boolean" === type) children = null;
        var invokeCallback = false;
        if (null === children) invokeCallback = true;
        else
          switch (type) {
            case "bigint":
            case "string":
            case "number":
              invokeCallback = true;
              break;
            case "object":
              switch (children.$$typeof) {
                case REACT_ELEMENT_TYPE:
                case REACT_PORTAL_TYPE:
                  invokeCallback = true;
                  break;
                case REACT_LAZY_TYPE:
                  return invokeCallback = children._init, mapIntoArray(
                    invokeCallback(children._payload),
                    array,
                    escapedPrefix,
                    nameSoFar,
                    callback
                  );
              }
          }
        if (invokeCallback) {
          invokeCallback = children;
          callback = callback(invokeCallback);
          var childKey = "" === nameSoFar ? "." + getElementKey(invokeCallback, 0) : nameSoFar;
          isArrayImpl(callback) ? (escapedPrefix = "", null != childKey && (escapedPrefix = childKey.replace(userProvidedKeyEscapeRegex, "$&/") + "/"), mapIntoArray(callback, array, escapedPrefix, "", function(c) {
            return c;
          })) : null != callback && (isValidElement(callback) && (null != callback.key && (invokeCallback && invokeCallback.key === callback.key || checkKeyStringCoercion(callback.key)), escapedPrefix = cloneAndReplaceKey(
            callback,
            escapedPrefix + (null == callback.key || invokeCallback && invokeCallback.key === callback.key ? "" : ("" + callback.key).replace(
              userProvidedKeyEscapeRegex,
              "$&/"
            ) + "/") + childKey
          ), "" !== nameSoFar && null != invokeCallback && isValidElement(invokeCallback) && null == invokeCallback.key && invokeCallback._store && !invokeCallback._store.validated && (escapedPrefix._store.validated = 2), callback = escapedPrefix), array.push(callback));
          return 1;
        }
        invokeCallback = 0;
        childKey = "" === nameSoFar ? "." : nameSoFar + ":";
        if (isArrayImpl(children))
          for (var i = 0; i < children.length; i++)
            nameSoFar = children[i], type = childKey + getElementKey(nameSoFar, i), invokeCallback += mapIntoArray(
              nameSoFar,
              array,
              escapedPrefix,
              type,
              callback
            );
        else if (i = getIteratorFn(children), "function" === typeof i)
          for (i === children.entries && (didWarnAboutMaps || console.warn(
            "Using Maps as children is not supported. Use an array of keyed ReactElements instead."
          ), didWarnAboutMaps = true), children = i.call(children), i = 0; !(nameSoFar = children.next()).done; )
            nameSoFar = nameSoFar.value, type = childKey + getElementKey(nameSoFar, i++), invokeCallback += mapIntoArray(
              nameSoFar,
              array,
              escapedPrefix,
              type,
              callback
            );
        else if ("object" === type) {
          if ("function" === typeof children.then)
            return mapIntoArray(
              resolveThenable(children),
              array,
              escapedPrefix,
              nameSoFar,
              callback
            );
          array = String(children);
          throw Error(
            "Objects are not valid as a React child (found: " + ("[object Object]" === array ? "object with keys {" + Object.keys(children).join(", ") + "}" : array) + "). If you meant to render a collection of children, use an array instead."
          );
        }
        return invokeCallback;
      }
      function mapChildren(children, func, context) {
        if (null == children) return children;
        var result = [], count = 0;
        mapIntoArray(children, result, "", "", function(child) {
          return func.call(context, child, count++);
        });
        return result;
      }
      function lazyInitializer(payload) {
        if (-1 === payload._status) {
          var ctor = payload._result;
          ctor = ctor();
          ctor.then(
            function(moduleObject) {
              if (0 === payload._status || -1 === payload._status)
                payload._status = 1, payload._result = moduleObject;
            },
            function(error) {
              if (0 === payload._status || -1 === payload._status)
                payload._status = 2, payload._result = error;
            }
          );
          -1 === payload._status && (payload._status = 0, payload._result = ctor);
        }
        if (1 === payload._status)
          return ctor = payload._result, void 0 === ctor && console.error(
            "lazy: Expected the result of a dynamic import() call. Instead received: %s\n\nYour code should look like: \n  const MyComponent = lazy(() => import('./MyComponent'))\n\nDid you accidentally put curly braces around the import?",
            ctor
          ), "default" in ctor || console.error(
            "lazy: Expected the result of a dynamic import() call. Instead received: %s\n\nYour code should look like: \n  const MyComponent = lazy(() => import('./MyComponent'))",
            ctor
          ), ctor.default;
        throw payload._result;
      }
      function resolveDispatcher() {
        var dispatcher = ReactSharedInternals.H;
        null === dispatcher && console.error(
          "Invalid hook call. Hooks can only be called inside of the body of a function component. This could happen for one of the following reasons:\n1. You might have mismatching versions of React and the renderer (such as React DOM)\n2. You might be breaking the Rules of Hooks\n3. You might have more than one copy of React in the same app\nSee https://react.dev/link/invalid-hook-call for tips about how to debug and fix this problem."
        );
        return dispatcher;
      }
      function noop() {
      }
      function enqueueTask(task) {
        if (null === enqueueTaskImpl)
          try {
            var requireString = ("require" + Math.random()).slice(0, 7);
            enqueueTaskImpl = (module2 && module2[requireString]).call(
              module2,
              "timers"
            ).setImmediate;
          } catch (_err) {
            enqueueTaskImpl = function(callback) {
              false === didWarnAboutMessageChannel && (didWarnAboutMessageChannel = true, "undefined" === typeof MessageChannel && console.error(
                "This browser does not have a MessageChannel implementation, so enqueuing tasks via await act(async () => ...) will fail. Please file an issue at https://github.com/facebook/react/issues if you encounter this warning."
              ));
              var channel = new MessageChannel();
              channel.port1.onmessage = callback;
              channel.port2.postMessage(void 0);
            };
          }
        return enqueueTaskImpl(task);
      }
      function aggregateErrors(errors) {
        return 1 < errors.length && "function" === typeof AggregateError ? new AggregateError(errors) : errors[0];
      }
      function popActScope(prevActQueue, prevActScopeDepth) {
        prevActScopeDepth !== actScopeDepth - 1 && console.error(
          "You seem to have overlapping act() calls, this is not supported. Be sure to await previous act() calls before making a new one. "
        );
        actScopeDepth = prevActScopeDepth;
      }
      function recursivelyFlushAsyncActWork(returnValue, resolve, reject) {
        var queue = ReactSharedInternals.actQueue;
        if (null !== queue)
          if (0 !== queue.length)
            try {
              flushActQueue(queue);
              enqueueTask(function() {
                return recursivelyFlushAsyncActWork(returnValue, resolve, reject);
              });
              return;
            } catch (error) {
              ReactSharedInternals.thrownErrors.push(error);
            }
          else ReactSharedInternals.actQueue = null;
        0 < ReactSharedInternals.thrownErrors.length ? (queue = aggregateErrors(ReactSharedInternals.thrownErrors), ReactSharedInternals.thrownErrors.length = 0, reject(queue)) : resolve(returnValue);
      }
      function flushActQueue(queue) {
        if (!isFlushing) {
          isFlushing = true;
          var i = 0;
          try {
            for (; i < queue.length; i++) {
              var callback = queue[i];
              do {
                ReactSharedInternals.didUsePromise = false;
                var continuation = callback(false);
                if (null !== continuation) {
                  if (ReactSharedInternals.didUsePromise) {
                    queue[i] = callback;
                    queue.splice(0, i);
                    return;
                  }
                  callback = continuation;
                } else break;
              } while (1);
            }
            queue.length = 0;
          } catch (error) {
            queue.splice(0, i + 1), ReactSharedInternals.thrownErrors.push(error);
          } finally {
            isFlushing = false;
          }
        }
      }
      "undefined" !== typeof __REACT_DEVTOOLS_GLOBAL_HOOK__ && "function" === typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStart && __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStart(Error());
      var REACT_ELEMENT_TYPE = Symbol.for("react.transitional.element"), REACT_PORTAL_TYPE = Symbol.for("react.portal"), REACT_FRAGMENT_TYPE = Symbol.for("react.fragment"), REACT_STRICT_MODE_TYPE = Symbol.for("react.strict_mode"), REACT_PROFILER_TYPE = Symbol.for("react.profiler");
      var REACT_CONSUMER_TYPE = Symbol.for("react.consumer"), REACT_CONTEXT_TYPE = Symbol.for("react.context"), REACT_FORWARD_REF_TYPE = Symbol.for("react.forward_ref"), REACT_SUSPENSE_TYPE = Symbol.for("react.suspense"), REACT_SUSPENSE_LIST_TYPE = Symbol.for("react.suspense_list"), REACT_MEMO_TYPE = Symbol.for("react.memo"), REACT_LAZY_TYPE = Symbol.for("react.lazy"), REACT_OFFSCREEN_TYPE = Symbol.for("react.offscreen"), MAYBE_ITERATOR_SYMBOL = Symbol.iterator, didWarnStateUpdateForUnmountedComponent = {}, ReactNoopUpdateQueue = {
        isMounted: function() {
          return false;
        },
        enqueueForceUpdate: function(publicInstance) {
          warnNoop(publicInstance, "forceUpdate");
        },
        enqueueReplaceState: function(publicInstance) {
          warnNoop(publicInstance, "replaceState");
        },
        enqueueSetState: function(publicInstance) {
          warnNoop(publicInstance, "setState");
        }
      }, assign = Object.assign, emptyObject = {};
      Object.freeze(emptyObject);
      Component.prototype.isReactComponent = {};
      Component.prototype.setState = function(partialState, callback) {
        if ("object" !== typeof partialState && "function" !== typeof partialState && null != partialState)
          throw Error(
            "takes an object of state variables to update or a function which returns an object of state variables."
          );
        this.updater.enqueueSetState(this, partialState, callback, "setState");
      };
      Component.prototype.forceUpdate = function(callback) {
        this.updater.enqueueForceUpdate(this, callback, "forceUpdate");
      };
      var deprecatedAPIs = {
        isMounted: [
          "isMounted",
          "Instead, make sure to clean up subscriptions and pending requests in componentWillUnmount to prevent memory leaks."
        ],
        replaceState: [
          "replaceState",
          "Refactor your code to use setState instead (see https://github.com/facebook/react/issues/3236)."
        ]
      }, fnName;
      for (fnName in deprecatedAPIs)
        deprecatedAPIs.hasOwnProperty(fnName) && defineDeprecationWarning(fnName, deprecatedAPIs[fnName]);
      ComponentDummy.prototype = Component.prototype;
      deprecatedAPIs = PureComponent.prototype = new ComponentDummy();
      deprecatedAPIs.constructor = PureComponent;
      assign(deprecatedAPIs, Component.prototype);
      deprecatedAPIs.isPureReactComponent = true;
      var isArrayImpl = Array.isArray, REACT_CLIENT_REFERENCE$2 = Symbol.for("react.client.reference"), ReactSharedInternals = {
        H: null,
        A: null,
        T: null,
        S: null,
        actQueue: null,
        isBatchingLegacy: false,
        didScheduleLegacyUpdate: false,
        didUsePromise: false,
        thrownErrors: [],
        getCurrentStack: null
      }, hasOwnProperty2 = Object.prototype.hasOwnProperty, REACT_CLIENT_REFERENCE$1 = Symbol.for("react.client.reference"), disabledDepth = 0, prevLog, prevInfo, prevWarn, prevError, prevGroup, prevGroupCollapsed, prevGroupEnd;
      disabledLog.__reactDisabledLog = true;
      var prefix, suffix, reentry = false;
      var componentFrameCache = new ("function" === typeof WeakMap ? WeakMap : Map)();
      var REACT_CLIENT_REFERENCE = Symbol.for("react.client.reference"), specialPropKeyWarningShown, didWarnAboutOldJSXRuntime;
      var didWarnAboutElementRef = {};
      var ownerHasKeyUseWarning = {}, didWarnAboutMaps = false, userProvidedKeyEscapeRegex = /\/+/g, reportGlobalError = "function" === typeof reportError ? reportError : function(error) {
        if ("object" === typeof window && "function" === typeof window.ErrorEvent) {
          var event = new window.ErrorEvent("error", {
            bubbles: true,
            cancelable: true,
            message: "object" === typeof error && null !== error && "string" === typeof error.message ? String(error.message) : String(error),
            error
          });
          if (!window.dispatchEvent(event)) return;
        } else if ("object" === typeof process && "function" === typeof process.emit) {
          process.emit("uncaughtException", error);
          return;
        }
        console.error(error);
      }, didWarnAboutMessageChannel = false, enqueueTaskImpl = null, actScopeDepth = 0, didWarnNoAwaitAct = false, isFlushing = false, queueSeveralMicrotasks = "function" === typeof queueMicrotask ? function(callback) {
        queueMicrotask(function() {
          return queueMicrotask(callback);
        });
      } : enqueueTask;
      exports2.Children = {
        map: mapChildren,
        forEach: function(children, forEachFunc, forEachContext) {
          mapChildren(
            children,
            function() {
              forEachFunc.apply(this, arguments);
            },
            forEachContext
          );
        },
        count: function(children) {
          var n = 0;
          mapChildren(children, function() {
            n++;
          });
          return n;
        },
        toArray: function(children) {
          return mapChildren(children, function(child) {
            return child;
          }) || [];
        },
        only: function(children) {
          if (!isValidElement(children))
            throw Error(
              "React.Children.only expected to receive a single React element child."
            );
          return children;
        }
      };
      exports2.Component = Component;
      exports2.Fragment = REACT_FRAGMENT_TYPE;
      exports2.Profiler = REACT_PROFILER_TYPE;
      exports2.PureComponent = PureComponent;
      exports2.StrictMode = REACT_STRICT_MODE_TYPE;
      exports2.Suspense = REACT_SUSPENSE_TYPE;
      exports2.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE = ReactSharedInternals;
      exports2.act = function(callback) {
        var prevActQueue = ReactSharedInternals.actQueue, prevActScopeDepth = actScopeDepth;
        actScopeDepth++;
        var queue = ReactSharedInternals.actQueue = null !== prevActQueue ? prevActQueue : [], didAwaitActCall = false;
        try {
          var result = callback();
        } catch (error) {
          ReactSharedInternals.thrownErrors.push(error);
        }
        if (0 < ReactSharedInternals.thrownErrors.length)
          throw popActScope(prevActQueue, prevActScopeDepth), callback = aggregateErrors(ReactSharedInternals.thrownErrors), ReactSharedInternals.thrownErrors.length = 0, callback;
        if (null !== result && "object" === typeof result && "function" === typeof result.then) {
          var thenable = result;
          queueSeveralMicrotasks(function() {
            didAwaitActCall || didWarnNoAwaitAct || (didWarnNoAwaitAct = true, console.error(
              "You called act(async () => ...) without await. This could lead to unexpected testing behaviour, interleaving multiple act calls and mixing their scopes. You should - await act(async () => ...);"
            ));
          });
          return {
            then: function(resolve, reject) {
              didAwaitActCall = true;
              thenable.then(
                function(returnValue) {
                  popActScope(prevActQueue, prevActScopeDepth);
                  if (0 === prevActScopeDepth) {
                    try {
                      flushActQueue(queue), enqueueTask(function() {
                        return recursivelyFlushAsyncActWork(
                          returnValue,
                          resolve,
                          reject
                        );
                      });
                    } catch (error$2) {
                      ReactSharedInternals.thrownErrors.push(error$2);
                    }
                    if (0 < ReactSharedInternals.thrownErrors.length) {
                      var _thrownError = aggregateErrors(
                        ReactSharedInternals.thrownErrors
                      );
                      ReactSharedInternals.thrownErrors.length = 0;
                      reject(_thrownError);
                    }
                  } else resolve(returnValue);
                },
                function(error) {
                  popActScope(prevActQueue, prevActScopeDepth);
                  0 < ReactSharedInternals.thrownErrors.length ? (error = aggregateErrors(
                    ReactSharedInternals.thrownErrors
                  ), ReactSharedInternals.thrownErrors.length = 0, reject(error)) : reject(error);
                }
              );
            }
          };
        }
        var returnValue$jscomp$0 = result;
        popActScope(prevActQueue, prevActScopeDepth);
        0 === prevActScopeDepth && (flushActQueue(queue), 0 !== queue.length && queueSeveralMicrotasks(function() {
          didAwaitActCall || didWarnNoAwaitAct || (didWarnNoAwaitAct = true, console.error(
            "A component suspended inside an `act` scope, but the `act` call was not awaited. When testing React components that depend on asynchronous data, you must await the result:\n\nawait act(() => ...)"
          ));
        }), ReactSharedInternals.actQueue = null);
        if (0 < ReactSharedInternals.thrownErrors.length)
          throw callback = aggregateErrors(ReactSharedInternals.thrownErrors), ReactSharedInternals.thrownErrors.length = 0, callback;
        return {
          then: function(resolve, reject) {
            didAwaitActCall = true;
            0 === prevActScopeDepth ? (ReactSharedInternals.actQueue = queue, enqueueTask(function() {
              return recursivelyFlushAsyncActWork(
                returnValue$jscomp$0,
                resolve,
                reject
              );
            })) : resolve(returnValue$jscomp$0);
          }
        };
      };
      exports2.cache = function(fn) {
        return function() {
          return fn.apply(null, arguments);
        };
      };
      exports2.cloneElement = function(element, config, children) {
        if (null === element || void 0 === element)
          throw Error(
            "The argument must be a React element, but you passed " + element + "."
          );
        var props = assign({}, element.props), key = element.key, owner = element._owner;
        if (null != config) {
          var JSCompiler_inline_result;
          a: {
            if (hasOwnProperty2.call(config, "ref") && (JSCompiler_inline_result = Object.getOwnPropertyDescriptor(
              config,
              "ref"
            ).get) && JSCompiler_inline_result.isReactWarning) {
              JSCompiler_inline_result = false;
              break a;
            }
            JSCompiler_inline_result = void 0 !== config.ref;
          }
          JSCompiler_inline_result && (owner = getOwner());
          hasValidKey(config) && (checkKeyStringCoercion(config.key), key = "" + config.key);
          for (propName in config)
            !hasOwnProperty2.call(config, propName) || "key" === propName || "__self" === propName || "__source" === propName || "ref" === propName && void 0 === config.ref || (props[propName] = config[propName]);
        }
        var propName = arguments.length - 2;
        if (1 === propName) props.children = children;
        else if (1 < propName) {
          JSCompiler_inline_result = Array(propName);
          for (var i = 0; i < propName; i++)
            JSCompiler_inline_result[i] = arguments[i + 2];
          props.children = JSCompiler_inline_result;
        }
        props = ReactElement(element.type, key, void 0, void 0, owner, props);
        for (key = 2; key < arguments.length; key++)
          validateChildKeys(arguments[key], props.type);
        return props;
      };
      exports2.createContext = function(defaultValue) {
        defaultValue = {
          $$typeof: REACT_CONTEXT_TYPE,
          _currentValue: defaultValue,
          _currentValue2: defaultValue,
          _threadCount: 0,
          Provider: null,
          Consumer: null
        };
        defaultValue.Provider = defaultValue;
        defaultValue.Consumer = {
          $$typeof: REACT_CONSUMER_TYPE,
          _context: defaultValue
        };
        defaultValue._currentRenderer = null;
        defaultValue._currentRenderer2 = null;
        return defaultValue;
      };
      exports2.createElement = function(type, config, children) {
        if (isValidElementType(type))
          for (var i = 2; i < arguments.length; i++)
            validateChildKeys(arguments[i], type);
        else {
          i = "";
          if (void 0 === type || "object" === typeof type && null !== type && 0 === Object.keys(type).length)
            i += " You likely forgot to export your component from the file it's defined in, or you might have mixed up default and named imports.";
          if (null === type) var typeString = "null";
          else
            isArrayImpl(type) ? typeString = "array" : void 0 !== type && type.$$typeof === REACT_ELEMENT_TYPE ? (typeString = "<" + (getComponentNameFromType(type.type) || "Unknown") + " />", i = " Did you accidentally export a JSX literal instead of a component?") : typeString = typeof type;
          console.error(
            "React.createElement: type is invalid -- expected a string (for built-in components) or a class/function (for composite components) but got: %s.%s",
            typeString,
            i
          );
        }
        var propName;
        i = {};
        typeString = null;
        if (null != config)
          for (propName in didWarnAboutOldJSXRuntime || !("__self" in config) || "key" in config || (didWarnAboutOldJSXRuntime = true, console.warn(
            "Your app (or one of its dependencies) is using an outdated JSX transform. Update to the modern JSX transform for faster performance: https://react.dev/link/new-jsx-transform"
          )), hasValidKey(config) && (checkKeyStringCoercion(config.key), typeString = "" + config.key), config)
            hasOwnProperty2.call(config, propName) && "key" !== propName && "__self" !== propName && "__source" !== propName && (i[propName] = config[propName]);
        var childrenLength = arguments.length - 2;
        if (1 === childrenLength) i.children = children;
        else if (1 < childrenLength) {
          for (var childArray = Array(childrenLength), _i = 0; _i < childrenLength; _i++)
            childArray[_i] = arguments[_i + 2];
          Object.freeze && Object.freeze(childArray);
          i.children = childArray;
        }
        if (type && type.defaultProps)
          for (propName in childrenLength = type.defaultProps, childrenLength)
            void 0 === i[propName] && (i[propName] = childrenLength[propName]);
        typeString && defineKeyPropWarningGetter(
          i,
          "function" === typeof type ? type.displayName || type.name || "Unknown" : type
        );
        return ReactElement(type, typeString, void 0, void 0, getOwner(), i);
      };
      exports2.createRef = function() {
        var refObject = { current: null };
        Object.seal(refObject);
        return refObject;
      };
      exports2.forwardRef = function(render) {
        null != render && render.$$typeof === REACT_MEMO_TYPE ? console.error(
          "forwardRef requires a render function but received a `memo` component. Instead of forwardRef(memo(...)), use memo(forwardRef(...))."
        ) : "function" !== typeof render ? console.error(
          "forwardRef requires a render function but was given %s.",
          null === render ? "null" : typeof render
        ) : 0 !== render.length && 2 !== render.length && console.error(
          "forwardRef render functions accept exactly two parameters: props and ref. %s",
          1 === render.length ? "Did you forget to use the ref parameter?" : "Any additional parameter will be undefined."
        );
        null != render && null != render.defaultProps && console.error(
          "forwardRef render functions do not support defaultProps. Did you accidentally pass a React component?"
        );
        var elementType = { $$typeof: REACT_FORWARD_REF_TYPE, render }, ownName;
        Object.defineProperty(elementType, "displayName", {
          enumerable: false,
          configurable: true,
          get: function() {
            return ownName;
          },
          set: function(name) {
            ownName = name;
            render.name || render.displayName || (Object.defineProperty(render, "name", { value: name }), render.displayName = name);
          }
        });
        return elementType;
      };
      exports2.isValidElement = isValidElement;
      exports2.lazy = function(ctor) {
        return {
          $$typeof: REACT_LAZY_TYPE,
          _payload: { _status: -1, _result: ctor },
          _init: lazyInitializer
        };
      };
      exports2.memo = function(type, compare) {
        isValidElementType(type) || console.error(
          "memo: The first argument must be a component. Instead received: %s",
          null === type ? "null" : typeof type
        );
        compare = {
          $$typeof: REACT_MEMO_TYPE,
          type,
          compare: void 0 === compare ? null : compare
        };
        var ownName;
        Object.defineProperty(compare, "displayName", {
          enumerable: false,
          configurable: true,
          get: function() {
            return ownName;
          },
          set: function(name) {
            ownName = name;
            type.name || type.displayName || (Object.defineProperty(type, "name", { value: name }), type.displayName = name);
          }
        });
        return compare;
      };
      exports2.startTransition = function(scope) {
        var prevTransition = ReactSharedInternals.T, currentTransition = {};
        ReactSharedInternals.T = currentTransition;
        currentTransition._updatedFibers = /* @__PURE__ */ new Set();
        try {
          var returnValue = scope(), onStartTransitionFinish = ReactSharedInternals.S;
          null !== onStartTransitionFinish && onStartTransitionFinish(currentTransition, returnValue);
          "object" === typeof returnValue && null !== returnValue && "function" === typeof returnValue.then && returnValue.then(noop, reportGlobalError);
        } catch (error) {
          reportGlobalError(error);
        } finally {
          null === prevTransition && currentTransition._updatedFibers && (scope = currentTransition._updatedFibers.size, currentTransition._updatedFibers.clear(), 10 < scope && console.warn(
            "Detected a large number of updates inside startTransition. If this is due to a subscription please re-write it to use React provided hooks. Otherwise concurrent mode guarantees are off the table."
          )), ReactSharedInternals.T = prevTransition;
        }
      };
      exports2.unstable_useCacheRefresh = function() {
        return resolveDispatcher().useCacheRefresh();
      };
      exports2.use = function(usable) {
        return resolveDispatcher().use(usable);
      };
      exports2.useActionState = function(action, initialState, permalink) {
        return resolveDispatcher().useActionState(
          action,
          initialState,
          permalink
        );
      };
      exports2.useCallback = function(callback, deps) {
        return resolveDispatcher().useCallback(callback, deps);
      };
      exports2.useContext = function(Context) {
        var dispatcher = resolveDispatcher();
        Context.$$typeof === REACT_CONSUMER_TYPE && console.error(
          "Calling useContext(Context.Consumer) is not supported and will cause bugs. Did you mean to call useContext(Context) instead?"
        );
        return dispatcher.useContext(Context);
      };
      exports2.useDebugValue = function(value, formatterFn) {
        return resolveDispatcher().useDebugValue(value, formatterFn);
      };
      exports2.useDeferredValue = function(value, initialValue) {
        return resolveDispatcher().useDeferredValue(value, initialValue);
      };
      exports2.useEffect = function(create, deps) {
        return resolveDispatcher().useEffect(create, deps);
      };
      exports2.useId = function() {
        return resolveDispatcher().useId();
      };
      exports2.useImperativeHandle = function(ref2, create, deps) {
        return resolveDispatcher().useImperativeHandle(ref2, create, deps);
      };
      exports2.useInsertionEffect = function(create, deps) {
        return resolveDispatcher().useInsertionEffect(create, deps);
      };
      exports2.useLayoutEffect = function(create, deps) {
        return resolveDispatcher().useLayoutEffect(create, deps);
      };
      exports2.useMemo = function(create, deps) {
        return resolveDispatcher().useMemo(create, deps);
      };
      exports2.useOptimistic = function(passthrough, reducer) {
        return resolveDispatcher().useOptimistic(passthrough, reducer);
      };
      exports2.useReducer = function(reducer, initialArg, init) {
        return resolveDispatcher().useReducer(reducer, initialArg, init);
      };
      exports2.useRef = function(initialValue) {
        return resolveDispatcher().useRef(initialValue);
      };
      exports2.useState = function(initialState) {
        return resolveDispatcher().useState(initialState);
      };
      exports2.useSyncExternalStore = function(subscribe, getSnapshot, getServerSnapshot) {
        return resolveDispatcher().useSyncExternalStore(
          subscribe,
          getSnapshot,
          getServerSnapshot
        );
      };
      exports2.useTransition = function() {
        return resolveDispatcher().useTransition();
      };
      exports2.version = "19.0.0";
      "undefined" !== typeof __REACT_DEVTOOLS_GLOBAL_HOOK__ && "function" === typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStop && __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStop(Error());
    }();
  })(react_development, react_development.exports);
  return react_development.exports;
}
if (process.env.NODE_ENV === "production") {
  react.exports = requireReact_production();
} else {
  react.exports = requireReact_development();
}
var reactExports = react.exports;
var shim = { exports: {} };
var useSyncExternalStoreShim_production = {};
/**
 * @license React
 * use-sync-external-store-shim.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
var hasRequiredUseSyncExternalStoreShim_production;
function requireUseSyncExternalStoreShim_production() {
  if (hasRequiredUseSyncExternalStoreShim_production) return useSyncExternalStoreShim_production;
  hasRequiredUseSyncExternalStoreShim_production = 1;
  var React = reactExports;
  function is(x, y) {
    return x === y && (0 !== x || 1 / x === 1 / y) || x !== x && y !== y;
  }
  var objectIs = "function" === typeof Object.is ? Object.is : is, useState = React.useState, useEffect = React.useEffect, useLayoutEffect = React.useLayoutEffect, useDebugValue = React.useDebugValue;
  function useSyncExternalStore$2(subscribe, getSnapshot) {
    var value = getSnapshot(), _useState = useState({ inst: { value, getSnapshot } }), inst = _useState[0].inst, forceUpdate = _useState[1];
    useLayoutEffect(
      function() {
        inst.value = value;
        inst.getSnapshot = getSnapshot;
        checkIfSnapshotChanged(inst) && forceUpdate({ inst });
      },
      [subscribe, value, getSnapshot]
    );
    useEffect(
      function() {
        checkIfSnapshotChanged(inst) && forceUpdate({ inst });
        return subscribe(function() {
          checkIfSnapshotChanged(inst) && forceUpdate({ inst });
        });
      },
      [subscribe]
    );
    useDebugValue(value);
    return value;
  }
  function checkIfSnapshotChanged(inst) {
    var latestGetSnapshot = inst.getSnapshot;
    inst = inst.value;
    try {
      var nextValue = latestGetSnapshot();
      return !objectIs(inst, nextValue);
    } catch (error) {
      return true;
    }
  }
  function useSyncExternalStore$1(subscribe, getSnapshot) {
    return getSnapshot();
  }
  var shim2 = "undefined" === typeof window || "undefined" === typeof window.document || "undefined" === typeof window.document.createElement ? useSyncExternalStore$1 : useSyncExternalStore$2;
  useSyncExternalStoreShim_production.useSyncExternalStore = void 0 !== React.useSyncExternalStore ? React.useSyncExternalStore : shim2;
  return useSyncExternalStoreShim_production;
}
var useSyncExternalStoreShim_development = {};
/**
 * @license React
 * use-sync-external-store-shim.development.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
var hasRequiredUseSyncExternalStoreShim_development;
function requireUseSyncExternalStoreShim_development() {
  if (hasRequiredUseSyncExternalStoreShim_development) return useSyncExternalStoreShim_development;
  hasRequiredUseSyncExternalStoreShim_development = 1;
  "production" !== process.env.NODE_ENV && function() {
    function is(x, y) {
      return x === y && (0 !== x || 1 / x === 1 / y) || x !== x && y !== y;
    }
    function useSyncExternalStore$2(subscribe, getSnapshot) {
      didWarnOld18Alpha || void 0 === React.startTransition || (didWarnOld18Alpha = true, console.error(
        "You are using an outdated, pre-release alpha of React 18 that does not support useSyncExternalStore. The use-sync-external-store shim will not work correctly. Upgrade to a newer pre-release."
      ));
      var value = getSnapshot();
      if (!didWarnUncachedGetSnapshot) {
        var cachedValue = getSnapshot();
        objectIs(value, cachedValue) || (console.error(
          "The result of getSnapshot should be cached to avoid an infinite loop"
        ), didWarnUncachedGetSnapshot = true);
      }
      cachedValue = useState({
        inst: { value, getSnapshot }
      });
      var inst = cachedValue[0].inst, forceUpdate = cachedValue[1];
      useLayoutEffect(
        function() {
          inst.value = value;
          inst.getSnapshot = getSnapshot;
          checkIfSnapshotChanged(inst) && forceUpdate({ inst });
        },
        [subscribe, value, getSnapshot]
      );
      useEffect(
        function() {
          checkIfSnapshotChanged(inst) && forceUpdate({ inst });
          return subscribe(function() {
            checkIfSnapshotChanged(inst) && forceUpdate({ inst });
          });
        },
        [subscribe]
      );
      useDebugValue(value);
      return value;
    }
    function checkIfSnapshotChanged(inst) {
      var latestGetSnapshot = inst.getSnapshot;
      inst = inst.value;
      try {
        var nextValue = latestGetSnapshot();
        return !objectIs(inst, nextValue);
      } catch (error) {
        return true;
      }
    }
    function useSyncExternalStore$1(subscribe, getSnapshot) {
      return getSnapshot();
    }
    "undefined" !== typeof __REACT_DEVTOOLS_GLOBAL_HOOK__ && "function" === typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStart && __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStart(Error());
    var React = reactExports, objectIs = "function" === typeof Object.is ? Object.is : is, useState = React.useState, useEffect = React.useEffect, useLayoutEffect = React.useLayoutEffect, useDebugValue = React.useDebugValue, didWarnOld18Alpha = false, didWarnUncachedGetSnapshot = false, shim2 = "undefined" === typeof window || "undefined" === typeof window.document || "undefined" === typeof window.document.createElement ? useSyncExternalStore$1 : useSyncExternalStore$2;
    useSyncExternalStoreShim_development.useSyncExternalStore = void 0 !== React.useSyncExternalStore ? React.useSyncExternalStore : shim2;
    "undefined" !== typeof __REACT_DEVTOOLS_GLOBAL_HOOK__ && "function" === typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStop && __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStop(Error());
  }();
  return useSyncExternalStoreShim_development;
}
if (process.env.NODE_ENV === "production") {
  shim.exports = requireUseSyncExternalStoreShim_production();
} else {
  shim.exports = requireUseSyncExternalStoreShim_development();
}
var shimExports = shim.exports;
const isServer = typeof window === "undefined";
var InternalNameSpace;
(function(InternalNameSpace2) {
  InternalNameSpace2["$$__ignore__$$"] = "$$__ignore__$$";
  InternalNameSpace2["$$__persist__$$"] = "$$__persist__$$";
  InternalNameSpace2["$$__subscribe__$$"] = "$$__subscribe__$$";
  InternalNameSpace2["$$__redux_dev_tool__$$"] = "$$__redux_dev_tool__$$";
})(InternalNameSpace || (InternalNameSpace = {}));
const jobs = /* @__PURE__ */ new Set();
const MAX_UPDATE = 20;
let process$1 = false;
let updateCount = 0;
const flushQueue = () => {
  const all = [...jobs.values()].slice(0);
  jobs.clear();
  for (const job of all) {
    job.notify();
  }
  process$1 = false;
  if (jobs.size) {
    updateCount++;
    if (updateCount > MAX_UPDATE) {
      throw new Error(`[reactivity-store] have a infinity update for current store, pendingJobs: ${new Set(jobs)}`);
    }
    flushQueue();
  }
};
const queueJob = (job) => {
  jobs.add(job);
  if (process$1)
    return;
  process$1 = true;
  updateCount = 0;
  Promise.resolve().then(flushQueue);
};
class ControllerEffect extends ReactiveEffect {
  get _isControllerEffect() {
    return true;
  }
  constructor(getter) {
    super(getter);
    if (process.env.NODE_ENV === "development") {
      this._devVersion = "3.5.16";
    }
  }
}
const catchError = (cb, instance2) => {
  return () => {
    if (!instance2._isActive)
      return;
    if (process.env.NODE_ENV === "development") {
      instance2._devRunCount = instance2._devRunCount || 0;
      instance2._devRunCount++;
    }
    try {
      const res = cb();
      if (isPromise(res)) {
        throw new Error(`[reactivity-store] selector should be a pure function, but current is a async function`);
      }
      return res;
    } catch (e) {
      if (process.env.NODE_ENV === "development") {
        console.error(`[reactivity-store] have an error for current selector, ${e === null || e === void 0 ? void 0 : e.message}, maybe you use the middleware with wrong usage, %o`, instance2);
      }
      return null;
    }
  };
};
class Controller {
  constructor(_getState, _compare, _lifeCycle, _list, _namespace, _onUpdate) {
    this._getState = _getState;
    this._compare = _compare;
    this._lifeCycle = _lifeCycle;
    this._namespace = _namespace;
    this._onUpdate = _onUpdate;
    this._listeners = /* @__PURE__ */ new Set();
    this._updateCount = 0;
    this._isActive = true;
    this.notify = () => {
      var _a;
      if (!this._isActive)
        return;
      if (process.env.NODE_ENV === "development" && isServer) {
        console.error(`[reactivity-store] unexpected update for reactivity-store, should not update a state on the server`);
      }
      this._updateCount++;
      try {
        (_a = this._onUpdate) === null || _a === void 0 ? void 0 : _a.call(this);
      } catch (e) {
        if (process.env.NODE_ENV === "development") {
          console.error(`[reactivity-store] have an error for current updater, ${e === null || e === void 0 ? void 0 : e.message}, please check your subscribe, %o`, this);
        }
        this._lifeCycle.canUpdateComponent = false;
      }
      this._listeners.forEach((f) => f());
    };
    this._scheduler = () => {
      const newState = this._effect.run();
      if (!this._isActive)
        return;
      const isSame = this._compare(this._state, newState);
      this._state = newState;
      if (!isSame) {
        if (this._lifeCycle.canUpdateComponent) {
          if (this._lifeCycle.syncUpdateComponent) {
            this.notify();
          } else {
            queueJob(this);
          }
        }
      }
    };
    this.subscribe = (listener) => {
      this._listeners.add(listener);
      return () => this._listeners.delete(listener);
    };
    this.getState = () => {
      return this._updateCount;
    };
    this.getEffect = () => {
      return this._effect;
    };
    this.getSelectorState = () => {
      return this._getStateSafe();
    };
    this.getLifeCycle = () => {
      return this._lifeCycle;
    };
    this._getStateSafe = catchError(_getState, this);
    this._effect = new ControllerEffect(this._getStateSafe);
    this._effect.scheduler = this._scheduler;
    if (this._namespace !== InternalNameSpace.$$__persist__$$ && this._namespace !== InternalNameSpace.$$__subscribe__$$ && this._namespace !== InternalNameSpace.$$__redux_dev_tool__$$) {
      this._list = _list;
      this._list.add(this);
    }
    if (process.env.NODE_ENV === "development") {
      this._devVersion = "0.3.11";
    }
  }
  // TODO move into constructor function?
  run() {
    this._state = this._effect.run();
  }
  stop() {
    var _a, _b;
    this._effect.stop();
    this._listeners.clear();
    (_b = (_a = this._list) === null || _a === void 0 ? void 0 : _a.delete) === null || _b === void 0 ? void 0 : _b.call(_a, this);
    this._isActive = false;
  }
  setActive(d) {
    this._isActive = d;
  }
}
typeof SuppressedError === "function" ? SuppressedError : function(error, suppressed, message) {
  var e = new Error(message);
  return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
};
const createLifeCycle = () => ({
  onBeforeMount: [],
  onBeforeUpdate: [],
  onBeforeUnmount: [],
  onMounted: [],
  onUpdated: [],
  onUnmounted: [],
  hasHookInstall: false,
  canUpdateComponent: true,
  syncUpdateComponent: false
});
function _traverse(value, seen) {
  if (!isObject(value) || value[ReactiveFlags.SKIP] || reactExports.isValidElement(value)) {
    return value;
  }
  seen = seen || /* @__PURE__ */ new Set();
  if (seen.has(value)) {
    return value;
  }
  seen.add(value);
  if (isRef(value)) {
    traverse(value.value, seen);
  } else if (isArray(value)) {
    for (let i = 0; i < value.length; i++) {
      traverse(value[i], seen);
    }
  } else if (isSet(value) || isMap(value)) {
    value.forEach((v) => {
      traverse(v, seen);
    });
  } else if (isPlainObject(value)) {
    for (const key in value) {
      traverse(value[key], seen);
    }
  }
  return value;
}
function traverseShallow(value) {
  if (!isObject(value) || value[ReactiveFlags.SKIP] || reactExports.isValidElement(value)) {
    return value;
  }
  if (isRef(value)) {
    value.value;
  } else if (isArray(value)) {
    for (let i = 0; i < value.length; i++) {
      value[i];
    }
  } else if (isSet(value) || isMap(value)) {
    value.forEach((v) => {
    });
  } else if (isPlainObject(value)) {
    for (const key in value) {
      value[key];
    }
  }
  return value;
}
function traverse(value, seen) {
  if (process.env.NODE_ENV === "development") {
    const start = Date.now();
    const re = _traverse(value, seen);
    const end = Date.now();
    if (end - start > 5) {
      console.warn(`[reactivity-store] 'traverse' current data: %o take a lot of time`, re);
    }
    return re;
  } else {
    return _traverse(value, seen);
  }
}
function checkHasReactive(value) {
  let hasReactive = false;
  function traverse2(value2, seen) {
    if (!isObject(value2))
      return;
    if (hasReactive)
      return;
    if (isReactive(value2) || isRef(value2) || isProxy(value2)) {
      hasReactive = true;
      return;
    }
    seen = seen || /* @__PURE__ */ new Set();
    if (seen.has(value2)) {
      return;
    }
    seen.add(value2);
    if (isArray(value2)) {
      for (let i = 0; i < value2.length; i++) {
        traverse2(value2[i], seen);
      }
    } else if (isSet(value2) || isMap(value2)) {
      value2.forEach((v) => {
        traverse2(v, seen);
      });
    } else if (isPlainObject(value2)) {
      for (const key in value2) {
        traverse2(value2[key], seen);
      }
    }
    return;
  }
  traverse2(value);
  return hasReactive;
}
function checkHasMiddleware(value) {
  if (value && (value === null || value === void 0 ? void 0 : value["$$__state__$$"]) && (value === null || value === void 0 ? void 0 : value["$$__middleware__$$"])) {
    return true;
  }
}
const useCallbackRef = (callback) => {
  const callbackRef = reactExports.useRef(callback);
  callbackRef.current = callback;
  const memoCallback = reactExports.useCallback((...args) => {
    var _a;
    return (_a = callbackRef.current) === null || _a === void 0 ? void 0 : _a.call(null, ...args);
  }, []);
  return memoCallback;
};
const useSubscribeCallbackRef = (callback, deepSelector) => {
  const callbackRef = reactExports.useRef();
  callbackRef.current = typeof callback === "function" ? callback : null;
  const memoCallback = useCallbackRef((arg) => {
    if (callbackRef.current) {
      const re = callbackRef.current(arg);
      if (deepSelector) {
        traverse(re);
      } else {
        traverseShallow(re);
      }
      return re;
    } else {
      if (deepSelector) {
        traverse(arg);
      } else {
        traverseShallow(arg);
      }
      return arg;
    }
  });
  return memoCallback;
};
const usePrevValue = (v) => {
  const vRef = reactExports.useRef(v);
  reactExports.useEffect(() => {
    vRef.current = v;
  }, [v]);
  return vRef.current;
};
const createHook = (reactiveState, readonlyState, initialState, lifeCycle, deepSelector = true, stableSelector = false, stableCompare = true, namespace, actions = void 0) => {
  const controllerList = /* @__PURE__ */ new Set();
  if (process.env.NODE_ENV === "development" && !isServer && namespace) ;
  let active = true;
  namespace = namespace || InternalNameSpace.$$__ignore__$$;
  const generateUseHook = (type) => {
    const currentIsDeep = type === "default" ? deepSelector : type === "deep" || type === "deep-stable";
    const currentIsStable = type === "default" ? stableSelector : type === "deep-stable" || type === "shallow-stable";
    function useReactiveHookWithSelector(selector, compare) {
      const ref2 = reactExports.useRef();
      const selectorRef = useSubscribeCallbackRef(selector, currentIsDeep);
      const getSelected = useCallbackRef(() => {
        if (selector) {
          ref2.current = selector(Object.assign(Object.assign({}, readonlyState), actions));
        } else {
          ref2.current = Object.assign(Object.assign({}, readonlyState), actions);
        }
      });
      const memoCompare = useCallbackRef((p, n) => {
        if (compare && typeof compare === "function") {
          return compare(p, n);
        }
        return false;
      });
      const prevSelector = currentIsStable ? selector : usePrevValue(selector);
      const prevCompare = stableCompare ? compare : usePrevValue(compare);
      const ControllerInstance = reactExports.useMemo(() => new Controller(() => selectorRef(reactiveState), memoCompare, lifeCycle, controllerList, namespace, getSelected), []);
      shimExports.useSyncExternalStore(ControllerInstance.subscribe, ControllerInstance.getState, ControllerInstance.getState);
      reactExports.useMemo(() => {
        ControllerInstance.run();
        getSelected();
      }, [ControllerInstance, getSelected]);
      reactExports.useMemo(() => {
        if (prevSelector !== selector) {
          ControllerInstance.run();
          getSelected();
        }
      }, [ControllerInstance, prevSelector, selector]);
      reactExports.useMemo(() => {
        if (prevCompare !== compare) {
          ControllerInstance.run();
          getSelected();
        }
      }, [ControllerInstance, prevCompare, compare]);
      if (process.env.NODE_ENV === "development") {
        ControllerInstance._devSelector = selector;
        ControllerInstance._devCompare = compare;
        ControllerInstance._devActions = actions;
        ControllerInstance._devWithDeep = currentIsDeep;
        ControllerInstance._devWithStable = currentIsStable;
        ControllerInstance._devType = type;
        ControllerInstance._devState = initialState;
        ControllerInstance._devResult = ref2.current;
      }
      reactExports.useEffect(() => {
        ControllerInstance.setActive(true);
        return () => {
          if (process.env.NODE_ENV === "development") {
            ControllerInstance.setActive(false);
          } else {
            ControllerInstance.stop();
          }
        };
      }, [ControllerInstance]);
      return ref2.current;
    }
    return useReactiveHookWithSelector;
  };
  const defaultHook = generateUseHook("default");
  const deepHook = generateUseHook("deep");
  const deepStableHook = generateUseHook("deep-stable");
  const shallowHook = generateUseHook("shallow");
  const shallowStableHook = generateUseHook("shallow-stable");
  function useSelector(selector, compare) {
    return defaultHook(selector, compare);
  }
  const typedUseSelector = useSelector;
  typedUseSelector.getState = () => toRaw(initialState);
  typedUseSelector.getLifeCycle = () => lifeCycle;
  typedUseSelector.getActions = () => actions;
  typedUseSelector.getReactiveState = () => reactiveState;
  typedUseSelector.getReadonlyState = () => readonlyState;
  typedUseSelector.useDeepSelector = deepHook;
  typedUseSelector.useDeepStableSelector = deepStableHook;
  typedUseSelector.useShallowSelector = shallowHook;
  typedUseSelector.useShallowStableSelector = shallowStableHook;
  typedUseSelector.subscribe = (selector, cb, shallow) => {
    const subscribeSelector = () => {
      const re = selector(reactiveState);
      if (process.env.NODE_ENV === "development" && isPromise(re)) {
        console.error(`[reactivity-store/subscribe] selector should return a plain object, but current is a promise`);
      }
      if (shallow) {
        traverseShallow(re);
      } else {
        traverse(re);
      }
      return re;
    };
    const controller = new Controller(subscribeSelector, Object.is, lifeCycle, controllerList, InternalNameSpace.$$__subscribe__$$, () => cb());
    controller.run();
    return () => controller.stop();
  };
  typedUseSelector.getIsActive = () => active;
  typedUseSelector.clear = () => {
    controllerList.forEach((i) => i.stop());
    if (process.env.NODE_ENV === "development" && !isServer && namespace) ;
    active = false;
  };
  return typedUseSelector;
};
const getFinalState = (state) => {
  if (state["$$__state__$$"])
    return state["$$__state__$$"];
  return state;
};
const _internalCreateStore = (creator, name = "createStore", lifeCycle) => {
  const state = creator();
  if (process.env.NODE_ENV === "development" && isPromise(state)) {
    console.error(`[reactivity-store] '${name}' expect receive a reactive object but got a promise %o, this is a unexpected usage. should not return a promise in this 'creator' function`, state);
  }
  if (process.env.NODE_ENV === "development" && !isObject(state)) {
    console.error(`[reactivity-store] '${name}' expect receive a reactive object but got a ${state}, this is a unexpected usage. should return a reactive object in this 'creator' function`);
  }
  if (process.env.NODE_ENV === "development" && checkHasMiddleware(state)) {
    console.error(`[reactivity-store] '${name}' not support middleware usage, please change to use 'createState'`);
  }
  if (process.env.NODE_ENV === "development" && !checkHasReactive(state)) {
    console.error(`[reactivity-store] '${name}' expect receive a reactive object but got a plain object %o, this is a unexpected usage. should return a reactive object in this 'creator' function`, state);
  }
  const _state = getFinalState(state);
  const rawState = toRaw(_state);
  const reactiveState = reactive(_state);
  const readonlyState = readonly(_state);
  const lifeCycleInstance = createLifeCycle();
  const useSelector = createHook(reactiveState, readonlyState, rawState, lifeCycleInstance);
  return useSelector;
};
const internalCreateStore = (creator, name = "createStore", lifeCycle) => {
  const scope = effectScope();
  const useSelector = scope.run(() => _internalCreateStore(creator, name));
  useSelector.scope = scope;
  return useSelector;
};
const createStore = (creator) => {
  return internalCreateStore(creator);
};
const createDiffConfigStore = (props, diffFileId) => {
  return createStore(() => {
    var _a, _b;
    const id = ref(diffFileId);
    const setId = (_id) => id.value = _id;
    const mode = ref(props.diffViewMode);
    const setMode = (_mode) => mode.value = _mode;
    const isMounted = ref(false);
    const setIsIsMounted = (_isMounted) => isMounted.value = _isMounted;
    const enableWrap = ref(props.diffViewWrap);
    const setEnableWrap = (_enableWrap) => enableWrap.value = _enableWrap;
    const enableAddWidget = ref(props.diffViewAddWidget);
    const setEnableAddWidget = (_enableAddWidget) => enableAddWidget.value = _enableAddWidget;
    const enableHighlight = ref(props.diffViewHighlight);
    const setEnableHighlight = (_enableHighlight) => enableHighlight.value = _enableHighlight;
    const fontSize = ref(props.diffViewFontSize);
    const setFontSize = (_fontSize) => fontSize.value = _fontSize;
    const extendData = ref({
      oldFile: { ...(_a = props.extendData) == null ? void 0 : _a.oldFile },
      newFile: { ...(_b = props.extendData) == null ? void 0 : _b.newFile }
    });
    const setExtendData = (_extendData) => {
      extendData.value = { oldFile: { ..._extendData == null ? void 0 : _extendData.oldFile }, newFile: { ..._extendData == null ? void 0 : _extendData.newFile } };
    };
    const renderWidgetLine = ref(props.renderWidgetLine);
    const setRenderWidgetLine = (_renderWidgetLine) => renderWidgetLine.value = _renderWidgetLine;
    const renderExtendLine = ref(props.renderExtendLine);
    const setRenderExtendLine = (_renderExtendLine) => renderExtendLine.value = _renderExtendLine;
    const onAddWidgetClick = { current: props.onAddWidgetClick };
    const setOnAddWidgetClick = (_onAddWidgetClick) => onAddWidgetClick.current = _onAddWidgetClick.current;
    return {
      id,
      setId,
      mode,
      setMode,
      isMounted,
      setIsIsMounted,
      enableWrap,
      setEnableWrap,
      enableAddWidget,
      setEnableAddWidget,
      enableHighlight,
      setEnableHighlight,
      fontSize,
      setFontSize,
      extendData,
      setExtendData,
      renderWidgetLine,
      setRenderWidgetLine,
      renderExtendLine,
      setRenderExtendLine,
      onAddWidgetClick,
      setOnAddWidgetClick
    };
  });
};
var _tmpl$ = /* @__PURE__ */ template(`<div class=diff-style-root><div>`), _tmpl$2 = /* @__PURE__ */ template(`<div class=diff-tailwindcss-wrapper data-component=git-diff-view>`);
core._cacheMap.name = "@git-diff-view/solid";
const InternalDiffView = (props) => {
  var _a, _b, _c;
  const getInstance = () => {
    var _a2, _b2, _c2, _d, _e, _f;
    let diffFile2 = null;
    if (props.diffFile) {
      diffFile2 = core.DiffFile.createInstance({});
      diffFile2._mergeFullBundle(props.diffFile._getFullBundle());
    } else if (props.data) {
      diffFile2 = new core.DiffFile(((_a2 = props.data.oldFile) == null ? void 0 : _a2.fileName) || "", ((_b2 = props.data.oldFile) == null ? void 0 : _b2.content) || "", ((_c2 = props.data.newFile) == null ? void 0 : _c2.fileName) || "", ((_d = props.data.newFile) == null ? void 0 : _d.content) || "", props.data.hunks || [], ((_e = props.data.oldFile) == null ? void 0 : _e.fileLang) || "", ((_f = props.data.newFile) == null ? void 0 : _f.fileLang) || "");
    }
    solidJs.onCleanup(() => {
      var _a3;
      return (_a3 = diffFile2 == null ? void 0 : diffFile2.clear) == null ? void 0 : _a3.call(diffFile2);
    });
    return diffFile2;
  };
  const [wrapperRef, setWrapRef] = solidJs.createSignal(null);
  const diffFile = solidJs.createMemo(getInstance);
  const [isMounted, setIsMounted] = solidJs.createSignal(false);
  solidJs.createEffect(() => {
    setIsMounted(true);
  });
  const [widgetState, setWidgetState] = solidJs.createSignal({
    side: (_a = props.initialWidgetState) == null ? void 0 : _a.side,
    lineNumber: (_b = props.initialWidgetState) == null ? void 0 : _b.lineNumber
  });
  const reactiveHook = createDiffConfigStore(props, ((_c = diffFile()) == null ? void 0 : _c.getId()) || "");
  solidJs.createEffect(() => {
    const {
      setId,
      setEnableAddWidget,
      setEnableHighlight,
      setEnableWrap,
      setExtendData,
      setFontSize,
      setIsIsMounted,
      setMode,
      setOnAddWidgetClick,
      setRenderExtendLine,
      setRenderWidgetLine
    } = reactiveHook.getReadonlyState();
    const currentDiffFile = diffFile();
    setId((currentDiffFile == null ? void 0 : currentDiffFile.getId()) || "");
    setEnableAddWidget(!!props.diffViewAddWidget);
    setEnableHighlight(!!props.diffViewHighlight);
    setEnableWrap(!!props.diffViewWrap);
    setFontSize(props.diffViewFontSize || 14);
    setIsIsMounted(isMounted());
    setMode(props.diffViewMode || exports.DiffModeEnum.Split);
    setOnAddWidgetClick({
      current: props.onAddWidgetClick
    });
    setRenderExtendLine(props.renderExtendLine);
    setRenderWidgetLine(props.renderWidgetLine);
    setExtendData(props.extendData);
  });
  solidJs.createEffect(() => {
    if (props.initialWidgetState) {
      setWidgetState({
        side: props.initialWidgetState.side,
        lineNumber: props.initialWidgetState.lineNumber
      });
    }
  });
  const initSubscribe = () => {
    const mounted = isMounted();
    const currentDiffFile = diffFile();
    if (mounted && props.diffFile && currentDiffFile) {
      props.diffFile._addClonedInstance(currentDiffFile);
      solidJs.onCleanup(() => {
        var _a2;
        return (_a2 = props.diffFile) == null ? void 0 : _a2._delClonedInstance(currentDiffFile);
      });
    }
  };
  const initDiff = () => {
    const mounted = isMounted();
    const currentDiffFile = diffFile();
    if (mounted && currentDiffFile) {
      currentDiffFile.initTheme(props.diffViewTheme);
      currentDiffFile.initRaw();
      currentDiffFile.buildSplitDiffLines();
      currentDiffFile.buildUnifiedDiffLines();
    }
  };
  const initSyntax = () => {
    const mounted = isMounted();
    const currentDiffFile = diffFile();
    if (mounted && currentDiffFile && props.diffViewHighlight) {
      currentDiffFile.initSyntax({
        registerHighlighter: props.registerHighlighter
      });
      currentDiffFile.notifyAll();
    }
  };
  const initAttribute = () => {
    const mounted = isMounted();
    const currentDiffFile = diffFile();
    props.diffViewTheme;
    if (mounted && currentDiffFile && wrapperRef()) {
      const init = () => {
        var _a2, _b2;
        (_a2 = wrapperRef()) == null ? void 0 : _a2.setAttribute("data-theme", currentDiffFile._getTheme() || "light");
        (_b2 = wrapperRef()) == null ? void 0 : _b2.setAttribute("data-highlighter", currentDiffFile._getHighlighterName());
      };
      init();
      const cb = currentDiffFile.subscribe(init);
      solidJs.onCleanup(() => cb());
    }
  };
  solidJs.createEffect(initSubscribe);
  solidJs.createEffect(initDiff);
  solidJs.createEffect(initSyntax);
  solidJs.createEffect(initAttribute);
  solidJs.onCleanup(() => reactiveHook.clear());
  return solidJs.createComponent(solidJs.Show, {
    get when() {
      return diffFile();
    },
    get children() {
      var _el$ = _tmpl$2();
      use(setWrapRef, _el$);
      setAttribute(_el$, "data-version", "0.0.1");
      insert(_el$, solidJs.createComponent(DiffViewContext.Provider, {
        value: reactiveHook,
        get children() {
          return solidJs.createComponent(DiffWidgetContext.Provider, {
            value: [widgetState, setWidgetState],
            get children() {
              var _el$2 = _tmpl$(), _el$3 = _el$2.firstChild;
              insert(_el$3, (() => {
                var _c$ = solidJs.createMemo(() => !!(!props.diffViewMode || props.diffViewMode & exports.DiffModeEnum.Split));
                return () => _c$() ? solidJs.createComponent(DiffSplitView, {
                  get diffFile() {
                    return diffFile();
                  }
                }) : solidJs.createComponent(DiffUnifiedView, {
                  get diffFile() {
                    return diffFile();
                  }
                });
              })());
              solidJs.createRenderEffect((_p$) => {
                var _a2;
                var _v$ = {
                  [diffFontSizeName]: (props.diffViewFontSize || 14) + "px"
                }, _v$2 = isMounted() ? `diff-root${(_a2 = diffFile()) == null ? void 0 : _a2.getId()}` : void 0, _v$3 = "diff-view-wrapper" + (props.class ? ` ${props.class}` : ""), _v$4 = props.style;
                _p$.e = style(_el$2, _v$, _p$.e);
                _v$2 !== _p$.t && setAttribute(_el$3, "id", _p$.t = _v$2);
                _v$3 !== _p$.a && className(_el$3, _p$.a = _v$3);
                _p$.o = style(_el$3, _v$4, _p$.o);
                return _p$;
              }, {
                e: void 0,
                t: void 0,
                a: void 0,
                o: void 0
              });
              return _el$2;
            }
          });
        }
      }));
      solidJs.createRenderEffect((_p$) => {
        var _a2, _b2, _c2, _d;
        var _v$5 = ((_b2 = (_a2 = diffFile()) == null ? void 0 : _a2._getTheme) == null ? void 0 : _b2.call(_a2)) || "light", _v$6 = (_d = (_c2 = diffFile()) == null ? void 0 : _c2._getHighlighterName) == null ? void 0 : _d.call(_c2);
        _v$5 !== _p$.e && setAttribute(_el$, "data-theme", _p$.e = _v$5);
        _v$6 !== _p$.t && setAttribute(_el$, "data-highlighter", _p$.t = _v$6);
        return _p$;
      }, {
        e: void 0,
        t: void 0
      });
      return _el$;
    }
  });
};
function SolidDiffView(props) {
  return solidJs.createComponent(InternalDiffView, props);
}
const DiffView = SolidDiffView;
Object.defineProperty(exports, "SplitSide", {
  enumerable: true,
  get: () => core.SplitSide
});
exports.DiffView = DiffView;
Object.keys(core).forEach((k) => {
  if (k !== "default" && !Object.prototype.hasOwnProperty.call(exports, k)) Object.defineProperty(exports, k, {
    enumerable: true,
    get: () => core[k]
  });
});
//# sourceMappingURL=solid-git-diff-view.cjs.map
